@@ -1,5 +1,9 @@
 #!/usr/bin/python2
 
+import gi
+gi.require_version('NumCosmo', '1.0')
+gi.require_version('NumCosmoMath', '1.0')
+
 from math import *
 from gi.repository import GObject
 import matplotlib.pyplot as plt
@@ -45,7 +49,7 @@ mset.set (cosmo)
 #  Setting parameters Omega_c and w to be fitted and change parameter
 #  Omega_x -> Omega_k.
 #
-cosmo.de_omega_x2omega_k ()
+##cosmo.de_omega_x2omega_k ()
 cosmo.props.Omegac_fit = True
 cosmo.props.w_fit = True
 
