@@ -1,9 +1,14 @@
 #!/usr/bin/python2
 
+import gi
+gi.require_version('NumCosmo', '1.0')
+gi.require_version('NumCosmoMath', '1.0')
+
 from math import *
+from gi.repository import GObject
+import matplotlib.pyplot as plt
 from gi.repository import NumCosmo as Nc
 from gi.repository import NumCosmoMath as Ncm
-import matplotlib.pyplot as plt
 
 #
 #  Initializing the library objects, this must be called before 
@@ -61,14 +66,18 @@ recomb.prepare (cosmo)
 #  Calculating Xe, equilibrium Xe, v_tau and its derivatives.
 #
 x_a = []
+x_dtau_a = []
 Xe_a = []
 Xefi_a = []
+dtau_dlambda_a = []
+x_E_a = []
+x_Etaub_a = []
 v_tau_a = []
 dv_tau_dlambda_a = []
 d2v_tau_dlambda2_a = []
 
 for i in range (10000):
-  alpha = -log (10000.0) + (log (10000.0) - log (100.0)) / 9999.0 * i
+  alpha = -log (1.0e4) + (log (1.0e4) - log (1.0e2)) / 9999.0 * i
   Xe = recomb.Xe (cosmo, alpha)
   x = exp (-alpha)
   Xefi = recomb.equilibrium_Xe (cosmo, x)
@@ -83,11 +92,23 @@ for i in range (10000):
   dv_tau_dlambda_a.append (-dv_tau_dlambda / 10.0)
   d2v_tau_dlambda2_a.append (-d2v_tau_dlambda2 / 200.0)
 
+for i in range (10000):
+  alpha = -log (1.0e10) + (log (1.0e10) - log (1.0e2)) / 9999.0 * i
+  x = exp (-alpha)
+  dtau_dlambda = abs(recomb.dtau_dlambda (cosmo, alpha))
+  x_E = x / sqrt (cosmo.E2 (x))
+  x_E_taub = x_E / dtau_dlambda
+  
+  x_dtau_a.append (x)
+  x_E_a.append (x_E)
+  x_Etaub_a.append (x_E_taub)
+  dtau_dlambda_a.append (dtau_dlambda)
+
 #
 #  Ploting ionization history.
 #
 
-plt.title ("Ionization history")
+plt.title ("Ionization History")
 plt.xscale('log')
 plt.plot (x_a, Xe_a, 'r', label="Recombination")
 plt.plot (x_a, Xefi_a, 'b--', label="Equilibrium")
@@ -105,7 +126,7 @@ plt.clf ()
 #  Ploting visibility function and derivatives.
 #
 
-plt.title ("Visibility function and derivatives")
+plt.title ("Visibility Function and Derivatives")
 plt.xscale('log')
 plt.plot (x_a, v_tau_a, 'r', label=r'$v_\tau$')
 plt.plot (x_a, dv_tau_dlambda_a, 'b-', label=r'$\frac{1}{10}\frac{dv_\tau}{d\lambda}$')
@@ -146,3 +167,21 @@ plt.annotate (r'$z^\star=%5.2f$' % (exp(-lambdastar)-1),
 
 plt.savefig ("recomb_v_tau.png")
 
+plt.clf ()
+
+#
+#  Ploting dtau_dlambda
+#
+
+plt.title (r'dtau_dlambda')
+plt.xscale('log')
+plt.yscale('log')
+plt.plot (x_dtau_a, dtau_dlambda_a, 'r', label=r'$\vert\mathrm{d}\tau/\mathrm{d}\lambda\vert$')
+plt.plot (x_dtau_a, x_E_a, 'b', label=r'$x/E$')
+plt.plot (x_dtau_a, (x_Etaub_a), 'b', label=r'$x/(E\bar{\tau})$')
+
+plt.legend()
+plt.legend(loc=2)
+
+plt.savefig ("recomb_dtau_dlambda.png")
+
