@@ -1,5 +1,9 @@
 #!/usr/bin/python2
 
+import gi
+gi.require_version('NumCosmo', '1.0')
+gi.require_version('NumCosmoMath', '1.0')
+
 from math import *
 from gi.repository import GObject
 import matplotlib.pyplot as plt
