@@ -7,7 +7,7 @@
  ****************************************************************************/
 /*
  * numcosmo
- * Copyright (C) 2012 Sandro Dias Pinto Vitenti <sandro@isoftware.com.br>
+ * Copyright (C) 2012 Mariana Penna Lima <pennalima@gmail.com>
  * 
  * numcosmo is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -125,53 +125,23 @@ nc_data_cluster_ncount_set_property (GObject *object, guint prop_id, const GValu
       nc_data_cluster_ncount_set_mass (ncount, g_value_get_object (value));
       break;
     case PROP_LNM_TRUE:
-    {
-      GVariant *var = g_value_get_variant (value);
-      const NcmVector *v = ncm_vector_const_new_variant (var);
-      nc_data_cluster_ncount_set_lnM_true (ncount, v);
-      ncm_vector_const_free (v);
+      nc_data_cluster_ncount_set_lnM_true (ncount, g_value_get_object (value));
       break;
-    }
     case PROP_Z_TRUE:
-    {
-      GVariant *var = g_value_get_variant (value);
-      const NcmVector *v = ncm_vector_const_new_variant (var);
-      nc_data_cluster_ncount_set_z_true (ncount, v);
-      ncm_vector_const_free (v);
+      nc_data_cluster_ncount_set_z_true (ncount, g_value_get_object (value));
       break;
-    }
     case PROP_Z_OBS:
-    {
-      GVariant *var = g_value_get_variant (value);
-      const NcmMatrix *m = ncm_matrix_const_new_variant (var);
-      nc_data_cluster_ncount_set_z_obs (ncount, m);
-      ncm_matrix_const_free (m);
+      nc_data_cluster_ncount_set_z_obs (ncount, g_value_get_object (value));
       break;
-    }
     case PROP_Z_OBS_PARAMS:
-    {
-      GVariant *var = g_value_get_variant (value);
-      const NcmMatrix *m = ncm_matrix_const_new_variant (var);
-      nc_data_cluster_ncount_set_z_obs_params (ncount, m);
-      ncm_matrix_const_free (m);
+      nc_data_cluster_ncount_set_z_obs_params (ncount, g_value_get_object (value));
       break;
-    }
     case PROP_LNM_OBS:
-    {
-      GVariant *var = g_value_get_variant (value);
-      const NcmMatrix *m = ncm_matrix_const_new_variant (var);
-      nc_data_cluster_ncount_set_lnM_obs (ncount, m);
-      ncm_matrix_const_free (m);
+      nc_data_cluster_ncount_set_lnM_obs (ncount, g_value_get_object (value));
       break;
-    }
     case PROP_LNM_OBS_PARAMS:
-    {
-      GVariant *var = g_value_get_variant (value);
-      const NcmMatrix *m = ncm_matrix_const_new_variant (var);
-      nc_data_cluster_ncount_set_lnM_obs_params (ncount, m);
-      ncm_matrix_const_free (m);
+      nc_data_cluster_ncount_set_lnM_obs_params (ncount, g_value_get_object (value));
       break;
-    }
     case PROP_SURVEY_AREA:
       ncount->area_survey = g_value_get_double (value);
       break;
@@ -183,26 +153,18 @@ nc_data_cluster_ncount_set_property (GObject *object, guint prop_id, const GValu
       break;
     case PROP_Z_NODES:
     {
-      GVariant *var = g_value_get_variant (value);
-      if (var != NULL)
-      {
-        NcmVector *v = ncm_vector_new_variant (var);
-        ncm_vector_clear (&ncount->z_nodes);
-        ncount->z_nodes = v;
+      ncm_vector_clear (&ncount->z_nodes);
+      ncount->z_nodes = g_value_dup_object (value);
+      if (ncount->z_nodes != NULL)
         ncount->binned = FALSE;
-      }
       break;
     }
     case PROP_LNM_NODES:
     {
-      GVariant *var = g_value_get_variant (value);
-      if (var != NULL)
-      {
-        NcmVector *v = ncm_vector_new_variant (var);
-        ncm_vector_clear (&ncount->lnM_nodes);
-        ncount->lnM_nodes = v;
+      ncm_vector_clear (&ncount->lnM_nodes);
+      ncount->lnM_nodes = g_value_dup_object (value);
+      if (ncount->lnM_nodes != NULL)
         ncount->binned = FALSE;
-      }
       break;
     }
     case PROP_FIDUCIAL:
@@ -240,53 +202,23 @@ nc_data_cluster_ncount_get_property (GObject *object, guint prop_id, GValue *val
       g_value_set_object (value, ncount->m);
       break;
     case PROP_LNM_TRUE:
-    {
-      if (ncount->lnM_true != NULL)
-      {
-        GVariant *var = ncm_vector_peek_variant (ncount->lnM_true); 
-        g_value_take_variant (value, var);
-      }
+      g_value_set_object (value, ncount->lnM_true);
       break;
-    }
     case PROP_Z_TRUE:
-    {
-      if (ncount->z_true != NULL)
-      {
-        GVariant *var = ncm_vector_peek_variant (ncount->z_true); 
-        g_value_take_variant (value, var);
-      }
+      g_value_set_object (value, ncount->z_true);
       break;
-    }
     case PROP_Z_OBS:
-    {
-      GVariant *var = ncm_matrix_peek_variant (ncount->z_obs); 
-      g_value_take_variant (value, var);
+      g_value_set_object (value, ncount->z_obs);
       break;
-    }
     case PROP_Z_OBS_PARAMS:
-    {
-      if (ncount->z_obs_params != NULL)
-      {
-        GVariant *var = ncm_matrix_peek_variant (ncount->z_obs_params); 
-        g_value_take_variant (value, var);
-      }
+      g_value_set_object (value, ncount->z_obs_params);
       break;
-    }
     case PROP_LNM_OBS:
-    {
-      GVariant *var = ncm_matrix_peek_variant (ncount->lnM_obs); 
-      g_value_take_variant (value, var);
+      g_value_set_object (value, ncount->lnM_obs);
       break;
-    }
     case PROP_LNM_OBS_PARAMS:
-    {
-      if (ncount->lnM_obs_params != NULL)
-      {
-        GVariant *var = ncm_matrix_peek_variant (ncount->lnM_obs_params); 
-        g_value_take_variant (value, var);
-      }
+      g_value_set_object (value, ncount->lnM_obs_params);
       break;
-    }
     case PROP_SURVEY_AREA:
       g_value_set_double (value, ncount->area_survey);
       break;
@@ -297,23 +229,11 @@ nc_data_cluster_ncount_get_property (GObject *object, guint prop_id, GValue *val
       g_value_set_boolean (value, ncount->binned);
       break;
     case PROP_Z_NODES:
-    {
-      if (ncount->z_nodes != NULL)
-      {
-        GVariant *var = ncm_vector_peek_variant (ncount->z_nodes); 
-        g_value_take_variant (value, var);
-      }
+      g_value_set_object (value, ncount->z_nodes);
       break;
-    }
     case PROP_LNM_NODES:
-    {
-      if (ncount->z_nodes != NULL)
-      {
-        GVariant *var = ncm_vector_peek_variant (ncount->lnM_nodes); 
-        g_value_take_variant (value, var);
-      }
+      g_value_set_object (value, ncount->lnM_nodes);
       break;
-    }
     case PROP_FIDUCIAL:
       g_value_set_boolean (value, ncount->fiducial);
       break;
@@ -350,7 +270,7 @@ nc_data_cluster_ncount_dispose (GObject *object)
   ncm_vector_clear (&ncount->z_nodes);  
 
   g_clear_pointer (&ncount->m2lnL_a, g_array_unref);
-  
+
   /* Chain up : end */
   G_OBJECT_CLASS (nc_data_cluster_ncount_parent_class)->dispose (object);
 }
@@ -394,7 +314,7 @@ nc_data_cluster_ncount_class_init (NcDataClusterNCountClass *klass)
                                                         "Cluster abundance",
                                                         NC_TYPE_CLUSTER_ABUNDANCE,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-  
+
   g_object_class_install_property (object_class,
                                    PROP_CLUSTERM,
                                    g_param_spec_object ("mass",
@@ -412,46 +332,46 @@ nc_data_cluster_ncount_class_init (NcDataClusterNCountClass *klass)
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_LNM_TRUE,
-                                   g_param_spec_variant ("lnM-true",
-                                                         NULL,
-                                                         "Clusters true masses",
-                                                         G_VARIANT_TYPE ("ad"), NULL,
-                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));  
+                                   g_param_spec_object ("lnM-true",
+                                                        NULL,
+                                                        "Clusters true masses",
+                                                        NCM_TYPE_VECTOR,
+                                                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));  
   g_object_class_install_property (object_class,
                                    PROP_Z_TRUE,
-                                   g_param_spec_variant ("z-true",
-                                                         NULL,
-                                                         "Clusters true redshifts",
-                                                         G_VARIANT_TYPE ("ad"), NULL,
-                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                   g_param_spec_object ("z-true",
+                                                        NULL,
+                                                        "Clusters true redshifts",
+                                                        NCM_TYPE_VECTOR,
+                                                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_LNM_OBS,
-                                   g_param_spec_variant ("lnM-obs",
-                                                         NULL,
-                                                         "Clusters mass observables",
-                                                         G_VARIANT_TYPE ("aad"), NULL,
-                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                   g_param_spec_object ("lnM-obs",
+                                                        NULL,
+                                                        "Clusters mass observables",
+                                                        NCM_TYPE_MATRIX,
+                                                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_LNM_OBS_PARAMS,
-                                   g_param_spec_variant ("lnM-obs-params",
-                                                         NULL,
-                                                         "Clusters mass observables parameters",
-                                                         G_VARIANT_TYPE ("aad"), NULL,
-                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                   g_param_spec_object ("lnM-obs-params",
+                                                        NULL,
+                                                        "Clusters mass observables parameters",
+                                                        NCM_TYPE_MATRIX,
+                                                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_Z_OBS,
-                                   g_param_spec_variant ("z-obs",
-                                                         NULL,
-                                                         "Clusters redshift observables",
-                                                         G_VARIANT_TYPE ("aad"), NULL,
-                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                   g_param_spec_object ("z-obs",
+                                                        NULL,
+                                                        "Clusters redshift observables",
+                                                        NCM_TYPE_MATRIX,
+                                                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_Z_OBS_PARAMS,
-                                   g_param_spec_variant ("z-obs-params",
-                                                         NULL,
-                                                         "Clusters redshift observables parameters",
-                                                         G_VARIANT_TYPE ("aad"), NULL,
-                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                   g_param_spec_object ("z-obs-params",
+                                                        NULL,
+                                                        "Clusters redshift observables parameters",
+                                                        NCM_TYPE_MATRIX,
+                                                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_SURVEY_AREA,
                                    g_param_spec_double ("area",
@@ -475,18 +395,18 @@ nc_data_cluster_ncount_class_init (NcDataClusterNCountClass *klass)
                                                          G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_Z_NODES,
-                                   g_param_spec_variant ("z-nodes",
-                                                         NULL,
-                                                         "Clusters redshifts nodes for binning",
-                                                         G_VARIANT_TYPE ("ad"), NULL,
-                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                   g_param_spec_object ("z-nodes",
+                                                        NULL,
+                                                        "Clusters redshifts nodes for binning",
+                                                        NCM_TYPE_VECTOR,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_LNM_NODES,
-                                   g_param_spec_variant ("lnM-nodes",
-                                                         NULL,
-                                                         "Clusters mass nodes for binning",
-                                                         G_VARIANT_TYPE ("ad"), NULL,
-                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                   g_param_spec_object ("lnM-nodes",
+                                                        NULL,
+                                                        "Clusters mass nodes for binning",
+                                                        NCM_TYPE_VECTOR,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 
   g_object_class_install_property (object_class,
                                    PROP_FIDUCIAL,
@@ -534,7 +454,7 @@ _nc_data_cluster_ncount_begin (NcmData *data)
 
 /**
  * nc_data_cluster_ncount_new:
- * @cad: FIXME
+ * @cad: a #NcClusterAbundance
  *
  * FIXME
  * 
@@ -552,7 +472,7 @@ nc_data_cluster_ncount_new (NcClusterAbundance *cad)
 
 /**
  * nc_data_cluster_ncount_ref:
- * @ncount: FIXME
+ * @ncount: a #NcDataClusterNCount
  *
  * FIXME
  * 
@@ -566,7 +486,7 @@ nc_data_cluster_ncount_ref (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_free:
- * @ncount: FIXME
+ * @ncount: a #NcDataClusterNCount
  *
  * FIXME
  * 
@@ -579,7 +499,7 @@ nc_data_cluster_ncount_free (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_clear:
- * @ncount: FIXME
+ * @ncount: a #NcDataClusterNCount
  *
  * FIXME
  * 
@@ -592,8 +512,8 @@ nc_data_cluster_ncount_clear (NcDataClusterNCount **ncount)
 
 /**
  * nc_data_cluster_ncount_set_mass:
- * @ncount: a #NcDataClusterNCount.
- * @m: the #NcClusterMass object.
+ * @ncount: a #NcDataClusterNCount
+ * @m: a #NcClusterMass
  *
  * Sets the mass observable object.
  * 
@@ -612,8 +532,8 @@ nc_data_cluster_ncount_set_mass (NcDataClusterNCount *ncount, NcClusterMass *m)
 
 /**
  * nc_data_cluster_ncount_set_redshift:
- * @ncount: a #NcDataClusterNCount.
- * @z: the #NcClusterRedshift object.
+ * @ncount: a #NcDataClusterNCount
+ * @z: a #NcClusterRedshift
  *
  * Sets the redshift observable object.
  * 
@@ -631,10 +551,10 @@ nc_data_cluster_ncount_set_redshift (NcDataClusterNCount *ncount, NcClusterRedsh
 
 /**
  * nc_data_cluster_ncount_set_lnM_true:
- * @ncount: a #NcDataClusterNCount.
- * @v: the value of the masses.
+ * @ncount: a #NcDataClusterNCount
+ * @v: a #NcmVector
  *
- * Sets the vector representing the clusters true masses.
+ * Sets the vector @v containing the values of the clusters true masses.
  * 
  */
 void 
@@ -659,10 +579,10 @@ nc_data_cluster_ncount_set_lnM_true (NcDataClusterNCount *ncount, const NcmVecto
 
 /**
  * nc_data_cluster_ncount_set_z_true:
- * @ncount: a #NcDataClusterNCount.
- * @v: the value of the redshifts.
+ * @ncount: a #NcDataClusterNCount
+ * @v: a #NcmVector 
  *
- * Sets the vector representing the clusters true redshifts.
+ * Sets the vector @v containing the true redshift values of the clusters.
  * 
  */
 void 
@@ -687,10 +607,10 @@ nc_data_cluster_ncount_set_z_true (NcDataClusterNCount *ncount, const NcmVector
 
 /**
  * nc_data_cluster_ncount_set_lnM_obs:
- * @ncount: a #NcDataClusterNCount.
- * @m: the value of the mass observables.
+ * @ncount: a #NcDataClusterNCount
+ * @m: a #NcmMatrix
  *
- * Sets the matrix representing the clusters mass observables.
+ * Sets the matrix @m representing the cluster mass observables.
  * 
  */
 void 
@@ -718,10 +638,10 @@ nc_data_cluster_ncount_set_lnM_obs (NcDataClusterNCount *ncount, const NcmMatrix
 
 /**
  * nc_data_cluster_ncount_set_lnM_obs_params:
- * @ncount: a #NcDataClusterNCount.
- * @m: the mass observables parameters.
+ * @ncount: a #NcDataClusterNCount
+ * @m: a #NcmMatrix
  *
- * Sets the matrix representing the clusters mass observables parameters.
+ * Sets the matrix @m representing the cluster mass-observable parameters.
  * 
  */
 void 
@@ -749,10 +669,10 @@ nc_data_cluster_ncount_set_lnM_obs_params (NcDataClusterNCount *ncount, const Nc
 
 /**
  * nc_data_cluster_ncount_set_z_obs:
- * @ncount: a #NcDataClusterNCount.
- * @m: the value of the redshift observables.
+ * @ncount: a #NcDataClusterNCount
+ * @m: a #NcmMatrix
  *
- * Sets the matrix representing the clusters redshift observables.
+ * Sets the matrix @m representing the cluster redshift observables.
  * 
  */
 void 
@@ -780,10 +700,10 @@ nc_data_cluster_ncount_set_z_obs (NcDataClusterNCount *ncount, const NcmMatrix *
 
 /**
  * nc_data_cluster_ncount_set_z_obs_params:
- * @ncount: a #NcDataClusterNCount.
- * @m: the redshift observables parameters.
+ * @ncount: a #NcDataClusterNCount
+ * @m: a #NcmMatrix
  *
- * Sets the matrix representing the clusters redshift observables parameters.
+ * Sets the matrix @m representing the cluster redshift observable parameters.
  * 
  */
 void 
@@ -811,7 +731,7 @@ nc_data_cluster_ncount_set_z_obs_params (NcDataClusterNCount *ncount, const NcmM
 
 /**
  * nc_data_cluster_ncount_has_lnM_true:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  *
  * Returns: TRUE if it contains the lnM truth table.
  * 
@@ -824,7 +744,7 @@ nc_data_cluster_ncount_has_lnM_true (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_has_z_true:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  *
  * Returns: TRUE if it contains the redshift truth table.
  * 
@@ -837,7 +757,7 @@ nc_data_cluster_ncount_has_z_true (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_get_len:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  * 
  * Gets the total number of objects.
  * 
@@ -851,7 +771,7 @@ nc_data_cluster_ncount_get_len (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_lnM_obs_len:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  * 
  * Number of doubles to describe the observational data related to the mass
  * of each object.
@@ -867,7 +787,7 @@ nc_data_cluster_ncount_lnM_obs_len (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_lnM_obs_params_len:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  * 
  * Number of doubles to describe the observational data parameters related 
  * mass to each object.
@@ -885,7 +805,7 @@ nc_data_cluster_ncount_lnM_obs_params_len (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_z_obs_len:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  * 
  * Number of doubles to describe the observational data related to
  * the redshift of each object.
@@ -901,7 +821,7 @@ nc_data_cluster_ncount_z_obs_len (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_z_params_len:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  * 
  * Number of doubles to describe the observational data parameters related 
  * to the redshift of each object.
@@ -919,7 +839,7 @@ nc_data_cluster_ncount_z_obs_params_len (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_get_lnM_true:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  * 
  * Gets the vector containing the true values of the masses.
  * 
@@ -936,7 +856,7 @@ nc_data_cluster_ncount_get_lnM_true (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_get_z_true:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  * 
  * Gets the vector containing the true values of the redshifts.
  * 
@@ -953,7 +873,7 @@ nc_data_cluster_ncount_get_z_true (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_get_lnM_obs:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  * 
  * Gets the matrix containing the mass observables.
  * 
@@ -970,7 +890,7 @@ nc_data_cluster_ncount_get_lnM_obs (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_get_lnM_obs_params:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  * 
  * Gets the matrix containing the mass observables parameters.
  * 
@@ -987,7 +907,7 @@ nc_data_cluster_ncount_get_lnM_obs_params (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_get_z_obs:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  * 
  * Gets the matrix containing the redshift observables.
  * 
@@ -1004,7 +924,7 @@ nc_data_cluster_ncount_get_z_obs (NcDataClusterNCount *ncount)
 
 /**
  * nc_data_cluster_ncount_get_z_obs_params:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  * 
  * Gets the matrix containing the redshift observables parameters.
  * 
@@ -1028,12 +948,6 @@ _nc_data_cluster_ncount_model_init (NcDataClusterNCount *ncount)
 {
   NcClusterAbundance *cad = ncount->cad;
 
-  if (ncount->z == NULL || ncount->m == NULL)
-    g_error ("_nc_data_cluster_ncount_model_init: Cannot init NcClusterAbundance missing NcClusterRedshift or NcClusterMass object.");
-
-  nc_cluster_abundance_set_redshift (cad, ncount->z);
-  nc_cluster_abundance_set_mass (cad, ncount->m);
-
   cad->completeness  = ncount->completeness;
   cad->purity        = ncount->purity;
   cad->sd_lnM        = ncount->sd_lnM;
@@ -1045,12 +959,16 @@ static void
 _nc_data_cluster_ncount_prepare (NcmData *data, NcmMSet *mset)
 {
   NcDataClusterNCount *ncount = NC_DATA_CLUSTER_NCOUNT (data);
-  NcHICosmo *cosmo = NC_HICOSMO (ncm_mset_peek (mset, nc_hicosmo_id ()));
+  NcHICosmo *cosmo            = NC_HICOSMO (ncm_mset_peek (mset, nc_hicosmo_id ()));
+  NcClusterRedshift *clusterz = NC_CLUSTER_REDSHIFT (ncm_mset_peek (mset, nc_cluster_redshift_id ()));
+  NcClusterMass *clusterm     = NC_CLUSTER_MASS (ncm_mset_peek (mset, nc_cluster_mass_id ()));
 
-  if (ncm_model_ctrl_update (ncount->cad->ctrl, NCM_MODEL (cosmo)))
-  {
-    nc_cluster_abundance_prepare (ncount->cad, cosmo);
-  }
+  g_assert ((cosmo != NULL) && (clusterz != NULL) && (clusterm != NULL));
+  
+  g_assert (ncount->z == NULL || g_type_is_a (G_OBJECT_TYPE (clusterz), G_OBJECT_TYPE (ncount->z)));
+  g_assert (ncount->m == NULL || g_type_is_a (G_OBJECT_TYPE (clusterm), G_OBJECT_TYPE (ncount->m)));
+    
+  nc_cluster_abundance_prepare_if_needed (ncount->cad, cosmo, clusterz, clusterm);
 }
 
 static gchar *
@@ -1082,10 +1000,12 @@ _nc_data_cluster_ncount_resample (NcmData *data, NcmMSet *mset, NcmRNG *rng)
   NcDataClusterNCount *ncount = NC_DATA_CLUSTER_NCOUNT (data);
   NcClusterAbundance *cad = ncount->cad;
   NcHICosmo *cosmo = NC_HICOSMO (ncm_mset_peek (mset, nc_hicosmo_id ()));
-  guint z_obs_len = nc_cluster_redshift_obs_len (cad->z);
-  guint z_obs_params_len = nc_cluster_redshift_obs_params_len (cad->z);
-  guint lnM_obs_len = nc_cluster_mass_obs_len (cad->m);
-  guint lnM_obs_params_len = nc_cluster_mass_obs_params_len (cad->m);
+  NcClusterRedshift *clusterz = NC_CLUSTER_REDSHIFT (ncm_mset_peek (mset, nc_cluster_redshift_id ()));
+  NcClusterMass *clusterm     = NC_CLUSTER_MASS (ncm_mset_peek (mset, nc_cluster_mass_id ()));
+  guint z_obs_len = nc_cluster_redshift_obs_len (clusterz);
+  guint z_obs_params_len = nc_cluster_redshift_obs_params_len (clusterz);
+  guint lnM_obs_len = nc_cluster_mass_obs_len (clusterm);
+  guint lnM_obs_params_len = nc_cluster_mass_obs_params_len (clusterm);
   GArray *lnM_true_array = NULL;
   GArray *z_true_array = NULL;
   GArray *z_obs_array = NULL;
@@ -1146,8 +1066,8 @@ _nc_data_cluster_ncount_resample (NcmData *data, NcmMSet *mset, NcmRNG *rng)
       const gdouble lnM_true = ncm_spline2d_eval (cad->inv_lnM_z, u2, z_true);
       ncm_rng_unlock (rng);
 
-      if ( nc_cluster_redshift_resample (cad->z, lnM_true, z_true, zi_obs, zi_obs_params, rng) &&
-          nc_cluster_mass_resample (cad->m, cosmo, lnM_true, z_true, lnMi_obs, lnMi_obs_params, rng) )
+      if ( nc_cluster_redshift_resample (clusterz, lnM_true, z_true, zi_obs, zi_obs_params, rng) &&
+          nc_cluster_mass_resample (clusterm, cosmo, lnM_true, z_true, lnMi_obs, lnMi_obs_params, rng) )
       {
         g_array_append_val (lnM_true_array, lnM_true);
         g_array_append_val (z_true_array, z_true);
@@ -1241,6 +1161,8 @@ typedef struct
 {
   NcClusterAbundance *cad;
   NcDataClusterNCount *ncount;
+  NcClusterRedshift *clusterz;
+  NcClusterMass *clusterm;
   NcHICosmo *cosmo;
 } _Evald2N;
 
@@ -1256,7 +1178,7 @@ _eval_z_p_lnm_p_d2n (glong i, glong f, gpointer data)
     gdouble *lnMn_obs_params = evald2n->ncount->lnM_obs_params != NULL ? ncm_matrix_ptr (evald2n->ncount->lnM_obs_params, n, 0) : NULL;
     gdouble *zn_obs = ncm_matrix_ptr (evald2n->ncount->z_obs, n, 0);
     gdouble *zn_obs_params = evald2n->ncount->z_obs_params != NULL ? ncm_matrix_ptr (evald2n->ncount->z_obs_params, n, 0) : NULL;
-    const gdouble mlnLn = -log (nc_cluster_abundance_z_p_lnm_p_d2n (evald2n->cad, evald2n->cosmo, lnMn_obs, lnMn_obs_params, zn_obs, zn_obs_params));
+    const gdouble mlnLn = -log (nc_cluster_abundance_z_p_lnm_p_d2n (evald2n->cad, evald2n->cosmo, evald2n->clusterz, evald2n->clusterm, lnMn_obs, lnMn_obs_params, zn_obs, zn_obs_params));
     g_array_index (evald2n->ncount->m2lnL_a, gdouble, n) = mlnLn;
   }
 }
@@ -1272,7 +1194,7 @@ _eval_z_p_d2n (glong i, glong f, gpointer data)
     const gdouble lnMn = ncm_vector_get (evald2n->ncount->lnM_true, n);
     gdouble *zn_obs = ncm_matrix_ptr (evald2n->ncount->z_obs, n, 0);
     gdouble *zn_obs_params = evald2n->ncount->z_obs_params != NULL ? ncm_matrix_ptr (evald2n->ncount->z_obs_params, n, 0) : NULL;
-    const gdouble mlnLn = -log (nc_cluster_abundance_z_p_d2n (evald2n->cad, evald2n->cosmo, lnMn, zn_obs, zn_obs_params));
+    const gdouble mlnLn = -log (nc_cluster_abundance_z_p_d2n (evald2n->cad, evald2n->cosmo, evald2n->clusterz, evald2n->clusterm, lnMn, zn_obs, zn_obs_params));
     g_array_index (evald2n->ncount->m2lnL_a, gdouble, n) = mlnLn;
   }
 }
@@ -1288,7 +1210,7 @@ _eval_lnm_p_d2n (glong i, glong f, gpointer data)
     const gdouble zn = ncm_vector_get (evald2n->ncount->z_true, n);
     gdouble *lnMn_obs = ncm_matrix_ptr (evald2n->ncount->lnM_obs, n, 0);
     gdouble *lnMn_obs_params = evald2n->ncount->lnM_obs_params != NULL ? ncm_matrix_ptr (evald2n->ncount->lnM_obs_params, n, 0) : NULL;
-    const gdouble mlnLn = -log (nc_cluster_abundance_lnm_p_d2n (evald2n->cad, evald2n->cosmo, lnMn_obs, lnMn_obs_params, zn));
+    const gdouble mlnLn = -log (nc_cluster_abundance_lnm_p_d2n (evald2n->cad, evald2n->cosmo, evald2n->clusterz, evald2n->clusterm, lnMn_obs, lnMn_obs_params, zn));
     g_array_index (evald2n->ncount->m2lnL_a, gdouble, n) = mlnLn;
   }
 }
@@ -1303,7 +1225,7 @@ _eval_d2n (glong i, glong f, gpointer data)
   {
     const gdouble lnMn = ncm_vector_get (evald2n->ncount->lnM_true, n);
     const gdouble zn = ncm_vector_get (evald2n->ncount->z_true, n);
-    const gdouble mlnLn = -log (nc_cluster_abundance_d2n (evald2n->cad, evald2n->cosmo, lnMn, zn));
+    const gdouble mlnLn = -log (nc_cluster_abundance_d2n (evald2n->cad, evald2n->cosmo, evald2n->clusterz, evald2n->clusterm, lnMn, zn));
     g_array_index (evald2n->ncount->m2lnL_a, gdouble, n) = mlnLn;
   }
 }
@@ -1318,7 +1240,7 @@ _eval_intp_d2n (glong i, glong f, gpointer data)
   {
     const gdouble lnMn = ncm_vector_get (evald2n->ncount->lnM_true, n);
     const gdouble zn = ncm_vector_get (evald2n->ncount->z_true, n);
-    const gdouble mlnLn = -log (nc_cluster_abundance_intp_d2n (evald2n->cad, evald2n->cosmo, lnMn, zn));
+    const gdouble mlnLn = -log (nc_cluster_abundance_intp_d2n (evald2n->cad, evald2n->cosmo, evald2n->clusterz, evald2n->clusterm, lnMn, zn));
     g_array_index (evald2n->ncount->m2lnL_a, gdouble, n) = mlnLn;
   }
 }
@@ -1329,6 +1251,8 @@ _nc_data_cluster_ncount_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
   NcDataClusterNCount *ncount = NC_DATA_CLUSTER_NCOUNT (data);
   NcClusterAbundance *cad = ncount->cad;
   NcHICosmo *cosmo = NC_HICOSMO (ncm_mset_peek (mset, nc_hicosmo_id ()));
+  NcClusterRedshift *clusterz = NC_CLUSTER_REDSHIFT (ncm_mset_peek (mset, nc_cluster_redshift_id ()));
+  NcClusterMass *clusterm = NC_CLUSTER_MASS (ncm_mset_peek (mset, nc_cluster_mass_id ()));
   
   *m2lnL = 0.0;
 
@@ -1337,7 +1261,7 @@ _nc_data_cluster_ncount_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
 
   if (ncount->np == 0)
   {
-    const gdouble n_th = nc_cluster_abundance_n (cad, cosmo);    
+    const gdouble n_th = nc_cluster_abundance_n (cad, cosmo, clusterz, clusterm);    
     *m2lnL = 2.0 * (ncount->log_np_fac + n_th);
     return;
   }
@@ -1346,7 +1270,7 @@ _nc_data_cluster_ncount_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
 
   if (ncount->use_true_data)
   {
-    _Evald2N evald2n = {cad, ncount, cosmo};
+    _Evald2N evald2n = {cad, ncount, clusterz, clusterm, cosmo};
     g_assert (ncount->z_true);
     g_assert (ncount->lnM_true);
     ncm_func_eval_threaded_loop_full (&_eval_intp_d2n, 0, ncount->np, &evald2n);
@@ -1360,33 +1284,33 @@ _nc_data_cluster_ncount_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
     
     if (z_p && lnM_p)
     {
-      _Evald2N evald2n = {cad, ncount, cosmo};
+      _Evald2N evald2n = {cad, ncount, clusterz, clusterm, cosmo};
       ncm_func_eval_threaded_loop_full (&_eval_z_p_lnm_p_d2n, 0, ncount->np, &evald2n);
     }
     else if (z_p && !lnM_p)
     {
       g_assert (ncount->lnM_true);
-      _Evald2N evald2n = {cad, ncount, cosmo};
+      _Evald2N evald2n = {cad, ncount, clusterz, clusterm, cosmo};
       ncm_func_eval_threaded_loop_full (&_eval_z_p_d2n, 0, ncount->np, &evald2n);
     }
     else if (!z_p && lnM_p)
     {
       g_assert (ncount->z_true);
-      _Evald2N evald2n = {cad, ncount, cosmo};
+      _Evald2N evald2n = {cad, ncount, clusterz, clusterm, cosmo};
       ncm_func_eval_threaded_loop_full (&_eval_lnm_p_d2n, 0, ncount->np, &evald2n);
     }
     else
     {
       g_assert (ncount->z_true);
       g_assert (ncount->lnM_true);
-      _Evald2N evald2n = {cad, ncount, cosmo};
+      _Evald2N evald2n = {cad, ncount, clusterz, clusterm, cosmo};
       ncm_func_eval_threaded_loop_full (&_eval_d2n, 0, ncount->np, &evald2n);
 
     }
   }
 
   {
-    const gdouble n_th = nc_cluster_abundance_n (cad, cosmo);
+    const gdouble n_th = nc_cluster_abundance_n (cad, cosmo, clusterz, clusterm);
     guint i;
     for (i = 0 ; i < ncount->np; i++) 
     { 
@@ -1400,7 +1324,7 @@ _nc_data_cluster_ncount_m2lnL_val (NcmData *data, NcmMSet *mset, gdouble *m2lnL)
 
 /**
  * nc_data_cluster_ncount_true_data:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  * @use_true_data: FIXME
  *
  * FIXME
@@ -1419,7 +1343,7 @@ nc_data_cluster_ncount_true_data (NcDataClusterNCount *ncount, gboolean use_true
 
 /**
  * nc_data_cluster_ncount_using_true_data:
- * @ncount: a #NcDataClusterNCount.
+ * @ncount: a #NcDataClusterNCount
  *
  * Returns: if it is using true data.
  */
@@ -1433,12 +1357,12 @@ static void _nc_data_cluster_ncount_model_init (NcDataClusterNCount *ncount);
 
 /**
  * nc_data_cluster_ncount_init_from_sampling:
- * @ncount: a #NcDataClusterNCount.
- * @mset: a #NcmMSet.
- * @clusterz: a #NcClusterRedshift.
- * @clusterm: a #NcClusterMass.
- * @area_survey: area in units of square degrees.
- * @rng: a #NcmRNG.
+ * @ncount: a #NcDataClusterNCount
+ * @mset: a #NcmMSet
+ * @clusterz: a #NcClusterRedshift
+ * @clusterm: a #NcClusterMass
+ * @area_survey: area in units of square degrees
+ * @rng: a #NcmRNG
  *
  * FIXME
  *
@@ -1460,8 +1384,8 @@ nc_data_cluster_ncount_init_from_sampling (NcDataClusterNCount *ncount, NcmMSet
 
 /**
  * nc_data_cluster_ncount_print:
- * @ncount: a #NcDataClusterNCount.
- * @cosmo: a NcHICosmo
+ * @ncount: a #NcDataClusterNCount
+ * @cosmo: a #NcHICosmo
  * @out: FIXME
  * @header: FIXME
  *
@@ -1590,9 +1514,9 @@ _nc_data_cluster_ncount_bin_alloc (NcDataClusterNCount *ncount, guint z_bins, gu
 
 /**
  * nc_data_cluster_ncount_set_bin_by_nodes:
- * @ncount: a #NcDataClusterNCount.
- * @z_nodes: FIXME
- * @lnM_nodes: FIXME
+ * @ncount: a #NcDataClusterNCount
+ * @z_nodes: a #NcmVector
+ * @lnM_nodes: a #NcmVector
  *
  * FIXME
  *
