@@ -79,7 +79,8 @@ nc_data_hubble_set_property (GObject *object, guint prop_id, const GValue *value
   switch (prop_id)
   {
     case PROP_Z:
-      ncm_vector_set_from_variant (hubble->x, g_value_get_variant (value));
+      ncm_vector_clear (&hubble->x);
+      hubble->x = g_value_dup_object (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -96,7 +97,7 @@ nc_data_hubble_get_property (GObject *object, guint prop_id, GValue *value, GPar
   switch (prop_id)
   {
     case PROP_Z:
-      g_value_take_variant (value, ncm_vector_get_variant (hubble->x));
+      g_value_set_object (value, hubble->x);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -142,12 +143,12 @@ nc_data_hubble_class_init (NcDataHubbleClass *klass)
 
   g_object_class_install_property (object_class,
                                    PROP_Z,
-                                   g_param_spec_variant ("z",
-                                                         NULL,
-                                                         "Data redshifts",
-                                                         G_VARIANT_TYPE ("ad"), NULL,
-                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-
+                                   g_param_spec_object ("z",
+                                                        NULL,
+                                                        "Data redshifts",
+                                                        NCM_TYPE_VECTOR,
+                                                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  
   data_class->prepare   = &_nc_data_hubble_prepare;
   diag_class->mean_func = &_nc_data_hubble_mean_func;
   diag_class->set_size  = &_nc_data_hubble_set_size;
