@@ -7,7 +7,7 @@
  ****************************************************************************/
 /*
  * numcosmo
- * Copyright (C) Mariana Penna Lima 2012 <pennalima@gmail.com> 
+ * Copyright (C) Mariana Penna Lima 2012 <pennalima@gmail.com>
  * Sandro Dias Pinto Vitenti 2012 <sandro@isoftware.com.br>
  *
  * numcosmo is free software: you can redistribute it and/or modify it
@@ -29,7 +29,10 @@
  * @title: NcClusterMass
  * @short_description: Abstract class for cluster mass distributions.
  *
- * FIXME
+ * NcClusterMass is the abstract class designed to abrigde the functions
+ * that any cluster mass distribution should implement, see NcClusterMassImpl.
+ * Its parent_class is NcmModel.
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,7 +47,7 @@
 G_DEFINE_ABSTRACT_TYPE (NcClusterMass, nc_cluster_mass, NCM_TYPE_MODEL);
 
 /**
- * nc_cluster_mass_new_from_name: 
+ * nc_cluster_mass_new_from_name:
  * @mass_name: string which specifies the type of the mass distribution.
  *
  * This function returns a new #NcClusterMass whose type is defined by @mass_name.
@@ -63,11 +66,11 @@ nc_cluster_mass_new_from_name (gchar *mass_name)
 
 /**
  * nc_cluster_mass_ref:
- * @clusterm: a #NcClusterMass.
+ * @clusterm: a #NcClusterMass
  *
- * FIXME
+ * Increases the reference count of @clusterm by one.
  *
- * Returns: (transfer full): @clusterm.
+ * Returns: (transfer full): @clusterm
  */
 NcClusterMass *
 nc_cluster_mass_ref (NcClusterMass *clusterm)
@@ -77,9 +80,10 @@ nc_cluster_mass_ref (NcClusterMass *clusterm)
 
 /**
  * nc_cluster_mass_free:
- * @clusterm: a #NcClusterMass.
+ * @clusterm: a #NcClusterMass
  *
- * FIXME
+ * Atomically decrements the reference count of @clusterm by one. If the reference count drops to 0,
+ * all memory allocated by @clusterm is released.
  *
  */
 void
@@ -90,9 +94,9 @@ nc_cluster_mass_free (NcClusterMass *clusterm)
 
 /**
  * nc_cluster_mass_clear:
- * @clusterm: a #NcClusterMass.
+ * @clusterm: a #NcClusterMass
  *
- * FIXME
+ * The reference count of @clusterm is decreased and the pointer is set to NULL.
  *
  */
 void
@@ -103,13 +107,13 @@ nc_cluster_mass_clear (NcClusterMass **clusterm)
 
 /**
  * nc_cluster_mass_impl:
- * @clusterm: a #NcClusterMass.
+ * @clusterm: a #NcClusterMass
  *
  * FIXME
- * 
+ *
  * Returns: FIXME
  */
-NcClusterMassImpl 
+NcClusterMassImpl
 nc_cluster_mass_impl (NcClusterMass *clusterm)
 {
   return NC_CLUSTER_MASS_GET_CLASS (clusterm)->impl;
@@ -117,11 +121,17 @@ nc_cluster_mass_impl (NcClusterMass *clusterm)
 
 /**
  * nc_cluster_mass_obs_len:
- * @clusterm: a #NcClusterMass.
+ * @clusterm: a #NcClusterMass
  *
- * FIXME
+ * The number of observable masses (or just the observable which is related to the cluster mass)
+ * of each cluster, e.g.,
+ * 1 - SZ mass,
+ * 1 - X-ray mass,
+ * 1 - Lensing mass,
+ * 2 - SZ and X-ray masses,
+ * 3 - SZ, X-ray and lensing masses.
  *
- * Returns: FIXME
+ * Returns: The number of observable masses.
  */
 guint
 nc_cluster_mass_obs_len (NcClusterMass *clusterm)
@@ -131,11 +141,14 @@ nc_cluster_mass_obs_len (NcClusterMass *clusterm)
 
 /**
  * nc_cluster_mass_obs_params_len:
- * @clusterm: a #NcClusterMass.
+ * @clusterm: a #NcClusterMass
  *
- * FIXME
+ * The number of parameters related to the observable masses of each cluster, e.g.,
+ * 1 - error of the SZ mass,
+ * 1 - error of the X-ray mass,
+ * 2 - errors of SZ and X-ray masses.
  *
- * Returns: FIXME
+ * Returns: The number of parameters related to the observable masses.
  */
 guint
 nc_cluster_mass_obs_params_len (NcClusterMass *clusterm)
@@ -145,44 +158,48 @@ nc_cluster_mass_obs_params_len (NcClusterMass *clusterm)
 
 /**
  * nc_cluster_mass_p:
- * @clusterm: a #NcClusterMass.
- * @model: a #NcHICosmo. 
- * @z: true redshift.
- * @lnM: logarithm base e of the true mass.
- * @lnM_obs: logarithm base e of the observed mass.
- * @lnM_obs_params: observed mass params.
+ * @clusterm: a #NcClusterMass
+ * @cosmo: a #NcHICosmo
+ * @lnM:logarithm base e of the true mass
+ * @z: true redshift
+ * @lnM_obs: (array) (element-type double): logarithm base e of the observed mass
+ * @lnM_obs_params: (array) (element-type double): observed mass paramaters
  *
- * FIXME
+ * It computes the probability density function (pdf) of the cluster mass distribution @clusterm
+ * given @cosmo, @lnM, @z and the observable cluster mass (or just the observable).
  *
- * Returns: FIXME
+ * Returns: The pdf of @clusterm.
 */
 gdouble
-nc_cluster_mass_p (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z, gdouble *lnM_obs, gdouble *lnM_obs_params)
+nc_cluster_mass_p (NcClusterMass *clusterm, NcHICosmo *cosmo, gdouble lnM, gdouble z, const gdouble *lnM_obs, const gdouble *lnM_obs_params)
 {
-  return NC_CLUSTER_MASS_GET_CLASS (clusterm)->P (clusterm, model, lnM, z, lnM_obs, lnM_obs_params);
+  return NC_CLUSTER_MASS_GET_CLASS (clusterm)->P (clusterm, cosmo, lnM, z, lnM_obs, lnM_obs_params);
 }
 
 /**
  * nc_cluster_mass_intp:
- * @clusterm: a #NcClusterMass.
- * @model: a #NcHICosmo.
- * @z: true redshift.
- * @lnM: logarithm base e of the true mass.
+ * @clusterm: a #NcClusterMass
+ * @cosmo: a #NcHICosmo
+ * @z: true redshift
+ * @lnM: logarithm base e of the true mass
  *
- * FIXME
+ * It computes the @clusterm probability distribution of @lnM lying
+ * in the range $[]$, namely,
+ * $$ intp = \int_{\ln M^{obs}_{min}}^{\ln M^{obs}_{max}} p \, d\ln M^{obs},$$
+ * where $p$ is [nc_cluster_mass_p()].
  *
- * Returns: FIXME
+ * Returns: The probability distribution of @lnM lying within $[\ln M^{obs}_{min}, \ln M^{obs}_{max}]$.
 */
 gdouble
-nc_cluster_mass_intp (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z)
+nc_cluster_mass_intp (NcClusterMass *clusterm, NcHICosmo *cosmo, gdouble lnM, gdouble z)
 {
-  return NC_CLUSTER_MASS_GET_CLASS (clusterm)->intP (clusterm, model, lnM, z);
+  return NC_CLUSTER_MASS_GET_CLASS (clusterm)->intP (clusterm, cosmo, lnM, z);
 }
 
 /**
  * nc_cluster_mass_resample:
  * @clusterm: a #NcClusterMass.
- * @model: a #NcHICosmo. 
+ * @cosmo: a #NcHICosmo.
  * @z: true redshift.
  * @lnM: logarithm base e of the true mass.
  * @lnM_obs: (out): logarithm base e of the observed mass.
@@ -190,19 +207,19 @@ nc_cluster_mass_intp (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gd
  * @rng: a #NcmRNG.
  *
  * FIXME
- * 
+ *
  * Returns: FIXME
  */
 gboolean
-nc_cluster_mass_resample (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z, gdouble *lnM_obs, gdouble *lnM_obs_params, NcmRNG *rng)
+nc_cluster_mass_resample (NcClusterMass *clusterm, NcHICosmo *cosmo, gdouble lnM, gdouble z, gdouble *lnM_obs, const gdouble *lnM_obs_params, NcmRNG *rng)
 {
-  return NC_CLUSTER_MASS_GET_CLASS (clusterm)->resample (clusterm,model, lnM, z, lnM_obs, lnM_obs_params, rng);
+  return NC_CLUSTER_MASS_GET_CLASS (clusterm)->resample (clusterm, cosmo, lnM, z, lnM_obs, lnM_obs_params, rng);
 }
 
 /**
  * nc_cluster_mass_p_limits:
  * @clusterm: a #NcClusterMass.
- * @model: a #NcHICosmo. 
+ * @cosmo: a #NcHICosmo.
  * @lnM_obs: observed mass.
  * @lnM_obs_params: observed mass params.
  * @lnM_lower: (out): pointer to the lower limit of the real mass integration.
@@ -212,15 +229,15 @@ nc_cluster_mass_resample (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM
  * The function which will call this one is responsible to allocate memory for @lnM_lower and @lnM_upper.
  */
 void
-nc_cluster_mass_p_limits (NcClusterMass *clusterm, NcHICosmo *model, gdouble *lnM_obs, gdouble *lnM_obs_params, gdouble *lnM_lower, gdouble *lnM_upper)
+nc_cluster_mass_p_limits (NcClusterMass *clusterm, NcHICosmo *cosmo, const gdouble *lnM_obs, const gdouble *lnM_obs_params, gdouble *lnM_lower, gdouble *lnM_upper)
 {
-  NC_CLUSTER_MASS_GET_CLASS (clusterm)->P_limits (clusterm, model, lnM_obs, lnM_obs_params, lnM_lower, lnM_upper);
+  NC_CLUSTER_MASS_GET_CLASS (clusterm)->P_limits (clusterm, cosmo, lnM_obs, lnM_obs_params, lnM_lower, lnM_upper);
 }
 
 /**
  * nc_cluster_mass_n_limits:
  * @clusterm: a #NcClusterMass.
- * @model: a #NcHICosmo. 
+ * @cosmo: a #NcHICosmo.
  * @lnM_lower: (out): lower limit of the logarithm base e of the true mass.
  * @lnM_upper: (out): upper limit of the lgarithm base e of the true mass.
  *
@@ -228,9 +245,9 @@ nc_cluster_mass_p_limits (NcClusterMass *clusterm, NcHICosmo *model, gdouble *ln
  * The function which will call this one is responsible to allocate memory for @lnM_lower and @lnM_upper.
  */
 void
-nc_cluster_mass_n_limits (NcClusterMass *clusterm, NcHICosmo *model, gdouble *lnM_lower, gdouble *lnM_upper)
+nc_cluster_mass_n_limits (NcClusterMass *clusterm, NcHICosmo *cosmo, gdouble *lnM_lower, gdouble *lnM_upper)
 {
-  NC_CLUSTER_MASS_GET_CLASS (clusterm)->N_limits (clusterm, model, lnM_lower, lnM_upper);
+  NC_CLUSTER_MASS_GET_CLASS (clusterm)->N_limits (clusterm, cosmo, lnM_lower, lnM_upper);
 }
 
 static void
@@ -257,7 +274,7 @@ _nc_cluster_mass_log_all_models_go (GType model_type, guint n)
 /**
  * nc_cluster_mass_log_all_models:
  *
- * FIXME
+ * This function lists all implemented models of cluster mass distributions.
  *
  */
 void
@@ -276,7 +293,7 @@ nc_cluster_mass_init (NcClusterMass *nc_cluster_mass)
 static void
 nc_cluster_mass_finalize (GObject *object)
 {
-  
+
   /* Chain up : end */
   G_OBJECT_CLASS (nc_cluster_mass_parent_class)->finalize (object);
 }
@@ -287,12 +304,14 @@ static void
 nc_cluster_mass_class_init (NcClusterMassClass *klass)
 {
   GObjectClass* object_class = G_OBJECT_CLASS (klass);
+  NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
 
   object_class->finalize = nc_cluster_mass_finalize;
 
-  ncm_model_class_add_params (NCM_MODEL_CLASS (klass), 0, 0, 1);
-  
-  ncm_mset_model_register_id (NCM_MODEL_CLASS (klass), 
+  ncm_model_class_set_name_nick (model_class, "Cluster mass abstract class", "NcClusterMass");
+  ncm_model_class_add_params (model_class, 0, 0, 1);
+
+  ncm_mset_model_register_id (model_class,
                               "NcClusterMass",
                               "Cluster mass observable relation models.",
                               NULL);
