@@ -24,8 +24,8 @@
 
 /**
  * SECTION:nc_cluster_redshift
- * @title: Abstract Cluster Redshift Object
- * @short_description: Observed redshift distribution
+ * @title: NcClusterRedshift
+ * @short_description: Abstract class for cluster redshift distributions.
  *
  * FIXME
  */
@@ -40,7 +40,7 @@
 #include "math/ncm_cfg.h"
 #include "math/ncm_util.h"
 
-G_DEFINE_ABSTRACT_TYPE (NcClusterRedshift, nc_cluster_redshift, G_TYPE_OBJECT);
+G_DEFINE_ABSTRACT_TYPE (NcClusterRedshift, nc_cluster_redshift, NCM_TYPE_MODEL);
 
 /**
  * nc_cluster_redshift_new_from_name:
@@ -56,17 +56,17 @@ nc_cluster_redshift_new_from_name (gchar *redshift_name)
   GObject *obj = ncm_serialize_global_from_string (redshift_name);
   GType redshift_type = G_OBJECT_TYPE (obj);
   if (!g_type_is_a (redshift_type, NC_TYPE_CLUSTER_REDSHIFT))
-	g_error ("nc_cluster_redshift_new_from_name: NcClusterRedshift %s do not descend from %s\n", redshift_name, g_type_name (NC_TYPE_CLUSTER_REDSHIFT));
+    g_error ("nc_cluster_redshift_new_from_name: NcClusterRedshift %s do not descend from %s.", redshift_name, g_type_name (NC_TYPE_CLUSTER_REDSHIFT));
   return NC_CLUSTER_REDSHIFT (obj);
 }
 
 /**
  * nc_cluster_redshift_ref:
- * @clusterz: FIXME.
+ * @clusterz: a #NcClusterRedshift
  *
- * FIXME
+ * Increases the reference count of @clusterz by one.
  *
- * Returns: (transfer full): @clusterz.
+ * Returns: (transfer full): @clusterz
  */
 NcClusterRedshift *
 nc_cluster_redshift_ref (NcClusterRedshift *clusterz)
@@ -76,7 +76,7 @@ nc_cluster_redshift_ref (NcClusterRedshift *clusterz)
 
 /**
  * nc_cluster_redshift_free:
- * @clusterz: a #NcClusterRedshift.
+ * @clusterz: a #NcClusterRedshift
  *
  * Atomically decrements the reference count of @clusterz by one. If the reference count drops to 0,
  * all memory allocated by @clusterz is released.
@@ -90,10 +90,9 @@ nc_cluster_redshift_free (NcClusterRedshift *clusterz)
 
 /**
  * nc_cluster_redshift_clear:
- * @clusterz: a #NcClusterRedshift.
+ * @clusterz: a #NcClusterRedshift
  *
- * Atomically decrements the reference count of @clusterz by one. If the reference count drops to 0,
- * all memory allocated by @clusterz is released. Set pointer to NULL.
+ * Atomically decrements the reference count of @clusterz by one. Set pointer to NULL.
  *
  */
 void
@@ -104,13 +103,13 @@ nc_cluster_redshift_clear (NcClusterRedshift **clusterz)
 
 /**
  * nc_cluster_redshift_impl:
- * @clusterz: FIXME.
+ * @clusterz: a #NcClusterRedshift
  *
  * FIXME
- * 
- * Returns: FIXME
+ *
+ * Returns: a #NcClusterRedshiftImpl
  */
-NcClusterRedshiftImpl 
+NcClusterRedshiftImpl
 nc_cluster_redshift_impl (NcClusterRedshift *clusterz)
 {
   return NC_CLUSTER_REDSHIFT_GET_CLASS (clusterz)->impl;
@@ -118,11 +117,14 @@ nc_cluster_redshift_impl (NcClusterRedshift *clusterz)
 
 /**
  * nc_cluster_redshift_obs_len:
- * @clusterz: FIXME.
+ * @clusterz: a #NcClusterRedshift
  *
- * FIXME
+ * The number of observable redshifts of each cluster, e.g.,
+ * 1 - only photometric redshift,
+ * 1 - only spectroscopic redshift,
+ * 2 - both photometric and spectroscopic redshifts.
  *
- * Returns: FIXME
+ * Returns: The number of observable redshifts.
  */
 guint
 nc_cluster_redshift_obs_len (NcClusterRedshift *clusterz)
@@ -132,11 +134,12 @@ nc_cluster_redshift_obs_len (NcClusterRedshift *clusterz)
 
 /**
  * nc_cluster_redshift_obs_params_len:
- * @clusterz: FIXME.
+ * @clusterz: a #NcClusterRedshift
  *
- * FIXME
+ * The number of parameters related to the observable redshifts of each cluster, e.g.,
+ * 1 - measured error of the photometric redshift.
  *
- * Returns: FIXME
+ * Returns: The number of parameters related to the observable redshifts.
  */
 guint
 nc_cluster_redshift_obs_params_len (NcClusterRedshift *clusterz)
@@ -146,15 +149,16 @@ nc_cluster_redshift_obs_params_len (NcClusterRedshift *clusterz)
 
 /**
  * nc_cluster_redshift_p:
- * @clusterz: a #NcClusterRedshift.
- * @z: true redshift.
- * @lnM: true mass.
- * @z_obs: observed redshift.
- * @z_obs_params: observed redshift params.
+ * @clusterz: a #NcClusterRedshift
+ * @z: true redshift
+ * @lnM: true mass
+ * @z_obs: measured redshift
+ * @z_obs_params: measured redshift params
  *
- * FIXME
+ * It computes the probability density function (pdf) of the cluster redshift distribution @clusterz
+ * given @z, @lnM and the measured redshit @z_obs and its parameter(s) @z_obs_params.
  *
- * Returns: FIXME
+ * Returns: The pdf of @clusterz.
 */
 gdouble
 nc_cluster_redshift_p (NcClusterRedshift *clusterz, gdouble lnM, gdouble z, gdouble *z_obs, gdouble *z_obs_params)
@@ -164,13 +168,16 @@ nc_cluster_redshift_p (NcClusterRedshift *clusterz, gdouble lnM, gdouble z, gdou
 
 /**
  * nc_cluster_redshift_intp:
- * @clusterz: a #NcClusterRedshift.
- * @z: true redshift.
- * @lnM: true mass.
+ * @clusterz: a #NcClusterRedshift
+ * @z: true redshift
+ * @lnM: true mass
  *
- * FIXME
+ * It computes the @clusterz probability distribution of @z lying
+ * in the range $[z^{obs}_{min}, z^{obs}_{max}]$, namely,
+ * $$ intp = \int_{z^{obs}_{min}}^{z^{obs}_{max}} p \, dz^{obs},$$
+ * where $p$ is [nc_cluster_redshift_p()].
  *
- * Returns: FIXME
+ * Returns: The probability distribution of @z lying within $[z^{obs}_{min}, z^{obs}_{max}]$.
  */
 gdouble
 nc_cluster_redshift_intp (NcClusterRedshift *clusterz, gdouble lnM, gdouble z)
@@ -180,15 +187,14 @@ nc_cluster_redshift_intp (NcClusterRedshift *clusterz, gdouble lnM, gdouble z)
 
 /**
  * nc_cluster_redshift_resample:
- * @clusterz: a #NcClusterRedshift.
- * @z: true redshift.
- * @lnM: true mass.
- * @z_obs: (out): observed redshift.
- * @z_obs_params: (out): observed redshift params.
-   @rng: a #NcmRNG.
+ * @clusterz: a #NcClusterRedshift
+ * @z: true redshift
+ * @lnM: true mass
+ * @z_obs: (out): observed redshift
+ * @z_obs_params: (out): observed redshift params
+   @rng: a #NcmRNG
  *
  * FIXME
- * The function which will call this one is responsible to allocate enough memory for @z_lower and @z_upper.
  *
  * Returns: FIXME
  */
@@ -200,11 +206,11 @@ nc_cluster_redshift_resample (NcClusterRedshift *clusterz, gdouble lnM, gdouble
 
 /**
  * nc_cluster_redshift_p_limits:
- * @clusterz: a #NcClusterRedshift.
- * @z_obs: observed redshift.
- * @z_obs_params: observed redshift params.
- * @z_lower: (out): pointer to the lower limit of the true redshift integration.
- * @z_upper: (out): pointer to the upper limit of the true redshift integration.
+ * @clusterz: a #NcClusterRedshift
+ * @z_obs: observed redshift
+ * @z_obs_params: observed redshift params
+ * @z_lower: (out): pointer to the lower limit of the true redshift integration
+ * @z_upper: (out): pointer to the upper limit of the true redshift integration
  *
  * FIXME
  * The function which will call this one is responsible to allocate memory for @z_lower and @z_upper.
@@ -217,9 +223,9 @@ nc_cluster_redshift_p_limits (NcClusterRedshift *clusterz, gdouble *z_obs, gdoub
 
 /**
  * nc_cluster_redshift_n_limits:
- * @clusterz: a #NcClusterRedshift.
- * @z_lower: (out): pointer to the lower limit of the true redshift.
- * @z_upper: (out): pointer to the upper limit of the true redshift.
+ * @clusterz: a #NcClusterRedshift
+ * @z_lower: (out): pointer to the lower limit of the true redshift
+ * @z_upper: (out): pointer to the upper limit of the true redshift
  *
  * FIXME
  * The function which will call this one is responsible to allocate memory for @z_lower and @z_upper.
@@ -230,6 +236,40 @@ nc_cluster_redshift_n_limits (NcClusterRedshift *clusterz, gdouble *z_lower, gdo
   NC_CLUSTER_REDSHIFT_GET_CLASS (clusterz)->N_limits (clusterz, z_lower, z_upper);
 }
 
+static void
+_nc_cluster_redshift_log_all_models_go (GType model_type, guint n)
+{
+  guint nc, i, j;
+  GType *models = g_type_children (model_type, &nc);
+  for (i = 0; i < nc; i++)
+  {
+    guint ncc;
+    GType *modelsc = g_type_children (models[i], &ncc);
+
+    g_message ("#  ");
+    for (j = 0; j < n; j++) g_message (" ");
+    g_message ("%s\n", g_type_name (models[i]));
+    if (ncc)
+      _nc_cluster_redshift_log_all_models_go (models[i], n + 2);
+
+    g_free (modelsc);
+  }
+  g_free (models);
+}
+
+/**
+ * nc_cluster_redshift_log_all_models:
+ *
+ * This function lists all implemented models of cluster redshift distributions.
+ *
+ */
+void
+nc_cluster_redshift_log_all_models (void)
+{
+  g_message ("# Registred NcClusterRedshift:%s are:\n", g_type_name (NC_TYPE_CLUSTER_REDSHIFT));
+  _nc_cluster_redshift_log_all_models_go (NC_TYPE_CLUSTER_REDSHIFT, 0);
+}
+
 static void
 nc_cluster_redshift_init (NcClusterRedshift *nc_cluster_redshift)
 {
@@ -244,12 +284,23 @@ _nc_cluster_redshift_finalize (GObject *object)
   G_OBJECT_CLASS (nc_cluster_redshift_parent_class)->finalize (object);
 }
 
+NCM_MSET_MODEL_REGISTER_ID (nc_cluster_redshift, NC_TYPE_CLUSTER_REDSHIFT);
+
 static void
 nc_cluster_redshift_class_init (NcClusterRedshiftClass *klass)
 {
   GObjectClass* object_class = G_OBJECT_CLASS (klass);
+  NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
   //GObjectClass* parent_class = G_OBJECT_CLASS (klass);
 
   object_class->finalize = _nc_cluster_redshift_finalize;
-}
 
+  ncm_model_class_set_name_nick (model_class, "Cluster redshift abstract class", "NcClusterRedshift");
+  ncm_model_class_add_params (model_class, 0, 0, 1);
+
+  ncm_mset_model_register_id (model_class,
+                              "NcClusterRedshift",
+                              "Cluster redshift observable models.",
+                              NULL);
+  ncm_model_class_check_params_info (NCM_MODEL_CLASS (klass));
+}
