@@ -24,10 +24,11 @@
 
 /**
  * SECTION:nc_mass_function
- * @title: Mass Function
- * @short_description: FIXME
+ * @title: NcMassFunction
+ * @short_description: Clusters mass function.
  *
  * FIXME
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,88 +51,24 @@ enum
   PROP_GROWTH,
   PROP_MULTIPLICITY,
   PROP_AREA,
+  PROP_PREC,
 };
 
 G_DEFINE_TYPE (NcMassFunction, nc_mass_function, G_TYPE_OBJECT);
 
-/**
- * nc_mass_function_new:
- * @dist: a #NcDistance sets to #NcMassFunction:distance.
- * @vp: a #NcMatterVar sets to #NcMassFunction:variance.
- * @gf: a #NcGrowthFunc sets to #NcMassFunction:growth.
- * @mulf: a #NcMultiplicityFunc sets to #NcMassFunction:multiplicity.
- *
- * This function allocates memory for a new #NcMassFunction object and sets its properties to the values from
- * the input arguments.
- *
- * Returns: A new #NcMassFunction.
- */
-NcMassFunction *
-nc_mass_function_new (NcDistance *dist, NcMatterVar *vp, NcGrowthFunc *gf, NcMultiplicityFunc *mulf)
-{
-  NcMassFunction *mfp = g_object_new (NC_TYPE_MASS_FUNCTION,
-                                      "distance", dist,
-                                      "variance", vp,
-                                      "growth", gf,
-                                      "multiplicity", mulf,
-                                      NULL);
-  return mfp;
-}
-
-/**
- * nc_mass_function_copy:
- * @mfp: a #NcMassFunction.
- *
- * This function duplicates the #NcMassFunction object setting the same values of the original propertities.
- *
- * Returns: (transfer full): A new #NcMassFunction.
-   */
-NcMassFunction *
-nc_mass_function_copy (NcMassFunction *mfp)
-{
-  return nc_mass_function_new (mfp->dist, mfp->vp, mfp->gf, mfp->mulf);
-}
-
-/**
- * nc_mass_function_free:
- * @mfp: a #NcMassFunction.
- *
- * Atomically decrements the reference count of @mfp by one. If the reference count drops to 0,
- * all memory allocated by @mfp is released.
- *
- */
-void
-nc_mass_function_free (NcMassFunction *mfp)
-{
-  g_object_unref (mfp);
-}
-
-/**
- * nc_mass_function_clear:
- * @mfp: a #NcMassFunction.
- *
- * Atomically decrements the reference count of @mfp by one. If the reference count drops to 0,
- * all memory allocated by @mfp is released. Set pointer to NULL.
- *
- */
-void
-nc_mass_function_clear (NcMassFunction **mfp)
-{
-  g_clear_object (mfp);
-}
-
 static void
 nc_mass_function_init (NcMassFunction *mfp)
 {
-  mfp->lnMi = 0.0;
-  mfp->lnMf = 0.0;
-  mfp->zi = 0.0;
-  mfp->zf = 0.0;
+  mfp->lnMi        = 0.0;
+  mfp->lnMf        = 0.0;
+  mfp->zi          = 0.0;
+  mfp->zf          = 0.0;
   mfp->area_survey = 0.0;
-  mfp->N_sigma = 0.0;
-  mfp->growth = 0.0;
-  mfp->d2NdzdlnM = NULL;
-  mfp->ctrl = ncm_model_ctrl_new (NULL);
+  mfp->N_sigma     = 0.0;
+  mfp->growth      = 0.0;
+  mfp->d2NdzdlnM   = NULL;
+  mfp->prec        = 0.0;
+  mfp->ctrl        = ncm_model_ctrl_new (NULL);
 }
 
 static void
@@ -181,6 +118,9 @@ _nc_mass_function_set_property (GObject *object, guint prop_id, const GValue *va
     case PROP_AREA:
       nc_mass_function_set_area (mfp, g_value_get_double (value));
       break;
+    case PROP_PREC:
+      nc_mass_function_set_prec (mfp, g_value_get_double (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -210,6 +150,9 @@ _nc_mass_function_get_property (GObject *object, guint prop_id, GValue *value, G
     case PROP_AREA:
       g_value_set_double (value, mfp->area_survey);
       break;
+    case PROP_PREC:
+      g_value_set_double (value, mfp->prec);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -295,20 +238,100 @@ nc_mass_function_class_init (NcMassFunctionClass *klass)
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME
                                                         | G_PARAM_STATIC_BLURB));
 
+  /**
+   * NcMassFunction:prec:
+   *
+   * This property sets the precision.
+   */
+  g_object_class_install_property (object_class,
+                                   PROP_PREC,
+                                   g_param_spec_double ("prec",
+                                                        NULL,
+                                                        "Precision",
+                                                        1.0e-15, 1.0, 1.0e-6,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME
+                                                        | G_PARAM_STATIC_BLURB));
+}
 
+
+/**
+ * nc_mass_function_new:
+ * @dist: a #NcDistance sets to #NcMassFunction:distance
+ * @vp: a #NcMatterVar sets to #NcMassFunction:variance
+ * @gf: a #NcGrowthFunc sets to #NcMassFunction:growth
+ * @mulf: a #NcMultiplicityFunc sets to #NcMassFunction:multiplicity
+ *
+ * This function allocates memory for a new #NcMassFunction object and sets its properties to the values from
+ * the input arguments.
+ *
+ * Returns: A new #NcMassFunction.
+ */
+NcMassFunction *
+nc_mass_function_new (NcDistance *dist, NcMatterVar *vp, NcGrowthFunc *gf, NcMultiplicityFunc *mulf)
+{
+  NcMassFunction *mfp = g_object_new (NC_TYPE_MASS_FUNCTION,
+                                      "distance", dist,
+                                      "variance", vp,
+                                      "growth", gf,
+                                      "multiplicity", mulf,
+                                      NULL);
+  return mfp;
 }
 
 /**
- * nc_mass_function_dn_dlnm:
- * @mfp: a #NcMassFunction.
- * @model: a #NcHICosmo.
- * @lnM: logarithm base e of mass.
- * @z: redshift.
+ * nc_mass_function_copy:
+ * @mfp: a #NcMassFunction
  *
- * This function computes the comoving number density of dark matter halos at redshift z and
+ * This function duplicates the #NcMassFunction object setting the same values of the original propertities.
+ *
+ * Returns: (transfer full): A new #NcMassFunction.
+   */
+NcMassFunction *
+nc_mass_function_copy (NcMassFunction *mfp)
+{
+  return nc_mass_function_new (mfp->dist, mfp->vp, mfp->gf, mfp->mulf);
+}
+
+/**
+ * nc_mass_function_free:
+ * @mfp: a #NcMassFunction
+ *
+ * Atomically decrements the reference count of @mfp by one. If the reference count drops to 0,
+ * all memory allocated by @mfp is released.
+ *
+ */
+void
+nc_mass_function_free (NcMassFunction *mfp)
+{
+  g_object_unref (mfp);
+}
+
+/**
+ * nc_mass_function_clear:
+ * @mfp: a #NcMassFunction
+ *
+ * Atomically decrements the reference count of @mfp by one. If the reference count drops to 0,
+ * all memory allocated by @mfp is released. Set pointer to NULL.
+ *
+ */
+void
+nc_mass_function_clear (NcMassFunction **mfp)
+{
+  g_clear_object (mfp);
+}
+
+
+/**
+ * nc_mass_function_dn_dlnm:
+ * @mfp: a #NcMassFunction
+ * @model: a #NcHICosmo
+ * @lnM: logarithm base e of mass
+ * @z: redshift
+ *
+ * This function computes the comoving number density of dark matter halos at redshift @z and
  * mass M.
  *
- * Returns: dn/dlnM.
+ * Returns: $\frac{dn}{dlnM}$
  */
 gdouble
 nc_mass_function_dn_dlnm (NcMassFunction *mfp, NcHICosmo *model, gdouble lnM, gdouble z)
@@ -320,12 +343,9 @@ nc_mass_function_dn_dlnm (NcMassFunction *mfp, NcHICosmo *model, gdouble lnM, gd
   gdouble dlnvar0_dlnR = nc_matter_var_dlnvar0_dlnR (mfp->vp, model, lnR);
   gdouble growth = nc_growth_func_eval (mfp->gf, model, z);
   gdouble sigma = nc_matter_var_sigma8_sqrtvar0 (mfp->vp, model) * sqrt (var0) * growth;
-  //gdouble sigma = sqrt(var0) * growth;
   gdouble f = nc_multiplicity_func_eval (mfp->mulf, model, sigma, z);
   gdouble dn_dlnM = -(1.0 / (3.0 * M_rho)) * f * 0.5 * dlnvar0_dlnR;	/* dn/dlnM = - (\rho/3M) * f * (R/\sigma)* dsigma_dR */
-
-  //printf ("% 20.15g f = % 20.15g % 20.15g\n", -(1.0 / (3.0 * M_rho)), f, 0.5 * dlnvar0_dlnR);
-
+  
   return dn_dlnM;
 }
 
@@ -333,15 +353,15 @@ nc_mass_function_dn_dlnm (NcMassFunction *mfp, NcHICosmo *model, gdouble lnM, gd
 /* coloca os valores de dn/dm e sigma nos enderecos apontados pelos ponteiros */
 /**
  * nc_mass_function_sigma:
- * @mfp: a #NcMassFunction.
- * @model: a #NcHICosmo.
- * @lnM: logarithm base e of mass.
- * @z: redshift.
- * @dn_dlnM_ptr: pointer to comoving number density of halos.
- * @sigma_ptr: pointer to the standard deviation of the density contrast.
+ * @mfp: a #NcMassFunction
+ * @model: a #NcHICosmo
+ * @lnM: logarithm base e of mass
+ * @z: redshift
+ * @dn_dlnM_ptr: pointer to comoving number density of halos
+ * @sigma_ptr: pointer to the standard deviation of the density contrast
  *
  * This function computes the standard deviation of density contrast of the matter fluctuations and
- * the the comoving number density of dark matter halos at redshift z and mass M.
+ * the the comoving number density of dark matter halos at redshift @z and mass M.
  * These values are stored in @sigma_ptr and @dn_dlnM_ptr, respectively.
  *
  */
@@ -366,9 +386,9 @@ nc_mass_function_sigma (NcMassFunction *mfp, NcHICosmo *model, gdouble lnM, gdou
 /* A funcao abaixo foi feita para compararmos o nosso calculo com o resultado (equacao 9) do artigo astro-ph/0110246.*/
 /**
  * nc_mass_function_alpha_eff:
- * @vp: a #NcMatterVar.
- * @model: a #NcHICosmo.
- * @lnM: logarithm base e of mass.
+ * @vp: a #NcMatterVar
+ * @model: a #NcHICosmo
+ * @lnM: logarithm base e of mass
  * @a_eff_ptr: FIXME
  *
  * FIXME
@@ -422,10 +442,10 @@ nc_mass_function_cluster_abundance_integrand (gdouble R, gpointer params)
 
 /**
  * nc_mass_function_dcluster_abundance_dv:
- * @mfp: a #NcMassFunction.
- * @model: a #NcHICosmo.
- * @M: mass in units of h^{-1} * M_sun.
- * @z: redshift.
+ * @mfp: a #NcMassFunction
+ * @model: a #NcHICosmo
+ * @M: mass in units of h^{-1} * M_sun
+ * @z: redshift
  *
  * FIXME
  *
@@ -469,11 +489,11 @@ nc_mass_function_dn_m_to_inf_dv (NcMassFunction *mfp, NcHICosmo *model, gdouble
 
 /**
  * nc_mass_function_dn_m1_to_m2_dv:
- * @mfp: a #NcMassFunction.
- * @model: a #NcHICosmo.
- * @M1: mass in units of h^{-1} M_sun - lower limit of integration.
- * @M2: mass in units of h^{-1} M_sun- upper limit of integration.
- * @z: redshift.
+ * @mfp: a #NcMassFunction
+ * @model: a #NcHICosmo
+ * @M1: mass in units of h^{-1} M_sun - lower limit of integration
+ * @M2: mass in units of h^{-1} M_sun- upper limit of integration
+ * @z: redshift
  *
  * FIXME
  *
@@ -503,19 +523,21 @@ nc_mass_function_dn_m1_to_m2_dv (NcMassFunction *mfp, NcHICosmo *model, gdouble
 
   gsl_integration_qag (&F, R1, R2, 0.0, NCM_DEFAULT_PRECISION, NCM_INTEGRAL_PARTITION, 1, w, &n, &error);
 
-  printf ("R1 = %5.5g R2 = %5.5g z = %5.5g n = %5.5g\n", R1, R2, z, n);
+  /*printf ("R1 = %5.5g R2 = %5.5g z = %5.5g n = %5.5g\n", R1, R2, z, n);*/
   return n;
 }
 
 /**
  * nc_mass_function_dv_dzdomega:
- * @mfp: a #NcMassFunction.
- * @model: a #NcHICosmo.
- * @z: redshift.
+ * @mfp: a #NcMassFunction
+ * @model: a #NcHICosmo
+ * @z: redshift
  *
- * FIXME
+ * This function computes the comoving volume (flat universe) element per unit solid angle $d\Omega$ 
+ * given @z, namely, $$\frac{d^2V}{dzd\Omega} = \frac{c}{H(z)} D_c^2(z),$$
+ * where $H(z)$ is the Hubble function and $D_c$ is the comoving distance.
  *
- * Returns: FIXME
+ * Returns: comoving volume element $d^2V / dzd\Omega$.
  */
 gdouble
 nc_mass_function_dv_dzdomega (NcMassFunction *mfp, NcHICosmo *model, gdouble z)
@@ -530,8 +552,8 @@ static void _nc_mass_function_generate_2Dspline_knots (NcMassFunction *mfp, NcHI
 
 /**
  * nc_mass_function_set_area:
- * @mfp: a #NcMassFunction.
- * @area: area in steradian.
+ * @mfp: a #NcMassFunction
+ * @area: area in steradian
  *
  * FIXME
  *
@@ -546,10 +568,28 @@ nc_mass_function_set_area (NcMassFunction *mfp, gdouble area)
   }
 }
 
+/**
+ * nc_mass_function_set_prec:
+ * @mfp: a #NcMassFunction
+ * @prec: precision
+ *
+ * FIXME
+ *
+ */
+void
+nc_mass_function_set_prec (NcMassFunction *mfp, gdouble prec)
+{
+  if (mfp->prec != prec)
+  {
+    mfp->prec = prec;
+    ncm_spline2d_clear (&mfp->d2NdzdlnM);
+  }
+}
+
 /**
  * nc_mass_function_set_area_sd:
- * @mfp: a #NcMassFunction.
- * @area_sd: area in square degree.
+ * @mfp: a #NcMassFunction
+ * @area_sd: area in square degree
  *
  * FIXME
  *
@@ -563,12 +603,12 @@ nc_mass_function_set_area_sd (NcMassFunction *mfp, gdouble area_sd)
 
 /**
  * nc_mass_function_set_eval_limits:
- * @mfp: a #NcMassFunction.
- * @model: a #NcHICosmo.
- * @lnMi: minimum logarithm base e of mass.
- * @lnMf: maximum logarithm base e of mass.
- * @zi: minimum redshift.
- * @zf: maximum redshift.
+ * @mfp: a #NcMassFunction
+ * @model: a #NcHICosmo
+ * @lnMi: minimum logarithm base e of mass
+ * @lnMf: maximum logarithm base e of mass
+ * @zi: minimum redshift
+ * @zf: maximum redshift
  *
  * FIXME
  *
@@ -586,13 +626,13 @@ nc_mass_function_set_eval_limits (NcMassFunction *mfp, NcHICosmo *model, gdouble
   }
 
   if (mfp->d2NdzdlnM == NULL)
-    _nc_mass_function_generate_2Dspline_knots (mfp, model, 1e-5);
+    _nc_mass_function_generate_2Dspline_knots (mfp, model, mfp->prec);
 }
 
 /**
  * nc_mass_function_prepare:
- * @mfp: a #NcMassFunction.
- * @model: a #NcHICosmo.
+ * @mfp: a #NcMassFunction
+ * @model: a #NcHICosmo
  *
  * FIXME
  *
@@ -620,7 +660,7 @@ nc_mass_function_prepare (NcMassFunction *mfp, NcHICosmo *model)
       const gdouble lnM = ncm_vector_get (D2NDZDLNM_LNM (mfp), j);
       const gdouble d2NdzdlnM_ij = (dVdz * nc_mass_function_dn_dlnm (mfp, model, lnM, z));
       ncm_matrix_set (D2NDZDLNM_VAL (mfp), i, j, d2NdzdlnM_ij);
-      //printf ("prep matrix % 20.15g % 20.15g % 20.15g\n", z, lnM, d2NdzdlnM_ij);
+      /*printf ("prep matrix % 20.15g % 20.15g % 20.15g\n", z, lnM, d2NdzdlnM_ij);*/
     }
   }
   ncm_spline2d_prepare (mfp->d2NdzdlnM);
@@ -630,12 +670,12 @@ nc_mass_function_prepare (NcMassFunction *mfp, NcHICosmo *model)
 
 /**
  * nc_mass_function_dn_dz:
- * @mfp: a #NcMassFunction.
- * @model: a #NcHICosmo.
- * @lnMl: logarithm base e of mass, lower threshold.
- * @lnMu: logarithm base e of mass, upper threshold.
- * @z: redshift.
- * @spline: Whenever to create an intermediary spline of the mass integration.
+ * @mfp: a #NcMassFunction
+ * @model: a #NcHICosmo
+ * @lnMl: logarithm base e of mass, lower threshold
+ * @lnMu: logarithm base e of mass, upper threshold
+ * @z: redshift
+ * @spline: whenever to create an intermediary spline of the mass integration
  *
  * FIXME
  *
@@ -658,13 +698,13 @@ nc_mass_function_dn_dz (NcMassFunction *mfp, NcHICosmo *model, gdouble lnMl, gdo
 
 /**
  * nc_mass_function_n:
- * @mfp: a #NcMassFunction.
- * @model: a #NcHICosmo.
- * @lnMl: logarithm base e of mass, lower threshold.
- * @lnMu: logarithm base e of mass, upper threshold.
- * @zl: minimum redshift.
- * @zu: maximum redshift.
- * @spline: Whenever to create an intermediary spline of the integration.
+ * @mfp: a #NcMassFunction
+ * @model: a #NcHICosmo
+ * @lnMl: logarithm base e of mass, lower threshold
+ * @lnMu: logarithm base e of mass, upper threshold
+ * @zl: minimum redshift
+ * @zu: maximum redshift
+ * @spline: whenever to create an intermediary spline of the integration
  *
  * FIXME
  *
@@ -713,12 +753,7 @@ _encapsulated_z (gdouble z, gpointer p)
     nc_mass_function_dv_dzdomega (args->mfp, args->model, z) *
     nc_mass_function_dn_dlnm (args->mfp, args->model, args->lnM, z);
 
-  //printf ("z %.5g lnM %.5g d2N % 20.15g mf % 20.15g dV % 20.15g\n", z, args->lnM, A, nc_mass_function_dn_dlnm (args->mfp, args->model, args->lnM, z), nc_mass_function_dv_dzdomega (args->mfp, args->model, z));
   return A;
-
-  //return args->mfp->area_survey *
-  //nc_dcomoving_volume_dzdomega (args->mfp, args->model, z) *
-  //nc_mass_function_dn_dlnm (args->mfp, args->model, args->lnM, z);
 }
 
 static gdouble
@@ -728,10 +763,7 @@ _encapsulated_lnM (gdouble lnM, gpointer p)
 
   gdouble A = args->dVdz * nc_mass_function_dn_dlnm (args->mfp, args->model, lnM, args->z);
 
-  //printf ("2 z %.5g lnM %.5g d2N % 20.15g\n", args->z, lnM, A);
   return A;
-
-  //return args->dVdz * nc_mass_function_dn_dlnm (args->mfp, args->model, lnM, args->z);
 }
 
 static void
@@ -760,18 +792,18 @@ _nc_mass_function_generate_2Dspline_knots (NcMassFunction *mfp, NcHICosmo *model
 
 /**
  * nc_mass_function_prepare_if_needed:
- * @mfp: a #NcMassFunction.
- * @model: a #NcHICosmo.
+ * @mfp: a #NcMassFunction
+ * @model: a #NcHICosmo
  *
  * FIXME
  *
  */
 /**
  * nc_mass_function_d2n_dzdlnm:
- * @mfp: a #NcMassFunction.
- * @model: a #NcHICosmo.
- * @lnM: logarithm base e of mass.
- * @z: redshift.
+ * @mfp: a #NcMassFunction
+ * @model: a #NcHICosmo
+ * @lnM: logarithm base e of mass
+ * @z: redshift
  *
  * FIXME
  *
