@@ -45,12 +45,6 @@ G_BEGIN_DECLS
 typedef struct _NcMassFunctionClass NcMassFunctionClass;
 typedef struct _NcMassFunction NcMassFunction;
 
-struct _NcMassFunctionClass
-{
-  /*< private > */
-  GObjectClass parent_class;
-};
-
 struct _NcMassFunction
 {
   /*< private > */
@@ -67,9 +61,16 @@ struct _NcMassFunction
   gdouble lnMf;
   gdouble zi;
   gdouble zf;
+  gdouble prec;
   NcmModelCtrl *ctrl;
 };
 
+struct _NcMassFunctionClass
+{
+  /*< private > */
+  GObjectClass parent_class;
+};
+
 /**
  * NcMassFunctionSplineOptimize:
  * @NC_MASS_FUNCTION_SPLINE_NONE: FIXME
@@ -94,6 +95,7 @@ void nc_mass_function_free (NcMassFunction *mfp);
 void nc_mass_function_clear (NcMassFunction **mfp);
 
 void nc_mass_function_set_area (NcMassFunction *mfp, gdouble area);
+void nc_mass_function_set_prec (NcMassFunction *mfp, gdouble prec);
 void nc_mass_function_set_area_sd (NcMassFunction *mfp, gdouble area_sd);
 void nc_mass_function_set_eval_limits (NcMassFunction *mfp, NcHICosmo *model, gdouble lnMi, gdouble lnMf, gdouble zi, gdouble zf);
 void nc_mass_function_prepare (NcMassFunction *mfp, NcHICosmo *model);
@@ -125,7 +127,7 @@ G_INLINE_FUNC void
 nc_mass_function_prepare_if_needed (NcMassFunction *mfp, NcHICosmo *model)
 {
   if (ncm_model_ctrl_update (mfp->ctrl, NCM_MODEL (model)))
-	nc_mass_function_prepare (mfp, model);
+	  nc_mass_function_prepare (mfp, model);
 }
 
 G_INLINE_FUNC gdouble
