@@ -25,10 +25,11 @@
 
 /**
  * SECTION:nc_matter_var
- * @title: Matter Fluctuation Variance
- * @short_description: FIXME
+ * @title: NcMatterVar
+ * @short_description: Matter perturbations variance.
  *
  * FIXME
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -153,6 +154,8 @@ nc_matter_var_init (NcMatterVar *vp)
   ncm_vector_free (dsigma2_over_growth_xv);
   ncm_vector_free (dsigma2_over_growth_yv);
 
+  vp->fftlog = NULL;
+  
   vp->ctrl = ncm_model_ctrl_new (NULL);
 }
 
@@ -282,7 +285,9 @@ nc_matter_var_class_init (NcMatterVarClass *klass)
 
 static void _nc_matter_var_prepare_numint (NcMatterVar *vp, NcHICosmo *model);
 static void _nc_matter_var_prepare_splineint (NcMatterVar *vp, NcHICosmo *model);
+#ifdef NUMCOSMO_HAVE_FFTW3
 static void _nc_matter_var_prepare_fft (NcMatterVar *vp, NcHICosmo *model);
+#endif /* NUMCOSMO_HAVE_FFTW3 */
 
 /**
  * nc_matter_var_prepare:
@@ -870,15 +875,38 @@ _nc_matter_var_prepare_splineint (NcMatterVar *vp, NcHICosmo *model)
 }
 
 #ifdef NUMCOSMO_HAVE_FFTW3
+
+typedef struct _NcMatterVarFftlog
+{
+  NcMatterVar *vp;
+  NcHICosmo *model;
+} NcMatterVarFftlog;
+
+/*
+static gdouble 
+_powspec_k2 (gdouble k, gpointer userdata)
+{
+  NcMatterVarFftlog *arg = (NcMatterVarFftlog *) userdata;
+  const gdouble c1 = 2.0 * M_PI * M_PI;
+  const gdouble k2 = k * k;
+  const gdouble matter_P = nc_transfer_func_matter_powerspectrum (arg->vp->tf, arg->model, k);
+  const gdouble f = matter_P * k2 / c1;
+  return f;
+}
+*/
+
 static void
 _nc_matter_var_prepare_fft (NcMatterVar *vp, NcHICosmo *model)
 {
+  _NCM_STATIC_MUTEX_DECL (prepare_fft_lock);
+  _NCM_MUTEX_LOCK (&prepare_fft_lock);
+  {
   const gint N = 500 + 1;
   const gint N_2 = 250;
   const gdouble L = 17.0 * M_LN10;
   const gdouble dr = L / (N * 1.0);
-  const gdouble r0 = 3.0 * M_LN10;
-  const gdouble lnk0 = -3.0 * M_LN10;
+  const gdouble r0 = 3.0 * M_LN10; 
+  const gdouble lnk0 = -3.0 * M_LN10; 
   static fftw_complex *in = NULL;
   static fftw_complex *out = NULL;
   static fftw_complex *u = NULL;
@@ -886,10 +914,10 @@ _nc_matter_var_prepare_fft (NcMatterVar *vp, NcHICosmo *model)
   static fftw_complex *Rdsigma2_dr = NULL;
   static fftw_complex *du = NULL;
   static fftw_plan p, p_Rsigma2, p_Rdsigma2_dr;
-  static gdouble planned = FALSE;
-  static gdouble calc_u = FALSE;
+  static gboolean planned = FALSE;
+  static gboolean calc_u = FALSE;
   gint i, j;
-
+  
   //fprintf (stderr, "# %d % 20.15g [% 20.15g]\n", N, dr, L);
   if (!planned)
   {
@@ -902,9 +930,9 @@ _nc_matter_var_prepare_fft (NcMatterVar *vp, NcHICosmo *model)
     du = (fftw_complex *) fftw_malloc (sizeof(fftw_complex) * N);
 
     ncm_cfg_load_fftw_wisdom ("nc_matter_var_wisdown.fftw3");
-    p = fftw_plan_dft_1d (N, in, out, FFTW_FORWARD, FFTW_PATIENT | FFTW_DESTROY_INPUT);
-    p_Rsigma2 = fftw_plan_dft_1d (N, out, in, FFTW_FORWARD, FFTW_PATIENT | FFTW_DESTROY_INPUT);
-    p_Rdsigma2_dr = fftw_plan_dft_1d (N, fftRdsigma2_dr, Rdsigma2_dr, FFTW_FORWARD, FFTW_PATIENT | FFTW_DESTROY_INPUT);
+    p = fftw_plan_dft_1d (N, in, out, FFTW_FORWARD, fftw_default_flags | FFTW_DESTROY_INPUT);
+    p_Rsigma2 = fftw_plan_dft_1d (N, out, in, FFTW_FORWARD, fftw_default_flags | FFTW_DESTROY_INPUT);
+    p_Rdsigma2_dr = fftw_plan_dft_1d (N, fftRdsigma2_dr, Rdsigma2_dr, FFTW_FORWARD, fftw_default_flags | FFTW_DESTROY_INPUT);
     ncm_cfg_save_fftw_wisdom ("nc_matter_var_wisdown.fftw3");
     
     planned = TRUE;
@@ -922,7 +950,7 @@ _nc_matter_var_prepare_fft (NcMatterVar *vp, NcHICosmo *model)
     in[ii] = f;
   }
   fftw_execute (p);
-
+ 
   if (!calc_u)
   {
     for (i = -N_2; i <= N_2; i++)
@@ -956,13 +984,17 @@ _nc_matter_var_prepare_fft (NcMatterVar *vp, NcHICosmo *model)
   fftw_execute (p_Rdsigma2_dr);
 
   j = 0;
-  for (i = -120; i < 30; i++)
+  
+  for (i = -120; i < 30; i++) 
   {
     const gint ii = (i < 0) ? i + N : i;
     const gdouble r = r0 + i * dr;
     const complex double Rsigma2 = in[ii] / N;
     const complex double lnsigma2 = log (Rsigma2) - r;
     const complex double dlnsigma2_dr = Rdsigma2_dr[ii] / (Rsigma2 * N);
+
+     //printf ("i = %d ii = %d lnr= %.5g lnsigma2 = % 20.15g dlnsigma2 = % 20.15g\n", i, ii, r, creal (lnsigma2), creal (dlnsigma2_dr));
+    
     /*
      const gdouble R = exp (r);
      const gdouble Rsigma2_old = R * nc_matter_var_over_growth2_tophat_old (vp, model, r);
@@ -970,16 +1002,75 @@ _nc_matter_var_prepare_fft (NcMatterVar *vp, NcHICosmo *model)
      const gdouble err_spline = fabs((Rsigma2_old - Rsigma2_spline) / Rsigma2_old);
      const gdouble err_fft = fabs((Rsigma2_old - creal(Rsigma2)) / Rsigma2_old);
      printf ("%d %d % 20.15g % 20.15g % 20.15g % 20.15g % 8.5e % 8.5e\n", i, ii, r / M_LN10, creal(Rsigma2), Rsigma2_spline, Rsigma2_old, err_fft, err_spline);
-     */
-    //printf ("%d %d % 20.15g % 20.15g % 20.15g\n", i, ii, r / M_LN10, creal(Rsigma2), creal(dlnsigma2_dr));
+    */
+/*    printf ("%d %d % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g\n", i, ii, r, creal (lnsigma2), creal (dlnsigma2_dr), 
+            ncm_vector_get (lnr_vec, ii),
+            ncm_vector_get (lnGr_vec, ii),
+            ncm_vector_get (dlnGr_dlnr_vec, ii)
+            );
+*/            
     ncm_vector_set (vp->sigma2_over_growth->xv, j, r);
     ncm_vector_set (vp->sigma2_over_growth->yv, j, lnsigma2);
     ncm_vector_set (vp->deriv_sigma2_over_growth->xv, j, r);
     ncm_vector_set (vp->deriv_sigma2_over_growth->yv, j, dlnsigma2_dr);
+
     j++;
   }
+
+  
+/*  
+  for (i = -N_2; i < N_2; i++)
+  {
+    const gint ii = (i < 0) ? i + N : i;
+    const gdouble r = r0 + i * dr;
+    const complex double Rsigma2 = in[ii] / N;
+    const complex double lnsigma2 = log (Rsigma2) - r;
+    const complex double dlnsigma2_dr = Rdsigma2_dr[ii] / (Rsigma2 * N);
+    printf ("Old: % 20.15g % 20.15g % 20.15g\n", r, creal (lnsigma2), creal (dlnsigma2_dr));
+  }
+  printf ("\n\n");
+ 
+  if (vp->fftlog == NULL)
+  {
+    gsl_function F;
+    NcMatterVarFftlog arg;
+    NcmVector *lnr_vec;
+    NcmVector *lnGr_vec;
+    NcmVector *dlnGr_dlnr_vec;
+    //vp->fftlog = ncm_fftlog_j1pow2_new (exp (-6.0 * M_LN10), exp (6.0 * M_LN10), 22.0 * M_LN10, 3500);
+    vp->fftlog = ncm_fftlog_j1pow2_new (exp (-6.0 * M_LN10), exp (6.0 * M_LN10), 24.0 * M_LN10, 3500);
+    arg.vp     = vp;
+    arg.model  = model;
+    F.function = &_powspec_k2;
+    F.params   = &arg;
+
+    ncm_fftlog_eval_by_function (vp->fftlog, &F);
+    
+    lnr_vec        = ncm_fftlog_get_vector_lnr (vp->fftlog);
+    lnGr_vec       = ncm_fftlog_get_vector_Gr (vp->fftlog, 0);
+    dlnGr_dlnr_vec = ncm_fftlog_get_vector_Gr (vp->fftlog, 1);
+
+    for (i = 0; i < ncm_vector_len (lnr_vec); i++)
+    {
+      printf ("New: % 20.15g % 20.15g % 20.15g\n", 
+              ncm_vector_get (lnr_vec, i),
+              ncm_vector_get (lnGr_vec, i),
+              ncm_vector_get (dlnGr_dlnr_vec, i));
+    }
+    printf ("\n\n");
+    ncm_vector_free (lnr_vec);
+    ncm_vector_free (lnGr_vec);
+    ncm_vector_free (dlnGr_dlnr_vec);
+  }
+ */
+
+  }
+  _NCM_MUTEX_UNLOCK (&prepare_fft_lock);
+
   ncm_spline_prepare (vp->sigma2_over_growth);
   ncm_spline_prepare (vp->deriv_sigma2_over_growth);
+
+
 }
 #endif /* NUMCOSMO_HAVE_FFTW3 */
 
@@ -1721,7 +1812,7 @@ nc_matter_var_dsigma0_dR (NcMatterVar *vp, NcHICosmo *model, gdouble lnR)
 /****************** Variance normalization with sigma8 ******************************************************************/
 
 /**
- * nc_matter_var_sigma_norma:
+ * nc_matter_var_sigma8_sqrtvar0:
  * @vp: a #NcMatterVar.
  * @model: a #NcHICosmo.
  *
@@ -1750,6 +1841,6 @@ nc_matter_var_sigma8_sqrtvar0 (NcMatterVar *vp, NcHICosmo *model)    /* eh calcu
     sigma0 = sqrt(nc_matter_var_var0 (vp, model, ln8));
 
   sigma0_norma = nc_hicosmo_sigma_8 (model) / sigma0;
-
+ 
   return sigma0_norma;
 }
