@@ -25,8 +25,8 @@
 
 /**
  * SECTION:nc_multiplicity_func_tinker_crit
- * @title: Critical Tinker Multiplicity Function
- * @short_description: Dark Matter Halo FIXME
+ * @title: NcMultiplicityFuncTinkerCrit
+ * @short_description: Dark matter halo -- Tinker multiplicity function critical density.
  *
  * FIXME
  */
@@ -88,99 +88,108 @@ _nc_multiplicity_func_tinker_crit_eval (NcMultiplicityFunc *mulf, NcHICosmo *mod
   const gdouble E2 = nc_hicosmo_E2 (model, z);
   const gdouble Delta_z = mulf_tinker_crit->Delta * E2 / (Omega_m * gsl_pow_3 (1.0 + z));
   const gdouble log10_Delta_z = log10 (Delta_z);
-  const gdouble log10_200 = log10 (200.0);
-  const gdouble log10_300 = log10 (300.0);
-  const gdouble log10_400 = log10 (400.0);
-  const gdouble log10_600 = log10 (600.0);
-  const gdouble log10_800 = log10 (800.0);
-  const gdouble log10_1200 = log10 (1200.0);
-  const gdouble log10_1600 = log10 (1600.0);
-  const gdouble log10_2400 = log10 (2400.0);
-  const gdouble log10_3200 = log10 (3200.0);
+/*
+  const gdouble log10_200  = 2.301029995663981195213738894724493026768; //log10 (200.0);
+  const gdouble log10_300  = 2.477121254719662437295027903255115309202; //log10 (300.0);
+  const gdouble log10_400  = 2.602059991327962390427477789448986053536; //log10 (400.0);
+  const gdouble log10_600  = 2.778151250383643632508766797979608335970; //log10 (600.0);
+  const gdouble log10_800  = 2.903089986991943585641216684173479080304; //log10 (800.0);
+  const gdouble log10_1200 = 3.079181246047624827722505692704101362737; //log10 (1200.0);
+  const gdouble log10_1600 = 3.204119982655924780854955578897972107072; //log10 (1600.0);
+  const gdouble log10_2400 = 3.380211241711606022936244587428594389505; //log10 (2400.0);
+  const gdouble log10_3200 = 3.505149978319905976068694473622465133840; //log10 (3200.0);
+*/
 
-  const gdouble coef_A[8][6] = {
-	{log10_200, log10_300, 0.186, 0.0, 0.2, 0.5},
-	{log10_300, log10_400, 0.2, 0.5, 0.212, -1.56},
-	{log10_400, log10_600, 0.212, -1.56, 0.218, 3.05},
-	{log10_600, log10_800, 0.218, 3.05, 0.248, -2.95},
-	{log10_800, log10_1200, 0.248, -2.95, 0.255, 1.07},
-	{log10_1200, log10_1600, 0.255, 1.07, 0.26, -0.71},
-	{log10_1600, log10_2400, 0.26, -0.71, 0.26, 0.21},
-	{log10_2400, log10_3200, 0.26, 0.21, 0.26, 0.0}};
-  const gdouble coef_a[8][6] = {
-	{log10_200, log10_300, 1.47, 0.0, 1.52, 1.19},
-	{log10_300, log10_400, 1.52, 1.19, 1.46, -6.34},
-	{log10_400, log10_600, 1.46, -6.34, 1.61, 21.36},
-	{log10_600, log10_800, 1.61, 21.36, 1.87, -10.95},
-	{log10_800, log10_1200, 1.87, -10.95, 2.13, 2.59},
-	{log10_1200, log10_1600, 2.13, 2.59, 2.3, -0.85},
-	{log10_1600, log10_2400, 2.3, -0.85, 2.53, -2.07},
-	{log10_2400, log10_3200, 2.53, -2.07, 2.66, 0.0}};
-  const gdouble coef_b[8][6] = {
-	{log10_200, log10_300, 2.57, 0.0, 2.25, -1.08},
-	{log10_300, log10_400, 2.25, -1.08, 2.05, 12.61},
-	{log10_400, log10_600, 2.05, 12.61, 1.87, -20.96},
-	{log10_600, log10_800, 1.87, -20.96, 1.59, 24.08},
-	{log10_800, log10_1200, 1.59, 24.08, 1.51, -6.64},
-	{log10_1200, log10_1600, 1.51, -6.64, 1.46, 3.84},
-	{log10_1600, log10_2400, 1.46, 3.84, 1.44, -2.09},
-	{log10_2400, log10_3200, 1.44, -2.09, 1.41, 0.0}};
-  const gdouble coef_c[8][6] = {
-	{log10_200, log10_300, 1.19, 0.0, 1.27, 0.94},
-	{log10_300, log10_400, 1.27, 0.94, 1.34, -0.43},
-	{log10_400, log10_600, 1.34, -0.43, 1.45, 4.61},
-	{log10_600, log10_800, 1.45, 4.61, 1.58, 0.01},
-	{log10_800, log10_1200, 1.58, 0.01, 1.8, 1.21},
-	{log10_1200, log10_1600, 1.8, 1.21, 1.97, 1.43},
-	{log10_1600, log10_2400, 1.97, 1.43, 2.24, 0.33},
-	{log10_2400, log10_3200, 2.24, 0.33, 2.44, 0.0}};
+#define log10_200  2.301029995663981195213738894724493026768 //log10 (200.0);
+#define log10_300  2.477121254719662437295027903255115309202 //log10 (300.0);
+#define log10_400  2.602059991327962390427477789448986053536 //log10 (400.0);
+#define log10_600  2.778151250383643632508766797979608335970 //log10 (600.0);
+#define log10_800  2.903089986991943585641216684173479080304 //log10 (800.0);
+#define log10_1200 3.079181246047624827722505692704101362737 //log10 (1200.0);
+#define log10_1600 3.204119982655924780854955578897972107072 //log10 (1600.0);
+#define log10_2400 3.380211241711606022936244587428594389505 //log10 (2400.0);
+#define log10_3200 3.505149978319905976068694473622465133840 //log10 (3200.0);
+  
+  static const gdouble coef_A[8][6] = {
+    {log10_200, log10_300, 0.186, 0.0, 0.2, 0.5},
+    {log10_300, log10_400, 0.2, 0.5, 0.212, -1.56},
+    {log10_400, log10_600, 0.212, -1.56, 0.218, 3.05},
+    {log10_600, log10_800, 0.218, 3.05, 0.248, -2.95},
+    {log10_800, log10_1200, 0.248, -2.95, 0.255, 1.07},
+    {log10_1200, log10_1600, 0.255, 1.07, 0.26, -0.71},
+    {log10_1600, log10_2400, 0.26, -0.71, 0.26, 0.21},
+    {log10_2400, log10_3200, 0.26, 0.21, 0.26, 0.0}};
+  static const gdouble coef_a[8][6] = {
+    {log10_200, log10_300, 1.47, 0.0, 1.52, 1.19},
+    {log10_300, log10_400, 1.52, 1.19, 1.46, -6.34},
+    {log10_400, log10_600, 1.46, -6.34, 1.61, 21.36},
+    {log10_600, log10_800, 1.61, 21.36, 1.87, -10.95},
+    {log10_800, log10_1200, 1.87, -10.95, 2.13, 2.59},
+    {log10_1200, log10_1600, 2.13, 2.59, 2.3, -0.85},
+    {log10_1600, log10_2400, 2.3, -0.85, 2.53, -2.07},
+    {log10_2400, log10_3200, 2.53, -2.07, 2.66, 0.0}};
+  static const gdouble coef_b[8][6] = {
+    {log10_200, log10_300, 2.57, 0.0, 2.25, -1.08},
+    {log10_300, log10_400, 2.25, -1.08, 2.05, 12.61},
+    {log10_400, log10_600, 2.05, 12.61, 1.87, -20.96},
+    {log10_600, log10_800, 1.87, -20.96, 1.59, 24.08},
+    {log10_800, log10_1200, 1.59, 24.08, 1.51, -6.64},
+    {log10_1200, log10_1600, 1.51, -6.64, 1.46, 3.84},
+    {log10_1600, log10_2400, 1.46, 3.84, 1.44, -2.09},
+    {log10_2400, log10_3200, 1.44, -2.09, 1.41, 0.0}};
+  static const gdouble coef_c[8][6] = {
+    {log10_200, log10_300, 1.19, 0.0, 1.27, 0.94},
+    {log10_300, log10_400, 1.27, 0.94, 1.34, -0.43},
+    {log10_400, log10_600, 1.34, -0.43, 1.45, 4.61},
+    {log10_600, log10_800, 1.45, 4.61, 1.58, 0.01},
+    {log10_800, log10_1200, 1.58, 0.01, 1.8, 1.21},
+    {log10_1200, log10_1600, 1.8, 1.21, 1.97, 1.43},
+    {log10_1600, log10_2400, 1.97, 1.43, 2.24, 0.33},
+    {log10_2400, log10_3200, 2.24, 0.33, 2.44, 0.0}};
   gint i;
   gdouble A0 = 0.0, a0 = 0.0, b0 = 0.0, c = 0.0;
 
   if (log10_Delta_z > log10_3200)
   {
-	const gdouble a0_fit_3200 = 1.43 + pow (log10_3200 - 2.3, 1.5);
-	const gdouble b0_fit_3200 = 1.0 + pow (log10_3200 - 1.6, -1.5);
-	const gdouble c_fit_3200 = 1.2 + pow (log10_3200 - 2.35, 1.6);
-	const gdouble A0_s_3200 = calc_polynomial (coef_A[7], log10_3200);
-	const gdouble a0_s_3200 = calc_polynomial (coef_a[7], log10_3200);
-	const gdouble b0_s_3200 = calc_polynomial (coef_b[7], log10_3200);
-	const gdouble c_s_3200  = calc_polynomial (coef_c[7], log10_3200);
+    const gdouble a0_fit_3200 = 1.43 + pow (log10_3200 - 2.3, 1.5);
+    const gdouble b0_fit_3200 = 1.0 + pow (log10_3200 - 1.6, -1.5);
+    const gdouble c_fit_3200 = 1.2 + pow (log10_3200 - 2.35, 1.6);
+    const gdouble A0_s_3200 = calc_polynomial (coef_A[7], log10_3200);
+    const gdouble a0_s_3200 = calc_polynomial (coef_a[7], log10_3200);
+    const gdouble b0_s_3200 = calc_polynomial (coef_b[7], log10_3200);
+    const gdouble c_s_3200  = calc_polynomial (coef_c[7], log10_3200);
 
-	A0 = A0_s_3200;
-	a0 = a0_s_3200 / a0_fit_3200 * (1.43 + pow (log10_Delta_z - 2.3, 1.5));
-	b0 = b0_s_3200 / b0_fit_3200 * (1.0 + pow (log10_3200 - 1.6, -1.5));
-	c = c_s_3200 / c_fit_3200 * (1.2 + pow (log10_3200 - 2.35, 1.6));
+    A0 = A0_s_3200;
+    a0 = a0_s_3200 / a0_fit_3200 * (1.43 + pow (log10_Delta_z - 2.3, 1.5));
+    b0 = b0_s_3200 / b0_fit_3200 * (1.0 + pow (log10_3200 - 1.6, -1.5));
+    c = c_s_3200 / c_fit_3200 * (1.2 + pow (log10_3200 - 2.35, 1.6));
   }
   else
   {
-	for (i = 0; i < 8; i++)
-	{
-	  if (log10_Delta_z >= coef_A[i][1])
-		continue;
-	  else 
-	  {
-		A0 = calc_polynomial (coef_A[i], log10_Delta_z);
-		a0 = calc_polynomial (coef_a[i], log10_Delta_z);
-		b0 = calc_polynomial (coef_b[i], log10_Delta_z);
-		c = calc_polynomial (coef_c[i], log10_Delta_z);
-		break;
-	  }
-	}
+    for (i = 0; i < 8; i++)
+    {
+      if (log10_Delta_z >= coef_A[i][1])
+        continue;
+      else 
+      {
+        A0 = calc_polynomial (coef_A[i], log10_Delta_z);
+        a0 = calc_polynomial (coef_a[i], log10_Delta_z);
+        b0 = calc_polynomial (coef_b[i], log10_Delta_z);
+        c = calc_polynomial (coef_c[i], log10_Delta_z);
+        break;
+      }
+    }
   }
-  
+
   {
-	const gdouble A = A0 * pow(1.0 + z, -0.14);
-	const gdouble a = a0 * pow(1.0 + z, -0.06);
-	const gdouble log10alpha = -pow(0.75 / log10 (Delta_z / 75.0), 1.2);
-	const gdouble alpha = pow(10.0, log10alpha);
-	const gdouble b = b0 * pow(1.0 + z, -alpha);
-	const gdouble f_Tinker_Delta_spline = A * (pow(sigma / b, -a) + 1.0) * exp(-c / (sigma * sigma));
+    const gdouble A          = A0 * pow(1.0 + z, -0.14);
+    const gdouble a          = a0 * pow(1.0 + z, -0.06);
+    const gdouble log10alpha = - pow (0.75 / log10 (Delta_z / 75.0), 1.2);
+    const gdouble alpha      = pow (10.0, log10alpha);
+    const gdouble b          = b0 * pow(1.0 + z, -alpha);
+    const gdouble f_Tinker_Delta_spline = A * (pow (sigma / b, -a) + 1.0) * exp(-c / (sigma * sigma));
 
-	//printf ("% 5.5g % 5.5g % 20.15g % 20.15g % 20.15g % 20.15g\n", z, Delta_z, A0, a0, b0, c);
-	//printf ("NEW % 5.5g % 5.5g % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g\n", z, Delta_z, sigma, A, a, b, c, f_Tinker_Delta_spline, Omega_m, E2);
-
-	return f_Tinker_Delta_spline;
+    return f_Tinker_Delta_spline;
   }
 }
 
@@ -214,15 +223,12 @@ nc_multiplicity_func_tinker_crit_get_Delta (const NcMultiplicityFuncTinkerCrit *
 static void
 nc_multiplicity_func_tinker_crit_init (NcMultiplicityFuncTinkerCrit *mulf_tinker_crit)
 {
-  /* TODO: Add initialization code here */
   mulf_tinker_crit->Delta = 200.0;
 }
 
 static void
 _nc_multiplicity_func_tinker_crit_finalize (GObject *object)
 {
-  /* TODO: Add deinitalization code here */
-
   G_OBJECT_CLASS (nc_multiplicity_func_tinker_crit_parent_class)->finalize (object);
 }
 
@@ -234,9 +240,9 @@ _nc_multiplicity_func_tinker_crit_set_property (GObject * object, guint prop_id,
 
   switch (prop_id)
   {
-	case PROP_DELTA:
-	  mulf_tinker_crit->Delta = g_value_get_double (value);
-	  break;
+    case PROP_DELTA:
+      nc_multiplicity_func_tinker_crit_set_Delta (mulf_tinker_crit, g_value_get_double (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -266,9 +272,9 @@ nc_multiplicity_func_tinker_crit_class_init (NcMultiplicityFuncTinkerCritClass *
 {
   GObjectClass* object_class = G_OBJECT_CLASS (klass);
   NcMultiplicityFuncClass* parent_class = NC_MULTIPLICITY_FUNC_CLASS (klass);
-  
+
   parent_class->eval = &_nc_multiplicity_func_tinker_crit_eval;
-  
+
   object_class->finalize = _nc_multiplicity_func_tinker_crit_finalize;
   object_class->set_property = _nc_multiplicity_func_tinker_crit_set_property;
   object_class->get_property = _nc_multiplicity_func_tinker_crit_get_property;
@@ -284,6 +290,6 @@ nc_multiplicity_func_tinker_crit_class_init (NcMultiplicityFuncTinkerCritClass *
                                                         NULL,
                                                         "Delta",
                                                         200.0, 3200.0, 200.0,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 }
 
