@@ -44,18 +44,18 @@ G_BEGIN_DECLS
 typedef struct _NcmCClass NcmCClass;
 typedef struct _NcmC NcmC;
 
-struct _NcmCClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-};
-
 struct _NcmC
 {
   /*< private >*/
   GObject parent_instance;
 };
 
+struct _NcmCClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+};
+
 GType ncm_c_get_type (void) G_GNUC_CONST;
 
 /*******************************************************************************
@@ -132,14 +132,14 @@ G_INLINE_FUNC gdouble ncm_c_HeI_Lyman_2s_wl3_8pi (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_HeI_Lyman_2p_wl3_8pi (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_H_reduced_mass (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_H_reduced_energy (void) G_GNUC_CONST;
-G_INLINE_FUNC gdouble ncm_c_H_bind (const gint n, const gint j) G_GNUC_CONST;
+G_INLINE_FUNC gdouble ncm_c_H_bind (const gdouble n, const gdouble j) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_H_bind_1s (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_H_bind_2s (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_H_bind_2p (void) G_GNUC_CONST;
-G_INLINE_FUNC gdouble ncm_c_H_Lyman_series (const gint n, const gint j) G_GNUC_CONST;
+G_INLINE_FUNC gdouble ncm_c_H_Lyman_series (const gdouble n, const gdouble j) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_H_Lyman_2s (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_H_Lyman_2p (void) G_GNUC_CONST;
-G_INLINE_FUNC gdouble ncm_c_H_Lyman_series_wl (const gint n, const gint j) G_GNUC_CONST;
+G_INLINE_FUNC gdouble ncm_c_H_Lyman_series_wl (const gdouble n, const gdouble j) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_H_Lyman_2s_wl (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_H_Lyman_2p_wl (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_H_Lyman_2s_wl3_8pi (void) G_GNUC_CONST;
@@ -184,15 +184,6 @@ G_INLINE_FUNC gdouble ncm_c_wmap5_coadded_I_Ka (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_wmap5_coadded_I_Q (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_wmap5_coadded_I_V (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_wmap5_coadded_I_W (void) G_GNUC_CONST;
-G_INLINE_FUNC gdouble ncm_c_bao_eisenstein_z (void) G_GNUC_CONST;
-G_INLINE_FUNC gdouble ncm_c_bao_eisenstein_A (void) G_GNUC_CONST;
-G_INLINE_FUNC gdouble ncm_c_bao_eisenstein_sigma_A (void) G_GNUC_CONST;
-G_INLINE_FUNC gdouble ncm_c_bao_eisenstein_DV (void) G_GNUC_CONST;
-G_INLINE_FUNC gdouble ncm_c_bao_eisenstein_sigma_DV (void) ;
-G_INLINE_FUNC gdouble ncm_c_bao_percival2007_DV_DV (void) G_GNUC_CONST;
-G_INLINE_FUNC gdouble ncm_c_bao_percival2007_sigma_DV_DV (void) G_GNUC_CONST;
-G_INLINE_FUNC gdouble ncm_c_bao_percival2010_DV_DV (void) G_GNUC_CONST;
-G_INLINE_FUNC gdouble ncm_c_bao_percival2010_sigma_DV_DV (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_hubble_cte_wmap (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_hubble_cte_hst (void) G_GNUC_CONST;
 G_INLINE_FUNC gdouble ncm_c_hubble_cte_msa (void) G_GNUC_CONST;
@@ -394,7 +385,7 @@ G_INLINE_FUNC gdouble ncm_c_H_reduced_mass (void)
 G_INLINE_FUNC gdouble ncm_c_H_reduced_energy (void)
 { return ncm_c_H_reduced_mass () * ncm_c_c2 (); }
 
-G_INLINE_FUNC gdouble ncm_c_H_bind (const gint n, const gint j)
+G_INLINE_FUNC gdouble ncm_c_H_bind (const gdouble n, const gdouble j)
 { return ncm_c_H_reduced_energy () *
 	(1.0 - 1.0 / sqrt (1.0 + ncm_c_fine_struct_square () /
 	                   pow (n - j - 0.5 + sqrt(pow(j + 0.5, 2.0) -
@@ -410,7 +401,7 @@ G_INLINE_FUNC gdouble ncm_c_H_bind_2s (void)
 G_INLINE_FUNC gdouble ncm_c_H_bind_2p (void)
 { return ncm_c_H_bind (2.0, 1.5); }
 
-G_INLINE_FUNC gdouble ncm_c_H_Lyman_series (const gint n, const gint j)
+G_INLINE_FUNC gdouble ncm_c_H_Lyman_series (const gdouble n, const gdouble j)
 { return ncm_c_H_bind_1s () - ncm_c_H_bind (n,j); }
 
 G_INLINE_FUNC gdouble ncm_c_H_Lyman_2s (void)
@@ -419,7 +410,7 @@ G_INLINE_FUNC gdouble ncm_c_H_Lyman_2s (void)
 G_INLINE_FUNC gdouble ncm_c_H_Lyman_2p (void)
 { return ncm_c_H_Lyman_series (2, 1.5); }
 
-G_INLINE_FUNC gdouble ncm_c_H_Lyman_series_wl (const gint n, const gint j)
+G_INLINE_FUNC gdouble ncm_c_H_Lyman_series_wl (const gdouble n, const gdouble j)
 { return ncm_c_hc () / ncm_c_H_Lyman_series (n,j); }
 
 G_INLINE_FUNC gdouble ncm_c_H_Lyman_2s_wl (void)
@@ -547,33 +538,6 @@ G_INLINE_FUNC gdouble ncm_c_wmap5_coadded_I_V (void)
 G_INLINE_FUNC gdouble ncm_c_wmap5_coadded_I_W (void)
 { return 6.538; }
 
-G_INLINE_FUNC gdouble ncm_c_bao_eisenstein_z (void)
-{ return 0.35; }
-
-G_INLINE_FUNC gdouble ncm_c_bao_eisenstein_A (void)
-{ return 0.469; }
-
-G_INLINE_FUNC gdouble ncm_c_bao_eisenstein_sigma_A (void)
-{ return 0.017; }
-
-G_INLINE_FUNC gdouble ncm_c_bao_eisenstein_DV (void)
-{ return 1334.0; }
-
-G_INLINE_FUNC gdouble ncm_c_bao_eisenstein_sigma_DV (void)
-{ return 88.0; }
-
-G_INLINE_FUNC gdouble ncm_c_bao_percival2007_DV_DV (void)
-{ return 1.812; }
-
-G_INLINE_FUNC gdouble ncm_c_bao_percival2007_sigma_DV_DV (void)
-{ return 0.060; }
-
-G_INLINE_FUNC gdouble ncm_c_bao_percival2010_DV_DV (void)
-{ return 1.736; }
-
-G_INLINE_FUNC gdouble ncm_c_bao_percival2010_sigma_DV_DV (void)
-{ return 0.065; }
-
 G_INLINE_FUNC gdouble ncm_c_hubble_cte_wmap (void)
 { return 73.0; }
 
