@@ -28,7 +28,7 @@
  * @short_description: Library configuration and helper functions.
  *
  * FIXME
- * 
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,6 +49,7 @@
 #include "math/ncm_reparam_linear.h"
 #include "math/ncm_stats_vec.h"
 #include "nc_hicosmo.h"
+#include "nc_cbe_precision.h"
 #include "perturbations/nc_hipert_iadiab.h"
 #include "model/nc_hicosmo_qconst.h"
 #include "model/nc_hicosmo_qlinear.h"
@@ -59,6 +60,9 @@
 #include "model/nc_hicosmo_de_pad.h"
 #include "model/nc_hicosmo_de_qe.h"
 #include "model/nc_hicosmo_qgrw.h"
+#include "model/nc_hicosmo_de_reparam_ok.h"
+#include "model/nc_hiprim_power_law.h"
+#include "model/nc_hiprim_atan.h"
 #include "lss/nc_window_tophat.h"
 #include "lss/nc_window_gaussian.h"
 #include "lss/nc_growth_func.h"
@@ -87,6 +91,7 @@
 #include "lss/nc_cluster_mass_vanderlinde.h"
 #include "lss/nc_cluster_mass_benson.h"
 #include "lss/nc_cluster_mass_benson_xray.h"
+#include "lss/nc_cluster_mass_plcl.h"
 #include "lss/nc_cluster_redshift.h"
 #include "lss/nc_cluster_redshift_nodist.h"
 #include "lss/nc_cluster_photoz_gauss_global.h"
@@ -97,11 +102,26 @@
 #include "lss/nc_halo_bias_type_st_spher.h"
 #include "lss/nc_halo_bias_type_tinker.h"
 #include "lss/nc_cluster_abundance.h"
+#include "lss/nc_cluster_pseudo_counts.h"
+#include "lss/nc_cor_cluster_cmb_lens_limber.h"
 #include "nc_distance.h"
 #include "nc_recomb.h"
 #include "nc_recomb_seager.h"
 #include "nc_snia_dist_cov.h"
+#include "nc_planck_fi.h"
+#include "nc_planck_fi_cor_tt.h"
+#include "data/nc_data_bao_a.h"
+#include "data/nc_data_bao_dv.h"
+#include "data/nc_data_bao_dvdv.h"
+#include "data/nc_data_bao_rdv.h"
 #include "data/nc_data_bao_empirical_fit.h"
+#include "data/nc_data_bao_dhr_dar.h"
+#include "data/nc_data_cluster_pseudo_counts.h"
+#include "xcor/nc_xcor.h"
+#include "xcor/nc_xcor_limber.h"
+#include "xcor/nc_xcor_limber_gal.h"
+#include "xcor/nc_xcor_limber_lensing.h"
+#include "xcor/nc_data_xcor.h"
 
 #include <gio/gio.h>
 #ifdef NUMCOSMO_HAVE_FFTW3
@@ -165,7 +185,7 @@ _ncm_cfg_log_error (const gchar *log_domain, GLogLevelFlags log_level, const gch
   }
 #endif
   fflush (_log_stream_err);
-  
+
   abort ();
 }
 
@@ -179,15 +199,19 @@ void clencurt_gen (int M);
   void MKL_Set_Num_Threads (gint);
 #endif /* HAVE_MKL_SET_NUM_THREADS */
 
+#ifdef HAVE_OPENMP
+#include <omp.h>
+#endif /* HAVE_OPENMP */
+
 /**
  * ncm_cfg_init:
  *
- * Main library configuration function. Must be called before any 
+ * Main library configuration function. Must be called before any
  * other function of NumCosmo.
- * 
+ *
  * Initializes internal variables and sets
  * all other library number of threads to one.
- * 
+ *
  */
 void
 ncm_cfg_init (void)
@@ -207,7 +231,11 @@ ncm_cfg_init (void)
 #ifdef HAVE_MKL_SET_NUM_THREADS
   MKL_Set_Num_Threads (1);
 #endif /* HAVE_MKL_SET_NUM_THREADS */
-  
+
+#ifdef HAVE_OPENMP
+  omp_set_num_threads (1);
+#endif /* HAVE_OPENMP */
+
   g_setenv ("CUBACORES", "0", TRUE);
   g_setenv ("CUBACORESMAX", "0", TRUE);
   g_setenv ("CUBAACCEL", "0", TRUE);
@@ -233,10 +261,10 @@ ncm_cfg_init (void)
   _log_err_id = g_log_set_handler (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION, _ncm_cfg_log_error, NULL);
 
   ncm_cfg_register_obj (NCM_TYPE_RNG);
-  
+
   ncm_cfg_register_obj (NCM_TYPE_VECTOR);
   ncm_cfg_register_obj (NCM_TYPE_MATRIX);
-  
+
   ncm_cfg_register_obj (NCM_TYPE_SPLINE);
   ncm_cfg_register_obj (NCM_TYPE_SPLINE_CUBIC);
   ncm_cfg_register_obj (NCM_TYPE_SPLINE_CUBIC_NOTAKNOT);
@@ -266,6 +294,13 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_QE);
   ncm_cfg_register_obj (NC_TYPE_HICOSMO_QGRW);
 
+  ncm_cfg_register_obj (NC_TYPE_HICOSMO_DE_REPARAM_OK);
+
+  ncm_cfg_register_obj (NC_TYPE_HIPRIM_POWER_LAW);
+  ncm_cfg_register_obj (NC_TYPE_HIPRIM_ATAN);
+
+  ncm_cfg_register_obj (NC_TYPE_CBE_PRECISION);
+
   ncm_cfg_register_obj (NC_TYPE_WINDOW);
   ncm_cfg_register_obj (NC_TYPE_WINDOW_TOPHAT);
   ncm_cfg_register_obj (NC_TYPE_WINDOW_GAUSSIAN);
@@ -303,6 +338,7 @@ ncm_cfg_init (void)
   ncm_cfg_register_obj (NC_TYPE_CLUSTER_MASS_VANDERLINDE);
   ncm_cfg_register_obj (NC_TYPE_CLUSTER_MASS_BENSON);
   ncm_cfg_register_obj (NC_TYPE_CLUSTER_MASS_BENSON_XRAY);
+  ncm_cfg_register_obj (NC_TYPE_CLUSTER_MASS_PLCL);
 
   ncm_cfg_register_obj (NC_TYPE_CLUSTER_REDSHIFT);
   ncm_cfg_register_obj (NC_TYPE_CLUSTER_REDSHIFT_NODIST);
@@ -319,6 +355,10 @@ ncm_cfg_init (void)
 
   ncm_cfg_register_obj (NC_TYPE_CLUSTER_ABUNDANCE);
 
+  ncm_cfg_register_obj (NC_TYPE_CLUSTER_PSEUDO_COUNTS);
+
+  ncm_cfg_register_obj (NC_TYPE_COR_CLUSTER_CMB_LENS_LIMBER);
+
   ncm_cfg_register_obj (NC_TYPE_DISTANCE);
 
   ncm_cfg_register_obj (NC_TYPE_RECOMB);
@@ -326,8 +366,24 @@ ncm_cfg_init (void)
 
   ncm_cfg_register_obj (NC_TYPE_SNIA_DIST_COV);
 
+  ncm_cfg_register_obj (NC_TYPE_PLANCK_FI);
+  ncm_cfg_register_obj (NC_TYPE_PLANCK_FI_COR_TT);
+
+  ncm_cfg_register_obj (NC_TYPE_DATA_BAO_A);
+  ncm_cfg_register_obj (NC_TYPE_DATA_BAO_DV);
+  ncm_cfg_register_obj (NC_TYPE_DATA_BAO_DVDV);
+  ncm_cfg_register_obj (NC_TYPE_DATA_BAO_RDV);
   ncm_cfg_register_obj (NC_TYPE_DATA_BAO_EMPIRICAL_FIT);
-  
+  ncm_cfg_register_obj (NC_TYPE_DATA_BAO_DHR_DAR);
+
+  ncm_cfg_register_obj (NC_TYPE_DATA_CLUSTER_PSEUDO_COUNTS);
+
+  ncm_cfg_register_obj (NC_TYPE_XCOR);
+  ncm_cfg_register_obj (NC_TYPE_XCOR_LIMBER);
+  ncm_cfg_register_obj (NC_TYPE_XCOR_LIMBER_GAL);
+  ncm_cfg_register_obj (NC_TYPE_XCOR_LIMBER_LENSING);
+  ncm_cfg_register_obj (NC_TYPE_DATA_XCOR);
+
   numcosmo_init = TRUE;
   return;
 }
@@ -370,7 +426,7 @@ void
 ncm_cfg_set_logfile (gchar *filename)
 {
   FILE *out = g_fopen (filename, "w");
-  
+
   if (out != NULL)
     _log_stream = out;
   else
@@ -427,7 +483,7 @@ ncm_message (const gchar *msg, ...)
  * @ncols: FIXME
  *
  * FIXME
- * 
+ *
  * Returns: (transfer full): word wraped string @msg.
  */
 gchar *
@@ -484,7 +540,7 @@ ncm_string_ww (const gchar *msg, const gchar *first, const gchar *rest, guint nc
  *
  * FIXME
  */
-void 
+void
 ncm_message_ww (const gchar *msg, const gchar *first, const gchar *rest, guint ncols)
 {
   gchar *msg_ww = ncm_string_ww (msg, first, rest, ncols);
@@ -496,9 +552,9 @@ ncm_message_ww (const gchar *msg, const gchar *first, const gchar *rest, guint n
  * ncm_cfg_msg_sepa:
  *
  * Log a message separator.
- * 
+ *
  */
-void 
+void
 ncm_cfg_msg_sepa (void)
 {
   g_message ("#----------------------------------------------------------------------------------\n");
@@ -572,7 +628,7 @@ ncm_cfg_keyfile_to_arg (GKeyFile *kfile, const gchar *group_name, GOptionEntry *
           gchar *val = g_key_file_get_value (kfile, group_name, entries[i].long_name, &error);
           if (error != NULL)
             g_error ("ncm_cfg_keyfile_to_arg: Cannot parse key file[%s]", error->message);
-          
+
           if (entries[i].arg == G_OPTION_ARG_NONE)
           {
             if ((g_ascii_strcasecmp (val, "1") == 0) ||
@@ -596,9 +652,9 @@ ncm_cfg_keyfile_to_arg (GKeyFile *kfile, const gchar *group_name, GOptionEntry *
 /**
  * ncm_cfg_string_to_comment:
  * @str: FIXME
- * 
+ *
  * FIXME
- * 
+ *
  * Returns: (transfer full): FIXME
  */
 gchar *
@@ -686,7 +742,7 @@ ncm_cfg_entries_to_keyfile (GKeyFile *kfile, const gchar *group_name, GOptionEnt
     }
     if (!skip_comment)
     {
-      gchar *desc = ncm_cfg_string_to_comment (entries[i].description); 
+      gchar *desc = ncm_cfg_string_to_comment (entries[i].description);
       if (!g_key_file_set_comment (kfile, group_name, entries[i].long_name, desc, &error))
         g_error ("ncm_cfg_entries_to_keyfile: %s", error->message);
       g_free (desc);
@@ -737,7 +793,7 @@ ncm_cfg_get_enum_by_id_name_nick (GType enum_type, const gchar *id_name_nick)
  * @n: FIXME
  *
  * FIXME
- * 
+ *
  * Returns: (transfer none)
  */
 const GEnumValue *
@@ -837,7 +893,7 @@ ncm_cfg_load_fftw_wisdom (const gchar *filename, ...)
     {
       g_error ("ncm_cfg_load_fftw_wisdom: cannot open wisdom file %s [%s].", full_filename, g_strerror (errno));
     }
-        
+
     fftw_import_wisdom_from_file (wis);
     fclose (wis);
     ret = TRUE;
@@ -915,7 +971,7 @@ ncm_cfg_fopen (const gchar *filename, const gchar *mode, ...)
   {
     g_error ("ncm_cfg_fopen: cannot open file %s [%s].", full_filename, g_strerror (errno));
   }
-  
+
   g_free (file);
   g_free (full_filename);
   return F;
@@ -1034,7 +1090,7 @@ ncm_cfg_load_spline (const gchar *filename, const gsl_interp_type *stype, NcmSpl
   }
   else
     ncm_spline_prepare (*s);
-    
+
   ncm_vector_free (xv);
   ncm_vector_free (yv);
 
@@ -1370,8 +1426,8 @@ load_save_vector_matrix(complex)
  *
  * Looks for @filename in the data path and returns
  * the full path if found.
- * 
- * Returns: (transfer full): Full path for @filename. 
+ *
+ * Returns: (transfer full): Full path for @filename.
  */
 gchar *
 ncm_cfg_get_data_filename (const gchar *filename, gboolean must_exist)
@@ -1390,7 +1446,16 @@ ncm_cfg_get_data_filename (const gchar *filename, gboolean must_exist)
   }
 
   if (full_filename == NULL)
+  {
     full_filename = g_build_filename (PACKAGE_DATA_DIR, "data", filename, NULL);
+    if (!g_file_test (full_filename, G_FILE_TEST_EXISTS))
+    {
+      g_clear_pointer (&full_filename, g_free);
+    }
+  }
+
+  if (full_filename == NULL)
+    full_filename = g_build_filename (PACKAGE_SOURCE_DIR, "data", filename, NULL);
 
   if (!g_file_test (full_filename, G_FILE_TEST_EXISTS))
   {
@@ -1427,7 +1492,7 @@ ncm_cfg_get_default_sqlite3 (void)
       g_error ("Connection to database failed: %s", sqlite3_errmsg (db));
 
     g_free (filename);
-    
+
   }
 
   return db;
@@ -1512,7 +1577,7 @@ ncm_cfg_variant_to_array (GVariant *var, gsize esize)
  * FIXME
  *
  */
-void 
+void
 ncm_cfg_array_set_variant (GArray *a, GVariant *var)
 {
   gsize esize = g_array_get_element_size (a);
@@ -1557,7 +1622,7 @@ guint fftw_default_flags = FFTW_PATIENT; /* FFTW_ESTIMATE, FFTW_MEASURE, FFTW_PA
  * to be used when building plans.
  *
  */
-void 
+void
 ncm_cfg_set_fftw_default_flag (guint flag)
 {
   fftw_default_flags = flag;
