@@ -455,9 +455,7 @@ ncm_fit_mc_keep_order (NcmFitMC *mc, gboolean keep_order)
 void 
 ncm_fit_mc_set_fiducial (NcmFitMC *mc, NcmMSet *fiduc)
 {
-  if (mc->fiduc != NULL)
-    ncm_mset_clear (&mc->fiduc);
-
+  ncm_mset_clear (&mc->fiduc);
   if (fiduc == NULL || fiduc == mc->fit->mset)
   {
     mc->fiduc = ncm_mset_dup (mc->fit->mset, mc->ser);
@@ -556,7 +554,7 @@ _ncm_fit_mc_resample_bstrap (NcmDataset *dset, NcmMSet *mset, NcmRNG *rng)
 void 
 ncm_fit_mc_start_run (NcmFitMC *mc)
 {
-  guint param_len = ncm_mset_param_len (mc->fit->mset);
+  guint param_len = ncm_mset_total_len (mc->fit->mset);
 
   if (mc->started)
     g_error ("ncm_fit_mc_start_run: run already started, run ncm_fit_mc_end_run() first.");
