@@ -55,12 +55,6 @@ typedef enum _NcmLHRatio1dRoot
   NCM_LH_RATIO1D_ROOT_NUMDIFF,
 } NcmLHRatio1dRoot;
 
-struct _NcmLHRatio1dClass
-{
-  /*< private >*/
-  GObjectClass parent_class;
-};
-
 struct _NcmLHRatio1d
 {
   /*< private >*/
@@ -80,9 +74,15 @@ struct _NcmLHRatio1d
   guint grad_eval;
 };
 
+struct _NcmLHRatio1dClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+};
+
 GType ncm_lh_ratio1d_get_type (void) G_GNUC_CONST;
 
-NcmLHRatio1d *ncm_lh_ratio1d_new (NcmFit *fit, NcmMSetPIndex *pi);
+NcmLHRatio1d *ncm_lh_ratio1d_new (NcmFit *fit, const NcmMSetPIndex *pi);
 void ncm_lh_ratio1d_free (NcmLHRatio1d *lhr1d);
 void ncm_lh_ratio1d_clear (NcmLHRatio1d **lhr1d);
 
