@@ -104,7 +104,6 @@ ncm_lh_ratio2d_constructed (GObject *object)
     for (i = 0; i < 2; i++)
     {
       g_assert_cmpint (lhr2d->pi[i].mid, >=, 0);
-      g_assert_cmpint (lhr2d->pi[i].mid,  <, NCM_MODEL_MAX_ID);
 
       if (ncm_mset_peek (lhr2d->fit->mset, lhr2d->pi[i].mid) == NULL)
         g_error ("ncm_lh_ratio2d_constructed: cannot use parameter[%d:%u], model not set.", 
@@ -269,7 +268,7 @@ ncm_lh_ratio2d_class_init (NcmLHRatio2dClass *klass)
  * Returns: FIXME
  */
 NcmLHRatio2d *
-ncm_lh_ratio2d_new (NcmFit *fit, NcmMSetPIndex *pi1, NcmMSetPIndex *pi2, gdouble border_prec)
+ncm_lh_ratio2d_new (NcmFit *fit, const NcmMSetPIndex *pi1, const NcmMSetPIndex *pi2, gdouble border_prec)
 {
   return g_object_new (NCM_TYPE_LH_RATIO2D, 
                        "fit", fit,
