@@ -47,9 +47,12 @@ G_BEGIN_DECLS
 typedef struct _NcmMSetClass NcmMSetClass;
 typedef struct _NcmMSet NcmMSet;
 typedef struct _NcmMSetPIndex NcmMSetPIndex;
-typedef struct _NcmMSetModelDesc NcmMSetModelDesc; 
+typedef struct _NcmMSetModelDesc NcmMSetModelDesc;
 
-#define NCM_MODEL_MAX_ID 5
+#define NCM_MSET_MAX_SUBMODEL 1000
+#define NCM_MSET_INIT_MARRAY 32
+#define NCM_MSET_GET_BASE_MID(mid) (mid / NCM_MSET_MAX_SUBMODEL)
+#define NCM_MSET_MID(id,pos) ((id) + pos)
 
 struct _NcmMSetModelDesc
 {
@@ -64,10 +67,13 @@ struct _NcmMSet
 {
   /*< private >*/
   GObject parent_instance;
-  NcmModel *model[NCM_MODEL_MAX_ID];
-  GArray *pi_array;
-  GArray *fpi_array[NCM_MODEL_MAX_ID];
+  GPtrArray *model_array;
+  GHashTable *mid_item_hash;
+  GHashTable *model_item_hash;
+  GHashTable *fpi_hash;
   GPtrArray *fullname_parray;
+  GArray *pi_array;
+  GRegex *fullname_regex;
   gboolean valid_map;
   guint total_len;
   guint fparam_len;
@@ -78,7 +84,7 @@ struct _NcmMSetClass
   /*< private >*/
   GObjectClass parent_class;
   GHashTable *ns_table;
-  NcmMSetModelDesc model_desc[NCM_MODEL_MAX_ID];
+  GArray *model_desc_array;
 };
 
 struct _NcmMSetPIndex
@@ -96,18 +102,18 @@ void ncm_mset_model_register_id (NcmModelClass *model_class, const gchar *ns, co
 /**
  * NCM_MSET_MODEL_ID_FUNC: (skip)
  * @model_ns: FIXME
- * 
+ *
  * FIXME
- * 
+ *
  */
 #define NCM_MSET_MODEL_ID_FUNC(model_ns) model_ns##_id
 
 /**
  * NCM_MSET_MODEL_DECLARE_ID: (skip)
  * @model_ns: FIXME
- * 
+ *
  * FIXME
- * 
+ *
  */
 #define NCM_MSET_MODEL_DECLARE_ID(model_ns) gint32 NCM_MSET_MODEL_ID_FUNC(model_ns) (void) G_GNUC_CONST
 
@@ -115,9 +121,9 @@ void ncm_mset_model_register_id (NcmModelClass *model_class, const gchar *ns, co
  * NCM_MSET_MODEL_REGISTER_ID: (skip)
  * @model_ns: FIXME
  * @typemacro: FIXME
- * 
+ *
  * FIXME
- * 
+ *
  */
 #define NCM_MSET_MODEL_REGISTER_ID(model_ns,typemacro) \
 gint32 NCM_MSET_MODEL_ID_FUNC(model_ns) (void) \
@@ -142,18 +148,28 @@ NcmMSet *ncm_mset_newv (NcmModel *model0, va_list ap);
 NcmMSet *ncm_mset_new_array (NcmModel **model);
 NcmMSet *ncm_mset_ref (NcmMSet *mset);
 NcmMSet *ncm_mset_dup (NcmMSet *mset, NcmSerialize *ser);
+NcmMSet *ncm_mset_shallow_copy (NcmMSet *mset);
 
 void ncm_mset_free (NcmMSet *mset);
 void ncm_mset_clear (NcmMSet **mset);
 
 NcmModel *ncm_mset_peek (NcmMSet *mset, NcmModelID mid);
+NcmModel *ncm_mset_peek_pos (NcmMSet *mset, NcmModelID base_mid, guint submodel_id);
 NcmModel *ncm_mset_get (NcmMSet *mset, NcmModelID mid);
+NcmModel *ncm_mset_peek_array_pos (NcmMSet *mset, guint i);
+NcmModelID ncm_mset_get_mid_array_pos (NcmMSet *mset, guint i);
 void ncm_mset_remove (NcmMSet *mset, NcmModelID mid);
 void ncm_mset_set (NcmMSet *mset, NcmModel *model);
+void ncm_mset_push (NcmMSet *mset, NcmModel *model);
+void ncm_mset_set_pos (NcmMSet *mset, NcmModel *model, guint submodel_id);
 gboolean ncm_mset_exists (NcmMSet *mset, NcmModel *model);
+gboolean ncm_mset_exists_pos (NcmMSet *mset, NcmModel *model, guint submodel_id);
+gboolean ncm_mset_is_subset (NcmMSet *mset, NcmMSet *sub_mset);
 
 gint ncm_mset_get_id_by_ns (const gchar *ns);
 
+void ncm_mset_set_fmap (NcmMSet *mset, const gchar *const *fmap, gboolean update_models);
+gchar **ncm_mset_get_fmap (NcmMSet *mset);
 void ncm_mset_prepare_fparam_map (NcmMSet *mset);
 
 guint ncm_mset_total_len (NcmMSet *mset);
@@ -161,6 +177,7 @@ guint ncm_mset_fparam_len (NcmMSet *mset);
 guint ncm_mset_max_param_name (NcmMSet *mset);
 guint ncm_mset_max_fparam_name (NcmMSet *mset);
 guint ncm_mset_max_model_nick (NcmMSet *mset);
+guint ncm_mset_nmodels (NcmMSet *mset);
 
 void ncm_mset_pretty_log (NcmMSet *mset);
 void ncm_mset_params_pretty_print (NcmMSet *mset, FILE *out, const gchar *header);
@@ -176,11 +193,13 @@ void ncm_mset_param_set (NcmMSet *mset, NcmModelID mid, guint pid, const gdouble
 gdouble ncm_mset_param_get (NcmMSet *mset, NcmModelID mid, guint pid);
 gdouble ncm_mset_orig_param_get (NcmMSet *mset, NcmModelID mid, guint pid);
 
-guint ncm_mset_param_len (NcmMSet *mset);
 const gchar *ncm_mset_param_name (NcmMSet *mset, NcmModelID mid, guint pid);
 const gchar *ncm_mset_param_symbol (NcmMSet *mset, NcmModelID mid, guint pid);
 void ncm_mset_param_set_ftype (NcmMSet *mset, NcmModelID mid, guint pid, NcmParamType ftype);
 void ncm_mset_param_set_all_ftype (NcmMSet *mset, NcmParamType ftype);
+void ncm_mset_param_set_mid_ftype (NcmMSet *mset, NcmModelID mid, NcmParamType ftype);
+void ncm_mset_param_set_all_but_mid_ftype (NcmMSet *mset, NcmModelID mid, NcmParamType ftype);
+void ncm_mset_param_set_ftype_from_fmap (NcmMSet *mset);
 void ncm_mset_param_set_vector (NcmMSet *mset, NcmVector *params);
 void ncm_mset_param_get_vector (NcmMSet *mset, NcmVector *params);
 gdouble ncm_mset_param_get_scale (NcmMSet *mset, NcmModelID mid, guint pid);
@@ -204,6 +223,7 @@ guint ncm_mset_fparams_len (NcmMSet *mset);
 const gchar *ncm_mset_fparam_name (NcmMSet *mset, guint n);
 const gchar *ncm_mset_fparam_symbol (NcmMSet *mset, guint n);
 const gchar *ncm_mset_fparam_full_name (NcmMSet *mset, guint n);
+NcmMSetPIndex *ncm_mset_param_get_by_full_name (NcmMSet *mset, const gchar *fullname);
 gdouble ncm_mset_fparam_get_scale (NcmMSet *mset, guint n);
 gdouble ncm_mset_fparam_get_lower_bound (NcmMSet *mset, guint n);
 gdouble ncm_mset_fparam_get_upper_bound (NcmMSet *mset, guint n);
@@ -214,10 +234,9 @@ gboolean ncm_mset_fparam_valid_bounds (NcmMSet *mset, NcmVector *theta);
 gdouble ncm_mset_fparam_get (NcmMSet *mset, guint n);
 void ncm_mset_fparam_set (NcmMSet *mset, guint n, const gdouble x);
 
-NcmMSetPIndex *ncm_mset_fparam_get_pi (NcmMSet *mset, guint n);
+const NcmMSetPIndex *ncm_mset_fparam_get_pi (NcmMSet *mset, guint n);
 gint ncm_mset_fparam_get_fpi (NcmMSet *mset, NcmModelID mid, guint pid);
-
-NcmMSetPIndex *ncm_mset_fparam_get_pi_by_name (NcmMSet *mset, const gchar *name);
+const NcmMSetPIndex *ncm_mset_fparam_get_pi_by_name (NcmMSet *mset, const gchar *name);
 
 void ncm_mset_save (NcmMSet *mset, NcmSerialize *ser, const gchar *filename, gboolean save_comment);
 NcmMSet *ncm_mset_load (const gchar *filename, NcmSerialize *ser);
