@@ -126,6 +126,7 @@ G_INLINE_FUNC void ncm_vector_fast_addto (NcmVector *cv, const guint i, const gd
 G_INLINE_FUNC void ncm_vector_subfrom (NcmVector *cv, const guint i, const gdouble val);
 G_INLINE_FUNC void ncm_vector_fast_subfrom (NcmVector *cv, const guint i, const gdouble val);
 G_INLINE_FUNC void ncm_vector_set_all (NcmVector *cv, const gdouble val);
+G_INLINE_FUNC void ncm_vector_set_array (NcmVector *cv, const gdouble *array);
 G_INLINE_FUNC void ncm_vector_scale (NcmVector *cv, const gdouble val);
 G_INLINE_FUNC void ncm_vector_div (NcmVector *cv1, const NcmVector *cv2);
 G_INLINE_FUNC void ncm_vector_add (NcmVector *cv1, const NcmVector *cv2);
@@ -144,6 +145,7 @@ G_INLINE_FUNC guint ncm_vector_len (const NcmVector *cv);
 G_INLINE_FUNC guint ncm_vector_stride (const NcmVector *cv);
 
 NcmVector *ncm_vector_dup (const NcmVector *cv);
+void ncm_vector_substitute (NcmVector **cv, NcmVector *nv, gboolean check_size);
 void ncm_vector_free (NcmVector *cv);
 void ncm_vector_clear (NcmVector **cv);
 void ncm_vector_const_free (const NcmVector *cv);
@@ -238,6 +240,15 @@ ncm_vector_set_all (NcmVector *cv, const gdouble val)
   gsl_vector_set_all (ncm_vector_gsl (cv), val);
 }
 
+G_INLINE_FUNC void
+ncm_vector_set_array (NcmVector *cv, const gdouble *array)
+{
+  register guint i;
+  const guint size = ncm_vector_len (cv);
+  for (i = 0; i < size; i++)
+    ncm_vector_set (cv, i, array[i]);
+}
+
 G_INLINE_FUNC void
 ncm_vector_scale (NcmVector *cv, const gdouble val)
 {
@@ -284,7 +295,7 @@ G_INLINE_FUNC GArray *
 ncm_vector_get_array (NcmVector *cv)
 {
   g_assert (cv->type == NCM_VECTOR_ARRAY);
-  return g_array_ref (cv->pdata); 
+  return g_array_ref (cv->pdata);
 }
 
 G_INLINE_FUNC GArray *
