@@ -131,7 +131,7 @@ NC_TRIVEC_SET_0(q->x); \
  */
 #define NC_QUATERNION_MEMCPY(a,b) memcpy (a, b, sizeof(NcmQ))
 
-NcmQ *ncm_quaternion_new ();
+NcmQ *ncm_quaternion_new (void);
 NcmQ *ncm_quaternion_new_from_vector (NcmTriVector v);
 NcmQ *ncm_quaternion_new_from_data (gdouble x, gdouble y, gdouble z, gdouble theta);
 
@@ -152,6 +152,8 @@ void ncm_quaternion_conjugate_q_mul (NcmQ *q, NcmQ *u, NcmQ *res);
 void ncm_quaternion_rotate (NcmQ *q, NcmTriVector v);
 void ncm_quaternion_inv_rotate (NcmQ *q, NcmTriVector v);
 
+#define NCM_QUATERNION_RNG_NAME "quaternion"
+
 G_END_DECLS
 
 #endif /* _NC_QUATERNION_H */
