@@ -47,14 +47,16 @@ typedef struct _NcHICosmoDE NcHICosmoDE;
  * NcHICosmoDEImpl:
  * @NC_HICOSMO_DE_IMPL_weff: FIXME
  * @NC_HICOSMO_DE_IMPL_dweff_dz: FIXME
+ * @NC_HICOSMO_DE_IMPL_d2weff_dz2: FIXME
  *
  * FIXME
  */
 typedef enum _NcHICosmoDEImpl
 {
-  NC_HICOSMO_DE_IMPL_weff     = NC_HICOSMO_IMPL_LAST << 0,
-  NC_HICOSMO_DE_IMPL_dweff_dz = NC_HICOSMO_IMPL_LAST << 1, /*< private >*/
-  NC_HICOSMO_DE_IMPL_LAST     = NC_HICOSMO_IMPL_LAST << 2, /*< skip >*/
+  NC_HICOSMO_DE_IMPL_weff       = NC_HICOSMO_IMPL_LAST << 0,
+  NC_HICOSMO_DE_IMPL_dweff_dz   = NC_HICOSMO_IMPL_LAST << 1,
+  NC_HICOSMO_DE_IMPL_d2weff_dz2 = NC_HICOSMO_IMPL_LAST << 2, /*< private >*/
+  NC_HICOSMO_DE_IMPL_LAST       = NC_HICOSMO_IMPL_LAST << 3, /*< skip >*/
 } NcHICosmoDEImpl;
 
 /**
@@ -63,9 +65,11 @@ typedef enum _NcHICosmoDEImpl
  * @NC_HICOSMO_DE_OMEGA_C: FIXME
  * @NC_HICOSMO_DE_OMEGA_X: FIXME
  * @NC_HICOSMO_DE_T_GAMMA0: FIXME
+ * @NC_HICOSMO_DE_ENNU: FIXME
  * @NC_HICOSMO_DE_OMEGA_B: FIXME
  * @NC_HICOSMO_DE_SPECINDEX: FIXME
  * @NC_HICOSMO_DE_SIGMA8: FIXME
+ * @NC_HICOSMO_DE_TAU_RE: FIXME
  *
  * FIXME
  */
@@ -75,9 +79,11 @@ typedef enum _NcHICosmoDEParams
   NC_HICOSMO_DE_OMEGA_C,
   NC_HICOSMO_DE_OMEGA_X,
   NC_HICOSMO_DE_T_GAMMA0,
+  NC_HICOSMO_DE_ENNU,
   NC_HICOSMO_DE_OMEGA_B,
   NC_HICOSMO_DE_SPECINDEX,
-  NC_HICOSMO_DE_SIGMA8,     /*< private >*/
+  NC_HICOSMO_DE_SIGMA8,
+  NC_HICOSMO_DE_TAU_RE,       /*< private >*/
   NC_HICOSMO_DE_SPARAM_LEN, /*< skip >*/
 } NcHICosmoDEParams;
 
@@ -86,8 +92,11 @@ typedef enum _NcHICosmoDEParams
 #define NC_HICOSMO_DE_DEFAULT_OMEGA_X   (0.70)
 #define NC_HICOSMO_DE_DEFAULT_OMEGA_B   (0.0432)
 #define NC_HICOSMO_DE_DEFAULT_T_GAMMA0  (2.7245)
+#define NC_HICOSMO_DE_DEFAULT_ENNU      (3.046)
 #define NC_HICOSMO_DE_DEFAULT_SPECINDEX (1.0)
 #define NC_HICOSMO_DE_DEFAULT_SIGMA8    (0.9)
+#define NC_HICOSMO_DE_DEFAULT_TAU_RE    (0.1)
+/*(11.357)*/
 
 struct _NcHICosmoDE
 {
@@ -101,19 +110,22 @@ struct _NcHICosmoDEClass
   NcHICosmoClass parent_class;
   NcmModelFunc1 weff;
   NcmModelFunc1 dweff_dz;
+  NcmModelFunc1 d2weff_dz2;
 };
 
 GType nc_hicosmo_de_get_type (void) G_GNUC_CONST;
 
-void nc_hicosmo_de_set_wmap5_params (NcHICosmo *cosmo);
-void nc_hicosmo_de_omega_x2omega_k (NcHICosmo *cosmo);
+void nc_hicosmo_de_set_wmap5_params (NcHICosmoDE *cosmo_de);
+void nc_hicosmo_de_omega_x2omega_k (NcHICosmoDE *cosmo_de);
 gboolean nc_hicosmo_de_new_add_bbn (NcmLikelihood *lh);
 
 void nc_hicosmo_de_set_weff_impl (NcHICosmoDEClass *cosmo_de_class, NcmModelFunc1 f);
 void nc_hicosmo_de_set_dweff_dz_impl (NcHICosmoDEClass *cosmo_de_class, NcmModelFunc1 f);
+void nc_hicosmo_de_set_d2weff_dz2_impl (NcHICosmoDEClass *cosmo_de_class, NcmModelFunc1 f);
 
 G_INLINE_FUNC gdouble nc_hicosmo_de_weff (NcHICosmoDE *cosmo, gdouble x);
 G_INLINE_FUNC gdouble nc_hicosmo_de_dweff_dz (NcHICosmoDE *cosmo, gdouble x);
+G_INLINE_FUNC gdouble nc_hicosmo_de_d2weff_dz2 (NcHICosmoDE *cosmo, gdouble x);
 
 G_END_DECLS
 
@@ -127,6 +139,7 @@ G_BEGIN_DECLS
 
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO_DE,NcHICosmoDE,nc_hicosmo_de,weff)
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO_DE,NcHICosmoDE,nc_hicosmo_de,dweff_dz)
+NCM_MODEL_FUNC1_IMPL (NC_HICOSMO_DE,NcHICosmoDE,nc_hicosmo_de,d2weff_dz2)
 
 G_END_DECLS
 
