@@ -24,11 +24,11 @@
 
 /**
  * SECTION:nc_hicosmo_de_pad
- * @title: Dark Energy -- Jassal-Bagla-Padmanabhan
- * @short_description: Jassal-Bagla-Padmanabhan  dark energy equation of state parametrization
+ * @title: NcHICosmoDEPad
+ * @short_description: Dark Energy -- Jassal-Bagla-Padmanabhan of state parametrization.
+ *
+ * See [Jassal et al. (2005)][XJassal2005].
  *
- * See <link linkend="XJassal2005">Jassal et al. (2005)</link>.
- * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -85,6 +85,7 @@ enum {
 static void
 nc_hicosmo_de_pad_init (NcHICosmoDEPad *pad)
 {
+  NCM_UNUSED (pad);
 }
 
 static void
@@ -101,8 +102,7 @@ nc_hicosmo_de_pad_class_init (NcHICosmoDEPadClass *klass)
   GObjectClass* object_class = G_OBJECT_CLASS (klass);
   NcHICosmoDEClass* parent_class = NC_HICOSMO_DE_CLASS (klass);
   NcmModelClass *model_class = NCM_MODEL_CLASS (klass);
-  object_class->set_property = &ncm_model_class_set_property;
-  object_class->get_property = &ncm_model_class_get_property;
+
   object_class->finalize     = &nc_hicosmo_de_pad_finalize;
 
   nc_hicosmo_de_set_weff_impl (parent_class, &_nc_hicosmo_de_pad_weff);
