@@ -24,8 +24,8 @@
 
 /**
  * SECTION:quantum_gravity
- * @title: Quantum Gravity Bouncing Model
- * @short_description: FIXME
+ * @title: NcHICosmoQG
+ * @short_description: Quantum gravity bouncing model.
  *
  * FIXME
  */
@@ -36,7 +36,6 @@
 #include "build_cfg.h"
 
 #include "model/quantum_gravity.h"
-#include "math/cvode_util.h"
 #include "math/ncm_spline_gsl.h"
 #include "math/magnus_iserles_ode.h"
 
@@ -112,18 +111,19 @@ nc_hicosmo_qg_new (void)
   g_assert_not_reached ();
 }
 
-
 gdouble
 nc_hicosmo_qg_cs2 (NcmModel *model, gdouble x, gpointer userdata)
 {
-  gdouble R = 3.0 * (1.0 + W) * OMEGA_M / (4.0 * OMEGA_R * x * pow(x, -3.0*W));
+  gdouble R = 3.0 * (1.0 + W) * OMEGA_M / (4.0 * OMEGA_R * x * pow(x, -3.0 * W));
+  NCM_UNUSED (userdata);
   return (1.0 + 3.0 * W * R) / (3.0 * (1.0 + R));
 }
 
 gdouble
 nc_hicosmo_qg_dcs2 (NcmModel *model, gdouble x, gpointer userdata)
 {
-  gdouble R = 3.0 * (1.0 + W) * OMEGA_M / (4.0 * OMEGA_R * x * pow(x, -3.0*W));
+  gdouble R = 3.0 * (1.0 + W) * OMEGA_M / (4.0 * OMEGA_R * x * pow(x, -3.0 * W));
+  NCM_UNUSED (userdata);
   return (1.0 - 3.0 * W) / (3.0 * x) * ((1.0 - 3.0 * W)*R+3.0*W*R*R) / gsl_pow_2 (1.0 + R);
 }
 
@@ -132,7 +132,7 @@ nc_hicosmo_qg_beta (NcmModel *model, gdouble x, gpointer userdata)
 {
   gdouble OM = 3.0 * (1.0 + W) * OMEGA_M * pow (x, 1.0 + 3.0 * W);
   gdouble OR = 4.0 * OMEGA_R * x * x;
-
+  NCM_UNUSED (userdata);
   return (OM + OR) / 2.0;
 }
 
@@ -140,6 +140,7 @@ gdouble
 nc_hicosmo_qg_Omega_x2 (NcmModel *model, gdouble x, gpointer userdata)
 {
   gdouble x3w = pow (x, 3.0 * W);
+  NCM_UNUSED (userdata);
   return x * (x3w * OMEGA_M + x * OMEGA_R);
 }
 
@@ -147,6 +148,7 @@ gdouble
 nc_hicosmo_qg_dOmega_x2 (NcmModel *model, gdouble x, gpointer userdata)
 {
   gdouble x3w = pow (x, 3.0 * W);
+  NCM_UNUSED (userdata);
   return ((1.0 + 3.0 * W) * x3w * OMEGA_M + 2.0 * x * OMEGA_R);
 }
 
@@ -154,6 +156,7 @@ gdouble
 nc_hicosmo_qg_xbar (NcmModel *model, gdouble x, gpointer userdata)
 {
   gdouble x3w = pow (x, 3.0 * W);
+  NCM_UNUSED (userdata);
   return sqrt (x * (x3w * OMEGA_M + x * OMEGA_R));
 }
 
@@ -174,6 +177,8 @@ nc_hicosmo_qg_d2sqrtxxbarzeta_sqrtxxbarzeta (NcmModel *model, gdouble x, gpointe
   gdouble Or2 = Or * Or;
   gdouble Or3 = Or2 * Or;
 
+  NCM_UNUSED (userdata);
+  
   return ( x3w / x2 * OMEGA_M *
           (81.0 * x9w * w2 * one_pw2 * (5.0 + 9.0 * (-2.0 + W) * W ) * Om3 +
            36.0 * x * x6w * W * one_pw * (-2.0 + 3.0 * W) * (-1.0 + 3.0 * W) * (7.0 + 15.0 * W) * Om2 * Or +
@@ -194,6 +199,9 @@ nc_hicosmo_qg_cs2_xxbar2 (NcmModel *model, gdouble x, gpointer userdata)
   gdouble x3w = pow (x, 3.0 * W);
   gdouble R = 3.0 * (1.0 + W) * OMEGA_M / (4.0 * OMEGA_R * x / x3w);
   gdouble cs2 = (1.0 + 3.0 * W * R) / (3.0 * (1.0 + R));
+
+  NCM_UNUSED (userdata);
+  
   return cs2 / (x3 * x3w * OMEGA_M + x4 * OMEGA_R);
 }
 
@@ -240,6 +248,8 @@ gdouble
 nc_hicosmo_qg_xxbarzeta2 (NcmModel *model, gdouble x, gpointer userdata)
 {
   gdouble x3w = pow (x, 3.0 * W);
+  
+  NCM_UNUSED (userdata);
 
   return (3.0 * gsl_pow_2 (3.0 * x3w * (1.0 + W) * OMEGA_M + 4.0 * x * OMEGA_R)) /
    (2.0 * sqrt (x * (x3w * OMEGA_M + x * OMEGA_R)) *
@@ -251,6 +261,8 @@ nc_hicosmo_qg_dxxbarzeta2_xxbarzeta2 (NcmModel *model, gdouble x, gpointer userd
 {
   gdouble x3w = pow (x, 3.0 * W);
 
+  NCM_UNUSED (userdata);
+  
   return ((-1.0 + 3.0 * W) *
           ( 1.0 / x + OMEGA_R * ( 1.0 / (x3w * OMEGA_M + x * OMEGA_R) -
                                  16.0 / ( 3.0 * x3w * (1.0 + W) * OMEGA_M + 4.0 * x * OMEGA_R) +
@@ -266,6 +278,8 @@ nc_hicosmo_qg_zeta (NcmModel *model, gdouble x, gpointer userdata)
   gdouble R = 3.0 * (1.0 + W) * OMEGA_M / (4.0 * OMEGA_R * x * pow(x, -3.0 * W));
   gdouble F = OMEGA_X / (OMEGA_R * gsl_pow_4 (x));
 
+  NCM_UNUSED (userdata);
+  
   return sqrt(6.0) / x * (1.0 + R) / sqrt ((1.0 + 4.0 * R / (3.0 * (1.0 + W)) + F) * (1.0 + 3.0 * W * R));
 }
 
@@ -278,6 +292,8 @@ nc_hicosmo_qg_dzeta_zeta (NcmModel *model, gdouble x, gpointer userdata)
   const gdouble G2 = 3.0 * W * (1.0 - 3.0 * W) * R / (2.0 * (1.0 + 3.0 * W * R));
   const gdouble cs2 = (1.0 + 3.0 * W * R) / (3.0 * (1.0 + R));
 
+  NCM_UNUSED (userdata);
+  
   return (3.0 * cs2 + G1 + G2 - 2.0) / x;
 }
 
@@ -285,6 +301,9 @@ gdouble
 nc_hicosmo_qg_dcs2zeta2_cs2zeta2 (NcmModel *model, gdouble x, gpointer userdata)
 {
   const gdouble R = 3.0 * (1.0 + W) * OMEGA_M / (4.0 * OMEGA_R * x * pow(x, -3.0 * W));
+
+  NCM_UNUSED (userdata);
+  
   return -((6.0 * (1.0 + W) + R * (13.0 + 8.0 * R + 3.0 * (4.0 - 3.0 * W) * W) ) / ((1.0 + R) * x  * (3.0 + 4.0 * R + 3.0 * W)));
 }
 
@@ -296,6 +315,9 @@ nc_hicosmo_qg_dx2dcs2zeta2_cs2zeta2 (NcmModel *model, gdouble x, gpointer userda
   const gdouble R3 = R2 * R;
   const gdouble R4 = R2 * R2;
   const gdouble onepw2 = (1.0 + W) * (1.0 + W);
+
+  NCM_UNUSED (userdata);
+  
   return (-32.0 * R4 - 18.0 * onepw2 + 3.0 * R * (1.0 + W) * (-28.0 + 9.0 * (-1.0 + W) * W * (1.0 + 3.0 * W)) + R2 * (-139.0 + 9.0 * W * (-19.0 + 3.0 * W * (1.0 + W))) - 4.0 * R3 * (26.0 + 9.0 * W * (3.0 + W * (-4.0 + 3.0 * W)))) / (gsl_pow_2 (1.0 + R) * gsl_pow_2 (3.0 + 4.0 * R + 3.0 * W));
 }
 
@@ -314,6 +336,8 @@ nc_hicosmo_qg_ddzeta_zeta (NcmModel *model, gdouble x, gpointer userdata)
 
   gdouble dzeta_zeta = (3.0 * cs2 + G1 + G2 - 2.0) / x;
 
+  NCM_UNUSED (userdata);
+  
   return ((3.0 * dcs2 + dG1 + dG2)  - dzeta_zeta) / x + dzeta_zeta * dzeta_zeta;
 }
 
@@ -329,6 +353,8 @@ nc_hicosmo_qg_xddzeta_zeta_mxdzeta_zeta2_dzeta_zeta (NcmModel *model, gdouble x,
   gdouble dG1 = (2.0 * G1*G1 - (24.0 * (1.0 + W) * F + 2.0 * gsl_pow_2 (1.0 - 3.0 * W) * R) / (3.0 * (1.0 + W)*(1.0 + F) + 4.0 * R)) / x;
   gdouble dG2 = (2.0 * G2*G2 - (1.0 - 3.0 * W) * G2) / x;
 
+  NCM_UNUSED (userdata);
+  
   return 3.0 * dcs2 + dG1 + dG2;
 }
 
@@ -484,6 +510,8 @@ nc_hicosmo_qg_V (NcmModel *model, gdouble x, gpointer userdata)
   gdouble V3 = -(OMEGA_R * x2/xb2 + OMEGA_M * ((1.0 - 3.0 * W) * x3w / (x * 2.0) + x2  * xb3w / xb3) +OMEGA_X * (2.0 / x4 + x2 / xb6)) *
     x3 * nc_hicosmo_qg_dzeta_zeta (model, x, NULL);
 
+  NCM_UNUSED (userdata);
+  
   if (gsl_finite (V1) && gsl_finite(V2))
     return V1 + V2 + V3;
   else
@@ -497,6 +525,8 @@ nc_hicosmo_qg_lambda_x (NcmModel *model, gdouble x, gpointer userdata)
   gdouble sqrt_Omega_m = sqrt(OMEGA_M);
   gdouble one_m3omega = (1.0 - 3.0 * W);
 
+  NCM_UNUSED (userdata);
+  
   return 2.0 * asinh (pow(x, one_m3omega / 2.0) * sqrt_Omega_r / sqrt_Omega_m) / (one_m3omega * sqrt_Omega_r);
 }
 
@@ -563,6 +593,8 @@ nc_hicosmo_qg_alphaprime2 (NcmModel *model, gdouble alpha, gpointer data)
      OMEGA_X / x2 * 3.0 * gsl_sf_exprel (-3.0 * alpha2)
      );
 
+  NCM_UNUSED (data);
+  
   return alphaprime2;
 }
 
@@ -578,6 +610,9 @@ nc_hicosmo_qg_dalphaprime2_dalpha (NcmModel *model, gdouble alpha, gpointer data
   const gdouble onep3W = 1.0 + 3.0 * W;
   const gdouble three_1mW_2alpha2 = three_1mW_2 * alpha2;
   gdouble er, em, ex, dalphaprime2_dalpha;
+
+  NCM_UNUSED (data);
+  
   if (alpha2 > -GSL_LOG_DBL_EPSILON + 3.0 * M_LN10)
   {
     er = (2.0 / alpha2 + 2.0 / alpha4);
@@ -738,6 +773,9 @@ nc_hicosmo_qg_get_lambda_f (NcmModel *model, gpointer userdata)
 {
   NcHICosmoQG *qgint = (NcHICosmoQG *)model;
   nc_hicosmo_qg_init_spline (qgint, model);
+
+  NCM_UNUSED (userdata);
+  
   return qgint->last_lambda;
 }
 
@@ -746,6 +784,9 @@ nc_hicosmo_qg_get_lambda_i (NcmModel *model, gpointer userdata)
 {
   NcHICosmoQG *qgint = (NcHICosmoQG *) model;
   nc_hicosmo_qg_init_spline (qgint, model);
+
+  NCM_UNUSED (userdata);
+  
   return qgint->lambda_i;
 }
 
@@ -754,12 +795,15 @@ nc_hicosmo_qg_get_lambda_d (NcmModel *model, gpointer userdata)
 {
   NcHICosmoQG *qgint = (NcHICosmoQG *) model;
   nc_hicosmo_qg_init_spline (qgint, model);
+
+  NCM_UNUSED (userdata);
+  
   return qgint->lambda_d;
 }
 
 
 NcHICosmoQG *
-nc_hicosmo_qgint_new ()
+nc_hicosmo_qgint_new (void)
 {
   NcHICosmoQG *qgint = g_slice_new (NcHICosmoQG);
   qgint->cvode = CVodeCreate(CV_BDF, CV_NEWTON);
@@ -820,8 +864,8 @@ scalefactor_step (realtype t, N_Vector y, N_Vector ydot, gpointer f_data)
   const gdouble exp31mwgb_2g = exp (3.0 * (1.0 - W) * G_B - 2.0 * g);
   const gdouble exp6gb_2g = exp (6.0 * G_B - 2.0 * g);
 
-//  printf ("%.15g %.15g %.15g\n", t, g, 1.0 / expg);
-
+  NCM_UNUSED (t);
+  
   if (qgscale->first_order)
   {
     NV_Ith_S(ydot, 0) = qgscale->sign * sqrt(nc_hicosmo_qg_gbar2 (model, 1.0 / expg));
@@ -850,6 +894,13 @@ scalefactor_step_J (_NCM_SUNDIALS_INT_TYPE N, realtype t, N_Vector y, N_Vector f
   gdouble exp31mwgb_2g = exp (3.0 * (1.0 - W) * G_B - 2.0 * g);
   gdouble exp6gb_2g = exp (6.0 * G_B-2.0 * g);
 
+  NCM_UNUSED (N);
+  NCM_UNUSED (t);
+  NCM_UNUSED (fy);
+  NCM_UNUSED (tmp1);
+  NCM_UNUSED (tmp2);
+  NCM_UNUSED (tmp3);
+  
   if (qgscale->first_order)
   {
     gdouble x = 1.0 / expg;
@@ -877,6 +928,9 @@ scale_factor_root (gdouble t, N_Vector y, gdouble *gout, gpointer g_data)
   gout[0] = NV_Ith_S(y, 0) - qgscale->x_root_scale;
   gout[1] = NV_Ith_S(y, 1);
   gout[2] = NV_Ith_S(y, 0) - 10.0 * M_LN10;
+
+  NCM_UNUSED (t);
+  
   return 0;
 }
 
@@ -889,8 +943,11 @@ scale_factor_time (gdouble t, N_Vector y, N_Vector yQdot, gpointer fQ_data)
   gdouble x = exp(-NV_Ith_S(y, 0));
   gdouble zeta = nc_hicosmo_qg_zeta (model, x, NULL);
 
+  NCM_UNUSED (t);
+  
   NV_Ith_S(yQdot, 0) = 1.0 / x;
   NV_Ith_S(yQdot, 1) = 1.0 / (x * zeta * zeta);
+
   return 0;
 }
 
@@ -910,7 +967,7 @@ nc_hicosmo_qgint_init (NcHICosmoQG *qgint, gdouble lambda, gdouble g, gdouble gb
   if (!qgint->initialized)
   {
     flag = CVodeInit (qgint->cvode, &scalefactor_step, lambda, qgint->y);
-    CVODE_CHECK (&flag, "CVodeMalloc", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVodeMalloc", 1, FALSE);
     CVodeQuadInit (qgint->cvode, scale_factor_time, qgint->yQ);
     qgint->initialized = TRUE;
   }
@@ -918,7 +975,7 @@ nc_hicosmo_qgint_init (NcHICosmoQG *qgint, gdouble lambda, gdouble g, gdouble gb
   {
     flag = CVodeReInit (qgint->cvode, lambda, qgint->y);
     CVodeQuadReInit (qgint->cvode, qgint->yQ);
-    CVODE_CHECK(&flag, "CVodeReInit", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVodeReInit", 1, FALSE);
   }
 
   return TRUE;
@@ -930,37 +987,37 @@ nc_hicosmo_qgint_set_opts (NcHICosmoQG *qgint, NcHICosmoQGScaleFactor *qgscale)
   gint flag;
 
   flag = CVodeSStolerances (qgint->cvode, qgint->reltol, qgint->abstol);
-  CVODE_CHECK(&flag, "CVodeSStolerances", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeSStolerances", 1, FALSE);
 
   flag = CVodeSetMaxNumSteps(qgint->cvode, 1000000);
-  CVODE_CHECK(&flag, "CVodeSetMaxNumSteps", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeSetMaxNumSteps", 1, FALSE);
 
   flag = CVodeSetUserData (qgint->cvode, qgscale);
-  CVODE_CHECK(&flag, "CVodeSetFdata", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeSetFdata", 1, FALSE);
 
   if (TRUE)
   {
     flag = CVDense(qgint->cvode, 2);
-    CVODE_CHECK(&flag, "CVDense", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVDense", 1, FALSE);
 
     flag = CVDlsSetDenseJacFn (qgint->cvode, &scalefactor_step_J);
-    CVODE_CHECK(&flag, "CVDlsSetDenseJacFn", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVDlsSetDenseJacFn", 1, FALSE);
   }
 
 //  flag = CVodeSetStabLimDet(qgint->cvode, TRUE);
-//  CVODE_CHECK(&flag, "CVodeSetStabLimDet", 1, FALSE);
+//  NCM_CVODE_CHECK (&flag, "CVodeSetStabLimDet", 1, FALSE);
 
 //  flag = CVodeSetMaxStep(qgint->cvode, 1.0);
-//  CVODE_CHECK(&flag, "CVodeSetMaxStep", 1, FALSE);
+//  NCM_CVODE_CHECK (&flag, "CVodeSetMaxStep", 1, FALSE);
 
 //  flag = CVodeSetInitStep (qgint->cvode, 1e-15);
-//  CVODE_CHECK(&flag, "CVodeSetMaxStep", 1, FALSE);
+//  NCM_CVODE_CHECK (&flag, "CVodeSetMaxStep", 1, FALSE);
 
   flag = CVodeRootInit (qgint->cvode, 3, scale_factor_root);
-  CVODE_CHECK(&flag, "CVodeRootInit", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeRootInit", 1, FALSE);
 
 //  flag = CVodeSetStopTime(qgint->cvode, qgint->tf);
-//  CVODE_CHECK(&flag, "CVodeSetStopTime", 1, FALSE);
+//  NCM_CVODE_CHECK (&flag, "CVodeSetStopTime", 1, FALSE);
 
   return TRUE;
 }
@@ -977,7 +1034,7 @@ nc_hicosmo_qg_init_spline (NcHICosmoQG *qgint, NcmModel *model)
     gdouble gbar = -sqrt(gbar2);
     gdouble gbarbar = nc_hicosmo_qg_gbarbar (model, x_i);
     gdouble cs2, V, k_cross;
-    NcHICosmoQGScaleFactor qgscale = {model, TRUE, -1.0};
+    NcHICosmoQGScaleFactor qgscale = {model, TRUE, -1.0, 0.0};
     gboolean end_dust = FALSE;
     gdouble lambda_iQ;
 
@@ -1014,7 +1071,7 @@ nc_hicosmo_qg_init_spline (NcHICosmoQG *qgint, NcmModel *model)
       gdouble x, zeta, xdzeta_zeta, cs2zeta2_int_1_zeta2;
 
       flag = CVode (qgint->cvode, lambda_i * 1.001, qgint->y, &lambda_i, CV_ONE_STEP);
-      CVODE_CHECK (&flag, "CVode[nc_hicosmo_qg_init_spline]", 1, FALSE);
+      NCM_CVODE_CHECK (&flag, "CVode[nc_hicosmo_qg_init_spline]", 1, FALSE);
       CVodeGetQuad (qgint->cvode, &lambda_iQ, qgint->yQ);
 
       printf ("% 20.15g % 20.15g % 20.15g % 20.15g % 20.15g\n", lambda_i, NV_Ith_S (qgint->y, 0), NV_Ith_S (qgint->y, 1), NV_Ith_S (qgint->yQ, 0), exp(-NV_Ith_S (qgint->y, 0)));
@@ -1036,7 +1093,7 @@ nc_hicosmo_qg_init_spline (NcHICosmoQG *qgint, NcmModel *model)
       gdouble x, zeta, xdzeta_zeta, cs2zeta2_int_1_zeta2;
 
       flag = CVode (qgint->cvode, lambda_i * 1.001, qgint->y, &lambda_i, CV_ONE_STEP);
-      CVODE_CHECK (&flag, "CVode[nc_hicosmo_qg_init_spline]", 1, FALSE);
+      NCM_CVODE_CHECK (&flag, "CVode[nc_hicosmo_qg_init_spline]", 1, FALSE);
       CVodeGetQuad (qgint->cvode, &lambda_iQ, qgint->yQ);
 
       printf ("% 20.15g % 20.15g % 20.15g % 20.15g % 20.15g\n", lambda_i, NV_Ith_S (qgint->y, 0), NV_Ith_S (qgint->y, 1), NV_Ith_S (qgint->yQ, 0), exp(-NV_Ith_S (qgint->y, 0)));
@@ -1089,7 +1146,7 @@ nc_hicosmo_qg_init_spline (NcHICosmoQG *qgint, NcmModel *model)
       gdouble x, zeta, xdzeta_zeta, cs2zeta2_int_1_zeta2;
 
       flag = CVode (qgint->cvode, lambda_i * 1.001, qgint->y, &lambda_i, CV_NORMAL);
-      CVODE_CHECK (&flag, "CVode[nc_hicosmo_qg_init_spline]", 1, FALSE);
+      NCM_CVODE_CHECK (&flag, "CVode[nc_hicosmo_qg_init_spline]", 1, FALSE);
       CVodeGetQuad (qgint->cvode, &lambda_iQ, qgint->yQ);
       CVodeGetDky (qgint->cvode, lambda_i, 1, dstep);
 
@@ -1219,6 +1276,8 @@ nc_hicosmo_qg_max_z (NcmModel *model, gdouble *max, gdouble *trans)
 gdouble
 nc_hicosmo_qg_get_eta_b (NcmModel *model, gpointer userdata)
 {
+  NCM_UNUSED (userdata);
+
   return ncm_c_c ()  * ncm_c_Mpc () / (sqrt(OMEGA_R) * exp(-G_B) * MACRO_H0 * 1.0e3);
 }
 
@@ -1294,21 +1353,21 @@ nc_hicosmo_qg_pert_set_opts (NcHICosmoQGMode *qgmode)
   gint flag;
 
   flag = CVodeSStolerances (qgmode->cvode, qgmode->reltol, qgmode->abstol);
-  CVODE_CHECK(&flag, "CVodeSStolerances", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeSStolerances", 1, FALSE);
 
   flag = CVodeSetMaxNumSteps (qgmode->cvode, 1000000);
-  CVODE_CHECK(&flag, "CVodeSetMaxNumSteps", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeSetMaxNumSteps", 1, FALSE);
 
   flag = CVodeSetUserData (qgmode->cvode, qgmode);
-  CVODE_CHECK(&flag, "CVodeSetUserData", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeSetUserData", 1, FALSE);
 
   if (TRUE)
   {
     flag = CVDense (qgmode->cvode, _PERT_SYS_DIM);
-    CVODE_CHECK(&flag, "CVDense", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVDense", 1, FALSE);
 
     flag = CVDlsSetDenseJacFn (qgmode->cvode, qgmode->jac);
-    CVODE_CHECK (&flag, "CVDlsSetDenseJacFn", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVDlsSetDenseJacFn", 1, FALSE);
   }
 
   return TRUE;
@@ -1354,13 +1413,13 @@ nc_hicosmo_qg_pert_init (NcHICosmoQGMode *qgmode, gdouble k)
   if (!(*qgmode->initialized))
   {
     flag = CVodeInit (qgmode->cvode, qgmode->f, qgmode->t_i, qgmode->y);
-    CVODE_CHECK(&flag, "CVodeMalloc", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVodeMalloc", 1, FALSE);
     *qgmode->initialized = TRUE;
   }
   else
   {
     flag = CVodeReInit (qgmode->cvode, qgmode->t_i, qgmode->y);
-    CVODE_CHECK(&flag, "CVodeReInit", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVodeReInit", 1, FALSE);
   }
 
   nc_hicosmo_qg_pert_set_opts (qgmode);
@@ -1435,13 +1494,13 @@ nc_hicosmo_qg_pert_switch (NcHICosmoQGMode *qgmode, NcHICosmoQGPertType type, gd
   if (!(*qgmode->initialized))
   {
     flag = CVodeInit (qgmode->cvode, qgmode->f, qgmode->t, qgmode->y);
-    CVODE_CHECK(&flag, "CVodeMalloc", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVodeMalloc", 1, FALSE);
     *qgmode->initialized = TRUE;
   }
   else
   {
     flag = CVodeReInit (qgmode->cvode, qgmode->t, qgmode->y);
-    CVODE_CHECK(&flag, "CVodeReInit", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVodeReInit", 1, FALSE);
   }
 
   nc_hicosmo_qg_pert_set_opts (qgmode);
@@ -1466,13 +1525,13 @@ nc_hicosmo_qg_pert_evolve (NcHICosmoQGMode *qgmode)
   printf ("# Going to %.15e %.15e\n", exp(qgmode->t), xeq);
 
   flag = CVodeSetStopTime(qgmode->cvode, alpha_eq);
-  CVODE_CHECK(&flag, "CVodeSetStopTime", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeSetStopTime", 1, FALSE);
 
   last_alpha = qgmode->t;
   while (1)
   {
     flag = CVode (qgmode->cvode, alpha_eq, qgmode->y, &alpha, CV_ONE_STEP);
-    CVODE_CHECK(&flag, "CVode[nc_hicosmo_qg_pert_evolve]", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVode[nc_hicosmo_qg_pert_evolve]", 1, FALSE);
 
     if (TRUE)
     {
@@ -1504,12 +1563,12 @@ nc_hicosmo_qg_pert_evolve (NcHICosmoQGMode *qgmode)
   nc_hicosmo_qg_pert_switch (qgmode, NC_HICOSMO_QG_PERT_CURVATURE, lambda_eq);
 
   flag = CVodeSetStopTime(qgmode->cvode, qgint->last_lambda);
-  CVODE_CHECK(&flag, "CVodeSetStopTime", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeSetStopTime", 1, FALSE);
 
   while (1)
   {
     flag = CVode (qgmode->cvode, qgint->last_lambda, qgmode->y, &lambda, CV_ONE_STEP);
-    CVODE_CHECK(&flag, "CVode[nc_hicosmo_qg_pert_evolve]", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVode[nc_hicosmo_qg_pert_evolve]", 1, FALSE);
     if (TRUE)
     {
       x = nc_hicosmo_qg_x_lambda (qgmode->cp, lambda, FALSE);
@@ -1661,13 +1720,13 @@ nc_hicosmo_qg_modefuncm_cvode_init (NcHICosmoQGMode *qgmode)
   if (!*qgmode->initialized)
   {
     flag = CVodeInit(qgmode->cvode, f, qgmode->alphai, qgmode->y);
-    CVODE_CHECK(&flag, "CVodeMalloc", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVodeMalloc", 1, FALSE);
     *qgmode->initialized = TRUE;
   }
   else
   {
     flag = CVodeReInit (qgmode->cvode, qgmode->alphai, qgmode->y);
-    CVODE_CHECK(&flag, "CVodeReInit", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVodeReInit", 1, FALSE);
   }
 
   nc_hicosmo_qg_modefunc_set_opts (qgmode);
@@ -1721,42 +1780,42 @@ nc_hicosmo_qg_modefunc_set_opts (NcHICosmoQGMode *qgmode)
   gint flag;
 
   flag = CVodeSStolerances (qgmode->cvode, qgmode->reltol, qgmode->abstol);
-  CVODE_CHECK(&flag, "CVodeSStolerances", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeSStolerances", 1, FALSE);
 
   flag = CVodeSetMaxNumSteps(qgmode->cvode, 10000000);
-  CVODE_CHECK(&flag, "CVodeSetMaxNumSteps", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeSetMaxNumSteps", 1, FALSE);
 
   flag = CVodeSetUserData (qgmode->cvode, qgmode);
-  CVODE_CHECK(&flag, "CVodeSetUserData", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeSetUserData", 1, FALSE);
 
   if (FALSE)
   {
     flag = CVDense(qgmode->cvode, 4);
-    CVODE_CHECK(&flag, "CVDense", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVDense", 1, FALSE);
 
     if (OLD_CODE)
     {
       flag = CVDlsSetDenseJacFn (qgmode->cvode, &hk_step_J);
-      CVODE_CHECK(&flag, "CVDenseSetJacFn", 1, FALSE);
+      NCM_CVODE_CHECK (&flag, "CVDenseSetJacFn", 1, FALSE);
     }
     else
     {
       flag = CVDlsSetDenseJacFn (qgmode->cvode, &Rak_step_J);
-      CVODE_CHECK(&flag, "CVDenseSetJacFn", 1, FALSE);
+      NCM_CVODE_CHECK (&flag, "CVDenseSetJacFn", 1, FALSE);
     }
   }
 
 //  flag = CVodeSetStabLimDet(qgmode->cvode, TRUE);
-//  CVODE_CHECK(&flag, "CVodeSetStabLimDet", 1, FALSE);
+//  NCM_CVODE_CHECK (&flag, "CVodeSetStabLimDet", 1, FALSE);
 
 //  flag = CVodeSetMaxStep(qgmode->cvode, 10.0);
-//  CVODE_CHECK(&flag, "CVodeSetMaxStep", 1, FALSE);
+//  NCM_CVODE_CHECK (&flag, "CVodeSetMaxStep", 1, FALSE);
 
 //  flag = CVodeRootInit (qgmode->cvode, 2, scale_factor_root, params);
-//  CVODE_CHECK(&flag, "CVodeRootInit", 1, FALSE);
+//  NCM_CVODE_CHECK (&flag, "CVodeRootInit", 1, FALSE);
 
 //  flag = CVodeSetStopTime(qgmode->cvode, qgmode->tf);
-//  CVODE_CHECK(&flag, "CVodeSetStopTime", 1, FALSE);
+//  NCM_CVODE_CHECK (&flag, "CVodeSetStopTime", 1, FALSE);
 
   return TRUE;
 }
@@ -1835,14 +1894,14 @@ nc_hicosmo_qg_modefunc_evolve (NcHICosmoQGMode *qgmode)
   //qgmode->alphaf = 0.0;
   nc_hicosmo_qg_modefuncm_cvode_init (qgmode);
   flag = CVodeSetStopTime(qgmode->cvode, qgmode->alphaf);
-  CVODE_CHECK(&flag, "CVodeSetStopTime", 1, FALSE);
+  NCM_CVODE_CHECK (&flag, "CVodeSetStopTime", 1, FALSE);
   printf ("# Ode solver changed at %.15g.\n", exp(qgmode->alphai));
   while (1)
   {
     gdouble x_i;
 
     flag = CVode (qgmode->cvode, qgmode->alphaf, qgmode->y, &x_i, CV_ONE_STEP);
-    CVODE_CHECK(&flag, "CVode[nc_hicosmo_qg_modefunc_evolve]", 1, FALSE);
+    NCM_CVODE_CHECK (&flag, "CVode[nc_hicosmo_qg_modefunc_evolve]", 1, FALSE);
 
     if (FALSE && (x_i < 0 ? (x_i > (last_x * 0.9999)) : ((x_i > (last_x * 1.0001)))))
     {
@@ -1963,6 +2022,13 @@ hk_step_J (_NCM_SUNDIALS_INT_TYPE N, realtype alpha, N_Vector y, N_Vector fy, Dl
   long double x2cs2_xxbar2;// = nc_hicosmo_qg_cs2_xxbar2 (data, x);
   gdouble k2 = gsl_pow_2 (qgmode->k);
 
+  NCM_UNUSED (N);
+  NCM_UNUSED (y);
+  NCM_UNUSED (fy);
+  NCM_UNUSED (tmp1);
+  NCM_UNUSED (tmp2);
+  NCM_UNUSED (tmp3);
+
   nc_hicosmo_qg_evolfunc (model, x, &x2d2sqrtxxbarzeta_sqrtxxbarzeta, &x2cs2_xxbar2);
 
   DENSE_ELEM (J, 0, 0) = 0.0;
@@ -2028,6 +2094,13 @@ Rk_step_J (_NCM_SUNDIALS_INT_TYPE N, realtype lambda, N_Vector y, N_Vector fy, D
   gdouble cs2k2_x2 =  cs2k2 / x2;
   gdouble x2dzeta_zeta_p1gbar = (2.0 * x * dzeta_zeta + 1.0) * gbar;
 
+  NCM_UNUSED (N);
+  NCM_UNUSED (y);
+  NCM_UNUSED (fy);
+  NCM_UNUSED (tmp1);
+  NCM_UNUSED (tmp2);
+  NCM_UNUSED (tmp3);
+  
   DENSE_ELEM (J, 0, 0) = 0.0;
   DENSE_ELEM (J, 0, 1) = 1.0;
   DENSE_ELEM (J, 0, 2) = 0.0;
@@ -2089,6 +2162,13 @@ Rak_step_J (_NCM_SUNDIALS_INT_TYPE N, realtype alpha, N_Vector y, N_Vector fy, D
   gdouble cs2 = nc_hicosmo_qg_cs2 (model, x, NULL);
   gdouble k2 = qgmode->k * qgmode->k;
 
+  NCM_UNUSED (N);
+  NCM_UNUSED (y);
+  NCM_UNUSED (fy);
+  NCM_UNUSED (tmp1);
+  NCM_UNUSED (tmp2);
+  NCM_UNUSED (tmp3);
+  
   DENSE_ELEM (J, 0, 0) = 0.0;
   DENSE_ELEM (J, 0, 1) = 1.0;
   DENSE_ELEM (J, 0, 2) = 0.0;
