@@ -25,9 +25,9 @@
 
 /**
  * SECTION:nc_hicosmo_priors
- * @title: Homogeneus and Isotropic Models Priors
- * @short_description: Priors for HICosmo models
- *
+ * @title: NcHICosmoPrior
+ * @short_description: Collection of priors for NcHICosmo models.
+ * 
  * FIXME
  *
  */
@@ -51,6 +51,9 @@ _nc_hicosmo_prior_top_f (NcmMSet *mset, gpointer obj, const gdouble *x, gdouble
   gdouble mean = tp->mean;
   gdouble sigma = tp->sigma;
   gdouble cd = nc_distance_comoving (tp->dist, model, z);
+
+  NCM_UNUSED (x);
+  
   if (!gsl_finite (cd))
     f[0] = GSL_POSINF;
   else if (k <= 0)
@@ -130,6 +133,6 @@ nc_hicosmo_prior_top_add (NcmLikelihood *lh, gdouble z, gdouble alpha, gdouble s
   NcHICosmoPriorTop *tp = g_slice_new (NcHICosmoPriorTop);
   NcmMSetFunc *prior = ncm_mset_func_new (&_nc_hicosmo_prior_top_f, 0, 1, tp, (GDestroyNotify)nc_hicosmo_prior_top_free);
   nc_hicosmo_prior_top_set (tp, z, alpha, sigma_alpha, n);
-  ncm_likelihood_priors_add (lh, prior);
+  ncm_likelihood_priors_add (lh, prior, FALSE);
   ncm_mset_func_free (prior);
 }
