@@ -33,10 +33,14 @@
 
 /* Base types and components */
 #include <numcosmo/nc_hicosmo.h>
+#include <numcosmo/nc_hiprim.h>
 #include <numcosmo/nc_distance.h>
 #include <numcosmo/nc_hicosmo_priors.h>
 #include <numcosmo/nc_snia_dist_cov.h>
+#include <numcosmo/nc_planck_fi.h>
+#include <numcosmo/nc_planck_fi_cor_tt.h>
 #include <numcosmo/scalefactor.h>
+#include <numcosmo/nc_cbe_precision.h>
 /* Cosmic thermodynamics */
 #include <numcosmo/nc_recomb.h>
 #include <numcosmo/nc_recomb_seager.h>
@@ -52,9 +56,11 @@
 #include <numcosmo/perturbations/nc_hipert_two_fluids.h>
 #include <numcosmo/perturbations/nc_hipert_boltzmann.h>
 #include <numcosmo/perturbations/nc_hipert_boltzmann_std.h>
+#include <numcosmo/perturbations/nc_hipert_boltzmann_cbe.h>
 
 /* Model implementations */
 #include <numcosmo/model/nc_hicosmo_de.h>
+#include <numcosmo/model/nc_hicosmo_de_reparam_ok.h>
 #include <numcosmo/model/nc_hicosmo_de_linder.h>
 #include <numcosmo/model/nc_hicosmo_de_pad.h>
 #include <numcosmo/model/nc_hicosmo_de_qe.h>
@@ -64,6 +70,8 @@
 #include <numcosmo/model/nc_hicosmo_qlinear.h>
 #include <numcosmo/model/nc_hicosmo_qspline.h>
 #include <numcosmo/model/nc_hicosmo_qgrw.h>
+#include <numcosmo/model/nc_hiprim_power_law.h>
+#include <numcosmo/model/nc_hiprim_atan.h>
 #include <numcosmo/model/quantum_gravity.h>
 
 /* Large Scale Structure / Structure Formation */
@@ -108,6 +116,7 @@
 #include <numcosmo/lss/nc_cluster_mass_benson_xray.h>
 #include <numcosmo/lss/nc_cluster_mass_plcl.h>
 #include <numcosmo/lss/nc_cluster_abundance.h>
+#include <numcosmo/lss/nc_cluster_pseudo_counts.h>
 #include <numcosmo/lss/nc_cor_cluster_cmb_lens_limber.h>
 
 /* Observable data */
@@ -121,14 +130,24 @@
 #include <numcosmo/data/nc_data_bao_rdv.h>
 #include <numcosmo/data/nc_data_bao_dvdv.h>
 #include <numcosmo/data/nc_data_bao_empirical_fit.h>
+#include <numcosmo/data/nc_data_bao_dhr_dar.h>
 #include <numcosmo/data/nc_data_bao.h>
 #include <numcosmo/data/nc_data_cmb_dist_priors.h>
 #include <numcosmo/data/nc_data_cmb_shift_param.h>
 #include <numcosmo/data/nc_data_cmb.h>
 #include <numcosmo/data/nc_data_cluster_ncount.h>
 #include <numcosmo/data/nc_data_cluster_poisson.h>
+#include <numcosmo/data/nc_data_cluster_pseudo_counts.h>
+#include <numcosmo/data/nc_data_planck_lkl.h>
 
 /* ABC */
 #include <numcosmo/abc/nc_abc_cluster_ncount.h>
 
+/* Cross-correlations */
+#include <numcosmo/xcor/nc_xcor.h>
+#include <numcosmo/xcor/nc_xcor_limber.h>
+#include <numcosmo/xcor/nc_xcor_limber_gal.h>
+#include <numcosmo/xcor/nc_xcor_limber_lensing.h>
+#include <numcosmo/xcor/nc_data_xcor.h>
+
 #endif /* _NUMCOSMO_H */
