@@ -106,8 +106,9 @@ _ncm_data_gauss_cov_test_cov_func (NcmDataGaussCov *gauss, NcmMSet *mset, NcmMat
 
   {
     NcmMatrix *t1 = ncm_matrix_dup (gauss->cov);
-    gint ret = gsl_blas_dgemm (CblasNoTrans, CblasTrans, 1.0, NCM_MATRIX_GSL (t1), NCM_MATRIX_GSL (t1), 0.0, NCM_MATRIX_GSL (gauss->cov));
+    gint ret = gsl_blas_dgemm (CblasNoTrans, CblasTrans, 1.0, ncm_matrix_gsl (t1), ncm_matrix_gsl (t1), 0.0, ncm_matrix_gsl (gauss->cov));
     NCM_TEST_GSL_RESULT ("_ncm_data_gauss_cov_test_cov_func", ret);
+    ncm_matrix_copy_triangle (gauss->cov, 'U');
     ncm_matrix_free (t1);
   }
   
@@ -131,7 +132,7 @@ ncm_data_gauss_cov_test_gen_cov (NcmDataGaussCovTest *gcov_test)
 
   ncm_data_gauss_cov_test_mean_func (gauss, NULL, gauss->y);
   _ncm_data_gauss_cov_test_cov_func (gauss, NULL, NULL);
-  ncm_data_set_init (NCM_DATA (gauss));
+  ncm_data_set_init (NCM_DATA (gauss), TRUE);
 }
 
 NcmData *
