@@ -209,7 +209,8 @@ main (gint argc, gchar *argv[])
       de_data_simple.cmb_id == NULL &&
       de_data_simple.bao_id == NULL &&
       de_data_simple.H_id == NULL &&
-      de_data_simple.cluster_id == NULL)
+      de_data_simple.cluster_id == NULL &&
+      de_data_simple.data_files == NULL)
   {
     printf ("No action or data were chosen.\n");
     printf ("%s", g_option_context_get_help (context, TRUE, NULL));
@@ -394,6 +395,28 @@ main (gint argc, gchar *argv[])
 
   }
 
+  if (de_data_simple.data_files != NULL)
+  {
+    guint ndata_files = g_strv_length (de_data_simple.data_files);
+    guint i;
+    
+    for (i = 0; i < ndata_files; i++)
+    {
+      if (!g_file_test (de_data_simple.data_files[i], G_FILE_TEST_EXISTS))
+      {
+        g_warning ("data file: `%s' not found, skipping.", de_data_simple.data_files[i]);
+        continue;
+      }
+      else
+      {
+        NcmData *data = ncm_data_new_from_file (de_data_simple.data_files[i]);
+        ncm_dataset_append_data (dset, data);
+        ncm_data_free (data);        
+      }
+    }
+    
+  }
+  
   if (de_data_simple.BBN)
     nc_hicosmo_de_new_add_bbn (lh);
 
@@ -729,42 +752,51 @@ main (gint argc, gchar *argv[])
   {
     while (de_fit.onedim_cr[0] != NULL)
     {
-      gint p_n = atoi (de_fit.onedim_cr[0]);
-      gdouble prob_sigma, err_inf, err_sup;
+      const gchar *pname = de_fit.onedim_cr[0];
+      const NcmMSetPIndex *pi = ncm_mset_fparam_get_pi_by_name (mset, de_fit.onedim_cr[0]);
       de_fit.onedim_cr = &de_fit.onedim_cr[1];
-      switch (de_fit.nsigma)
+
+      if (pi == NULL)
       {
-        case 1:
-          prob_sigma = ncm_c_stats_1sigma ();
-          break;
-        case 2:
-          prob_sigma = ncm_c_stats_2sigma ();
-          break;
-        case 3:
-          prob_sigma = ncm_c_stats_3sigma ();
-          break;
-        default:
-          prob_sigma = ncm_c_stats_1sigma ();
-          break;
+        g_warning ("darkenergy: parameter `%s' not found.", pname);
+        continue;
       }
+      else
       {
-        NcmMSetPIndex pi = {nc_hicosmo_id (), p_n};
-        NcmLHRatio1d *lhr1d = ncm_lh_ratio1d_new (fit, &pi);
-        ncm_lh_ratio1d_find_bounds (lhr1d, fit->mtype, prob_sigma, &err_inf, &err_sup);
+        NcmLHRatio1d *lhr1d = ncm_lh_ratio1d_new (fit, pi);
+        gdouble prob_sigma, err_inf, err_sup;
+
+        switch (de_fit.nsigma)
+        {
+          case 1:
+            prob_sigma = ncm_c_stats_1sigma ();
+            break;
+          case 2:
+            prob_sigma = ncm_c_stats_2sigma ();
+            break;
+          case 3:
+            prob_sigma = ncm_c_stats_3sigma ();
+            break;
+          default:
+            prob_sigma = ncm_c_stats_1sigma ();
+            break;
+        }
+
+        ncm_lh_ratio1d_find_bounds (lhr1d, prob_sigma, fit->mtype, &err_inf, &err_sup);
         ncm_lh_ratio1d_free (lhr1d);
+
+        ncm_message ("#  One dimension confidence region for %s[%05d:%02d] = % .5g (% .5g, % .5g)\n",
+                     ncm_mset_param_name (mset, pi->mid, pi->pid), pi->mid, pi->pid,
+                     ncm_mset_param_get (mset, pi->mid, pi->pid), err_inf, err_sup);
       }
-      //ncm_fit_cr_1dim (fit, nc_hicosmo_id (), p_n, prob_sigma, 1, &err_inf, &err_sup);
-      ncm_message ("#  One dimension confidence region for %s[%02d] = % .5g (% .5g, % .5g)\n",
-                   ncm_model_param_name (NCM_MODEL (model), p_n), p_n,
-                   ncm_model_param_get (NCM_MODEL (model), p_n), err_inf, err_sup);
     }
   }
 
   if (de_fit.nsigma >= 0 && (de_fit.bidim_cr[0] != NULL) && (de_fit.bidim_cr[1] != NULL))
   {
     NcmLHRatio2d *lhr2d;
-    NcmMSetPIndex *pi1 = ncm_mset_fparam_get_pi_by_name (mset, de_fit.bidim_cr[0]);
-    NcmMSetPIndex *pi2 = ncm_mset_fparam_get_pi_by_name (mset, de_fit.bidim_cr[1]);
+    const NcmMSetPIndex *pi1 = ncm_mset_fparam_get_pi_by_name (mset, de_fit.bidim_cr[0]);
+    const NcmMSetPIndex *pi2 = ncm_mset_fparam_get_pi_by_name (mset, de_fit.bidim_cr[1]);
     NcmLHRatio2dRegion *rg_1sigma = NULL;
     NcmLHRatio2dRegion *rg_2sigma = NULL;
     NcmLHRatio2dRegion *rg_3sigma = NULL;
@@ -834,8 +866,8 @@ main (gint argc, gchar *argv[])
   if (de_fit.nsigma_fisher >= 0 && (de_fit.bidim_cr[0] != NULL) && (de_fit.bidim_cr[1] != NULL))
   {
     NcmLHRatio2d *lhr2d;
-    NcmMSetPIndex *pi1 = ncm_mset_fparam_get_pi_by_name (mset, de_fit.bidim_cr[0]);
-    NcmMSetPIndex *pi2 = ncm_mset_fparam_get_pi_by_name (mset, de_fit.bidim_cr[1]);
+    const NcmMSetPIndex *pi1 = ncm_mset_fparam_get_pi_by_name (mset, de_fit.bidim_cr[0]);
+    const NcmMSetPIndex *pi2 = ncm_mset_fparam_get_pi_by_name (mset, de_fit.bidim_cr[1]);
     NcmLHRatio2dRegion *rg_1sigma = NULL;
     NcmLHRatio2dRegion *rg_2sigma = NULL;
     NcmLHRatio2dRegion *rg_3sigma = NULL;
